@@ -30,6 +30,7 @@
  */
 
 #include <future>
+#include <vector>
 
 #include <unistd.h>
 
@@ -78,8 +79,8 @@ Result<describe_result, std::string>
 describe(const fs::path& filename)
 {
 #if HAVE_ARCHIVE_H
-    static constexpr auto RAW_FORMAT_NAME = string_fragment::from_const("raw");
-    static constexpr auto GZ_FILTER_NAME = string_fragment::from_const("gzip");
+    static constexpr auto RAW_FORMAT_NAME = "raw"_frag;
+    static constexpr auto GZ_FILTER_NAME = "gzip"_frag;
 
     auto_mem<archive> arc(archive_read_free);
 
@@ -119,14 +120,18 @@ describe(const fs::path& filename)
             };
 
             do {
-                ai.ai_entries.emplace_back(archive_info::entry{
-                    archive_entry_pathname_utf8(entry),
-                    archive_entry_strmode(entry),
-                    archive_entry_mtime(entry),
-                    archive_entry_size_is_set(entry)
-                        ? std::make_optional(archive_entry_size(entry))
-                        : std::nullopt,
-                });
+                const auto* entry_path = archive_entry_pathname_utf8(entry);
+
+                if (entry_path != nullptr) {
+                    ai.ai_entries.emplace_back(archive_info::entry{
+                        entry_path,
+                        archive_entry_strmode(entry),
+                        archive_entry_mtime(entry),
+                        archive_entry_size_is_set(entry)
+                            ? std::make_optional(archive_entry_size(entry))
+                            : std::nullopt,
+                    });
+                }
             } while (archive_read_next_header(arc, &entry) == ARCHIVE_OK);
 
             return Ok(describe_result{ai});
@@ -316,9 +321,14 @@ extract(const std::string& filename, const extract_cb& cb)
         const auto* format_name = archive_format_name(arc);
         auto filter_count = archive_filter_count(arc);
 
+        const auto* entry_path_str = archive_entry_pathname_utf8(entry);
+        if (entry_path_str == nullptr) {
+            continue;
+        }
+
         auto_mem<archive_entry> wentry(archive_entry_free);
         wentry = archive_entry_clone(entry);
-        auto desired_pathname = fs::path(archive_entry_pathname(entry));
+        auto desired_pathname = fs::path(entry_path_str).relative_path();
         if (strcmp(format_name, "raw") == 0 && filter_count >= 2) {
             desired_pathname = fs::path(filename).filename();
         }
@@ -396,41 +406,42 @@ walk_archive_files(
 #endif
 }
 
-void
+std::future<void>
 cleanup_cache()
 {
-    (void) std::async(std::launch::async, []() {
-        auto now = std::filesystem::file_time_type::clock::now();
-        auto cache_path = archive_cache_path();
-        const auto& cfg = injector::get<const config&>();
-        std::vector<fs::path> to_remove;
+    return std::async(
+        std::launch::async, +[]() {
+            auto now = std::filesystem::file_time_type::clock::now();
+            auto cache_path = archive_cache_path();
+            const auto& cfg = injector::get<const config&>();
+            std::vector<fs::path> to_remove;
 
-        log_debug("cache-ttl %d", cfg.amc_cache_ttl.count());
-        for (const auto& entry : fs::directory_iterator(cache_path)) {
-            if (entry.path().extension() != ".done") {
-                continue;
+            log_debug("cache-ttl %d", cfg.amc_cache_ttl.count());
+            for (const auto& entry : fs::directory_iterator(cache_path)) {
+                if (entry.path().extension() != ".done") {
+                    continue;
+                }
+
+                auto mtime = fs::last_write_time(entry.path());
+                auto exp_time = mtime + cfg.amc_cache_ttl;
+                if (now < exp_time) {
+                    continue;
+                }
+
+                to_remove.emplace_back(entry.path());
             }
 
-            auto mtime = fs::last_write_time(entry.path());
-            auto exp_time = mtime + cfg.amc_cache_ttl;
-            if (now < exp_time) {
-                continue;
+            for (auto& entry : to_remove) {
+                log_debug("removing cached archive: %s", entry.c_str());
+                fs::remove(entry);
+
+                entry.replace_extension(".lck");
+                fs::remove(entry);
+
+                entry.replace_extension();
+                fs::remove_all(entry);
             }
-
-            to_remove.emplace_back(entry.path());
-        }
-
-        for (auto& entry : to_remove) {
-            log_debug("removing cached archive: %s", entry.c_str());
-            fs::remove(entry);
-
-            entry.replace_extension(".lck");
-            fs::remove(entry);
-
-            entry.replace_extension();
-            fs::remove_all(entry);
-        }
-    });
+        });
 }
 
 }  // namespace archive_manager
