@@ -44,7 +44,7 @@ static const lnav::pcre2pp::code&
 ansi_regex()
 {
     static const auto retval = lnav::pcre2pp::code::from_const(
-        R"(\x1b\[([\d=;\?]*)([a-zA-Z])|\x1b\](\d+);(.*?)(?:\x07|\x1b\\)|(?:\X\x08\X)+|(\x16+))");
+        R"(\x00|\x1b\[([\d=;:\?]*)([a-zA-Z])|\x1b\](\d+);(.*?)(?:\x07|\x1b\\)|(?:\X\x08\X)+|(\x16+))");
 
     return retval;
 }
@@ -71,6 +71,14 @@ erase_ansi_escapes(string_fragment input)
         }
 
         auto sf = md[0].value();
+
+        if (sf == "\x00"_frag) {
+            *input.writable_data(0) = ' ';
+            move_start = sf.sf_end;
+            fill_index += 1;
+            continue;
+        }
+
         auto bs_index_res = sf.codepoint_to_byte_index(1);
 
         if (move_start) {
@@ -122,6 +130,8 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
 {
     thread_local auto md = lnav::pcre2pp::match_data::unitialized();
     static constexpr auto semi_pred = string_fragment::tag1{';'};
+    static constexpr auto colon_pred
+        = [](char ch) { return ch == ';' || ch == ':'; };
 
     const auto& regex = ansi_regex();
     std::optional<std::string> href;
@@ -131,8 +141,8 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
     size_t cp_start = std::string::npos;
     int last_origin_end = 0;
     int erased = 0;
+    size_t tmp_sa_open = 0;
 
-    std::replace(str.begin(), str.end(), '\0', ' ');
     auto matcher = regex.capture_from(str).into(md);
     while (true) {
         auto match_res = matcher.matches(PCRE2_NO_UTF_CHECK);
@@ -156,6 +166,13 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
             cp_dst = sf.sf_begin;
         }
 
+        if (sf == "\x00"_frag) {
+            str[cp_dst] = ' ';
+            cp_start = sf.sf_end;
+            cp_dst = sf.sf_end;
+            continue;
+        }
+
         if (sf.length() >= 3 && bs_index_res.isOk()
             && sf[bs_index_res.unwrap()] == '\b')
         {
@@ -263,7 +280,7 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
             continue;
         }
 
-        struct line_range lr;
+        line_range lr;
         text_attrs attrs;
         bool has_attrs = false;
         std::optional<role_t> role;
@@ -326,8 +343,8 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
                         }
                         if (ansi_code == 38 || ansi_code == 48) {
                             auto color_code_pair
-                                = seq.split_when(semi_pred).second.split_pair(
-                                    semi_pred);
+                                = seq.split_when(colon_pred)
+                                      .second.split_pair(colon_pred);
                             if (!color_code_pair) {
                                 break;
                             }
@@ -337,19 +354,24 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
                                 break;
                             }
                             if (color_type->value() == 2) {
-                                auto scan_res
-                                    = scn::scan<uint8_t, uint8_t, uint8_t>(
+                                auto scan_res = scn::
+                                    scan<uint8_t, char, uint8_t, char, uint8_t>(
                                         color_code_pair->second
                                             .to_string_view(),
-                                        "{};{};{}");
+                                        "{}{}{}{}{}");
                                 if (scan_res) {
-                                    auto [r, g, b] = scan_res->values();
-                                    attrs.ta_fg_color = rgb_color{r, g, b};
+                                    auto [r, sep1, g, sep2, b]
+                                        = scan_res->values();
+                                    if ((sep1 == ';' && sep2 == ';')
+                                        || (sep1 == ':' && sep2 == ':'))
+                                    {
+                                        attrs.ta_fg_color = rgb_color{r, g, b};
+                                    }
                                 }
                             } else if (color_type->value() == 5) {
                                 auto color_index_pair
                                     = color_code_pair->second.split_when(
-                                        semi_pred);
+                                        colon_pred);
                                 auto color_index = scn::scan_value<short>(
                                     color_index_pair.first.to_string_view());
                                 if (!color_index.has_value()
@@ -446,22 +468,30 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
                 shift_string_attrs(*sa, sf.sf_begin, -sf.length());
 
                 if (has_attrs) {
-                    for (auto rit = tmp_sa.rbegin(); rit != tmp_sa.rend();
-                         rit++)
+                    for (auto tmp_sa_curr = tmp_sa_open;
+                         tmp_sa_curr < tmp_sa.size();
+                         tmp_sa_curr++)
                     {
-                        if (rit->sa_range.lr_end != -1) {
+                        auto& sa = tmp_sa[tmp_sa_curr];
+                        if (sa.sa_range.lr_end != -1) {
                             continue;
                         }
-                        rit->sa_range.lr_end = cp_dst;
+                        sa.sa_range.lr_end = cp_dst;
+                        if (sa.sa_range.empty()) {
+                            sa.sa_type = &SA_INVALID;
+                        }
                     }
-                    lr.lr_start = cp_dst;
-                    lr.lr_end = -1;
-                    if (!attrs.empty()) {
-                        tmp_sa.emplace_back(lr, VC_STYLE.value(attrs));
+                    if (sf.sf_end < str.size()) {
+                        lr.lr_start = cp_dst;
+                        lr.lr_end = -1;
+                        tmp_sa_open = tmp_sa.size();
+                        if (!attrs.empty()) {
+                            tmp_sa.emplace_back(lr, VC_STYLE.value(attrs));
+                        }
+                        role | [&lr, &tmp_sa](role_t r) {
+                            tmp_sa.emplace_back(lr, VC_ROLE.value(r));
+                        };
                     }
-                    role | [&lr, &tmp_sa](role_t r) {
-                        tmp_sa.emplace_back(lr, VC_ROLE.value(r));
-                    };
                 }
                 if (cp_dst > 0) {
                     tmp_sa.emplace_back(
@@ -484,7 +514,9 @@ scrub_ansi_string(std::string& str, string_attrs_t* sa)
         cp_dst += cp_len;
         str.resize(cp_dst);
     }
-    if (sa != nullptr && last_origin_end > 0 && last_origin_end != str.size()) {
+    if (sa != nullptr && last_origin_end > 0
+        && last_origin_end != (ssize_t) str.size())
+    {
         tmp_sa.emplace_back(line_range{(int) last_origin_end, (int) str.size()},
                             SA_ORIGIN_OFFSET.value(erased));
     }
