@@ -31,9 +31,9 @@
 #include <cmath>
 
 #include "color_spaces.hh"
-#include "from_trait.hh"
 
 #include "config.h"
+#include "from_trait.hh"
 
 bool
 rgb_color::operator<(const rgb_color& rhs) const
@@ -86,7 +86,7 @@ rgb_color::from(const ansi_color color)
         case ansi_color::cyan:
             return rgb_color(0, 255, 255);
         case ansi_color::white:
-            return rgb_color(255, 255, 255);
+            return rgb_color(192, 192, 192);
         default:
             return rgb_color(0, 0, 0);
     }
@@ -126,6 +126,60 @@ lab_color::lab_color(const rgb_color& rgb)
     this->lc_b = 200.0 * (y - z);
 }
 
+rgb_color
+lab_color::to_rgb() const
+{
+    float var_Y = (this->lc_l + 16.) / 116.;
+    float var_X = this->lc_a / 500. + var_Y;
+    float var_Z = var_Y - this->lc_b / 200.;
+
+    if (pow(var_Y, 3) > 0.008856)
+        var_Y = pow(var_Y, 3);
+    else
+        var_Y = (var_Y - 16. / 116.) / 7.787;
+    if (pow(var_X, 3) > 0.008856)
+        var_X = pow(var_X, 3);
+    else
+        var_X = (var_X - 16. / 116.) / 7.787;
+    if (pow(var_Z, 3) > 0.008856)
+        var_Z = pow(var_Z, 3);
+    else
+        var_Z = (var_Z - 16. / 116.) / 7.787;
+
+    float X
+        = 95.047 * var_X;  // ref_X =  95.047     Observer= 2°, Illuminant= D65
+    float Y = 100.000 * var_Y;  // ref_Y = 100.000
+    float Z = 108.883 * var_Z;  // ref_Z = 108.883
+
+    var_X = X
+        / 100.;  // X from 0 to  95.047      (Observer = 2°, Illuminant = D65)
+    var_Y = Y / 100.;  // Y from 0 to 100.000
+    var_Z = Z / 100.;  // Z from 0 to 108.883
+
+    float var_R = var_X * 3.2406 + var_Y * -1.5372 + var_Z * -0.4986;
+    float var_G = var_X * -0.9689 + var_Y * 1.8758 + var_Z * 0.0415;
+    float var_B = var_X * 0.0557 + var_Y * -0.2040 + var_Z * 1.0570;
+
+    if (var_R > 0.0031308)
+        var_R = 1.055 * pow(var_R, (1 / 2.4)) - 0.055;
+    else
+        var_R = 12.92 * var_R;
+    if (var_G > 0.0031308)
+        var_G = 1.055 * pow(var_G, (1 / 2.4)) - 0.055;
+    else
+        var_G = 12.92 * var_G;
+    if (var_B > 0.0031308)
+        var_B = 1.055 * pow(var_B, (1 / 2.4)) - 0.055;
+    else
+        var_B = 12.92 * var_B;
+
+    return rgb_color{
+        (short) (var_R * 255.),
+        (short) (var_G * 255.),
+        (short) (var_B * 255.),
+    };
+}
+
 double
 lab_color::deltaE(const lab_color& other) const
 {
@@ -198,7 +252,8 @@ to_ansi_color(const rgb_color& color)
     double lowest_delta = std::numeric_limits<double>::max();
     for (uint8_t lpc = 0; lpc < term_colors.size(); ++lpc) {
         if (const auto this_delta = term_colors[lpc].deltaE(desired);
-            this_delta < lowest_delta) {
+            this_delta < lowest_delta)
+        {
             lowest_delta = this_delta;
             retval = ansi_color{lpc};
         }
@@ -222,10 +277,16 @@ lab_color::operator!=(const lab_color& rhs) const
 bool
 lab_color::sufficient_contrast(const lab_color& other) const
 {
-    if (std::abs(this->lc_l - other.lc_l) > 15) {
-        return true;
+    if (std::abs(this->lc_l - other.lc_l) < 45) {
+        return false;
     }
 
     return (std::signbit(this->lc_a) != std::signbit(other.lc_a)
             || std::signbit(this->lc_b) != std::signbit(other.lc_b));
 }
+
+namespace styling {
+
+const color_unit color_unit::EMPTY = color_unit{transparent{}};
+
+}  // namespace styling
