@@ -31,6 +31,7 @@
 #include <iterator>
 #include <regex>
 #include <sstream>
+#include <string_view>
 
 #include "string_util.hh"
 
@@ -39,18 +40,22 @@
 #include "lnav_log.hh"
 #include "scn/scan.h"
 
+using namespace std::string_view_literals;
+
 void
 scrub_to_utf8(char* buffer, size_t length)
 {
-    while (true) {
-        auto frag = string_fragment::from_bytes(buffer, length);
-        auto scan_res = is_utf8(frag);
-
-        if (scan_res.is_valid()) {
-            break;
-        }
-        for (size_t lpc = 0; lpc < scan_res.usr_faulty_bytes; lpc++) {
-            buffer[scan_res.usr_valid_frag.sf_end + lpc] = '?';
+    size_t index = 0;
+    while (index < length) {
+        auto start_index = index;
+        auto ch_res = ww898::utf::utf8::read([buffer, &index, length]() {
+            if (index < length) {
+                return buffer[index++];
+            }
+            return '\x00';
+        });
+        if (ch_res.isErr()) {
+            buffer[start_index] = '?';
         }
     }
 }
@@ -124,7 +129,9 @@ unquote_content(char* dst, const char* str, size_t len, char quote_char)
 size_t
 unquote(char* dst, const char* str, size_t len)
 {
-    if (str[0] == 'r' || str[0] == 'u') {
+    if (str[0] == 'f' || str[0] == 'r' || str[0] == 'u' || str[0] == 'R'
+        || str[0] == 'x' || str[0] == 'X')
+    {
         str += 1;
         len -= 1;
     }
@@ -208,13 +215,6 @@ utf8_char_to_byte_index(const std::string& str, ssize_t ch_index)
 
     return retval;
 }
-bool
-is_url(const std::string& fn)
-{
-    static const auto url_re = std::regex("^(file|https?|ftps?|scp|sftp):.*");
-
-    return std::regex_match(fn, url_re);
-}
 
 size_t
 last_word_str(char* str, size_t len, size_t max_len)
@@ -277,11 +277,20 @@ abbreviate_str(char* str, size_t len, size_t max_len)
 void
 split_ws(const std::string& str, std::vector<std::string>& toks_out)
 {
-    std::stringstream ss(str);
-    std::string buf;
+    auto str_sf = string_fragment::from_str(str);
 
-    while (ss >> buf) {
-        toks_out.push_back(buf);
+    while (true) {
+        auto split_pair = str_sf.split_when(isspace);
+        if (split_pair.first.empty()) {
+            if (split_pair.second.empty()) {
+                break;
+            }
+            str_sf = split_pair.second;
+            continue;
+        }
+
+        toks_out.emplace_back(split_pair.first.to_string());
+        str_sf = split_pair.second;
     }
 }
 
@@ -321,9 +330,9 @@ is_blank(const std::string& str)
 std::string
 scrub_ws(const char* in, ssize_t len)
 {
-    static const std::string TAB_SYMBOL = "\u21e5";
-    static const std::string LF_SYMBOL = "\u240a";
-    static const std::string CR_SYMBOL = "\u240d";
+    static constexpr auto TAB_SYMBOL = "\u21e5"sv;
+    static constexpr auto LF_SYMBOL = "\u240a"sv;
+    static constexpr auto CR_SYMBOL = "\u240d"sv;
 
     std::string retval;
 
@@ -331,7 +340,7 @@ scrub_ws(const char* in, ssize_t len)
         retval.reserve(len);
     }
 
-    for (size_t lpc = 0; (len == -1 && in[lpc]) || (len >= 0 && lpc < len);
+    for (ssize_t lpc = 0; (len == -1 && in[lpc]) || (len >= 0 && lpc < len);
          lpc++)
     {
         auto ch = in[lpc];
@@ -434,9 +443,9 @@ char_escape_seq(char ch)
             return "\\t";
         case '\n':
             return "\\n";
+        default:
+            return std::nullopt;
     }
-
-    return std::nullopt;
 }
 
 std::string
