@@ -31,10 +31,27 @@
 
 #include "base/string_util.hh"
 
+#include "base/fts_fuzzy_match.hh"
 #include "base/strnatcmp.h"
 #include "config.h"
 #include "doctest/doctest.h"
 
+TEST_CASE("fuzzy_match")
+{
+    {
+        // escape sequences should be ignored
+        const char* str = "com.example.foo";
+        const char* pattern1 = "c\\.e\\.f";
+        const char* pattern2 = "c.e.f";
+
+        int score1, score2;
+        CHECK(fts::fuzzy_match(pattern1, str, score1));
+        CHECK(fts::fuzzy_match(pattern2, str, score2));
+
+        CHECK(score1 == score2);
+    }
+}
+
 TEST_CASE("endswith")
 {
     std::string hw("hello");
@@ -43,6 +60,14 @@ TEST_CASE("endswith")
     CHECK(endswith(hw, "lo") == true);
 }
 
+TEST_CASE("scrub_to_utf8")
+{
+    char buffer[8]{};
+
+    scrub_to_utf8(buffer, sizeof(buffer));
+    CHECK(buffer[0] == '\x00');
+}
+
 TEST_CASE("truncate_to")
 {
     const std::string orig = "0123456789abcdefghijklmnopqrstuvwxyz";
