@@ -52,6 +52,7 @@ breadcrumb_curses::breadcrumb_curses()
     this->bc_match_view.set_height(0_vl);
     this->bc_match_view.set_show_scrollbar(true);
     this->bc_match_view.set_default_role(role_t::VCR_POPUP);
+    this->bc_match_view.set_head_space(0_vl);
     this->add_child_view(&this->bc_match_view);
 }
 
@@ -62,8 +63,12 @@ breadcrumb_curses::do_update()
         return false;
     }
 
+    if (!this->vc_needs_update) {
+        return view_curses::do_update();
+    }
+
     size_t sel_crumb_offset = 0;
-    auto width = ncplane_dim_x(this->bc_window);
+    auto width = ncplane_dim_x(this->vc_window);
     auto crumbs = this->bc_focused_crumbs.empty() ? this->bc_line_source()
                                                   : this->bc_focused_crumbs;
     if (this->bc_last_selected_crumb
@@ -97,16 +102,32 @@ breadcrumb_curses::do_update()
         }
 
         this->bc_displayed_crumbs.emplace_back(
-            line_range{(int) accum_width,
-                       (int) (accum_width + elem_width),
-                       line_range::unit::codepoint},
+            line_range{
+                (int) accum_width,
+                (int) (accum_width + elem_width),
+                line_range::unit::codepoint,
+            },
             crumb_index);
         crumbs_line.append(" \uff1a"_breadcrumb);
     }
 
+    if (!this->vc_enabled) {
+        for (auto& attr : crumbs_line.al_attrs) {
+            if (attr.sa_type != &VC_ROLE) {
+                continue;
+            }
+
+            auto role = attr.sa_value.get<role_t>();
+            if (role == role_t::VCR_STATUS_TITLE) {
+                attr.sa_value = role_t::VCR_STATUS_DISABLED_TITLE;
+            }
+        }
+    }
+
     line_range lr{0, static_cast<int>(width)};
-    mvwattrline(
-        this->bc_window, this->vc_y, 0, crumbs_line, lr, role_t::VCR_STATUS);
+    auto default_role = this->vc_enabled ? role_t::VCR_STATUS
+                                         : role_t::VCR_INACTIVE_STATUS;
+    mvwattrline(this->vc_window, this->vc_y, 0, crumbs_line, lr, default_role);
 
     if (this->bc_selected_crumb) {
         this->bc_match_view.set_x(sel_crumb_offset);
@@ -134,6 +155,9 @@ breadcrumb_curses::reload_data()
                                   this->bc_current_search,
                                   128)
         | lnav::itertools::sort_with(breadcrumb::possibility::sort_cmp);
+    for (auto& al : this->bc_similar_values) {
+        al.p_display_value.highlight_fuzzy_matches(this->bc_current_search);
+    }
     if (selected_crumb_ref.c_key.is<std::string>()
         && selected_crumb_ref.c_expected_input
             != breadcrumb::crumb::expected_input_t::anything)
@@ -221,6 +245,19 @@ breadcrumb_curses::blur()
     this->set_needs_update();
 }
 
+void
+breadcrumb_curses::focus_next()
+{
+    this->blur();
+    this->focus();
+    this->reload_data();
+    if (this->bc_selected_crumb.value() < this->bc_focused_crumbs.size() - 1) {
+        this->bc_selected_crumb = this->bc_selected_crumb.value() + 1;
+    }
+    this->bc_current_search.clear();
+    this->reload_data();
+}
+
 bool
 breadcrumb_curses::handle_key(const ncinput& ch)
 {
@@ -235,7 +272,6 @@ breadcrumb_curses::handle_key(const ncinput& ch)
             case 'A':
                 mapped_id = KEY_CTRL('a');
                 break;
-                break;
             case 'e':
             case 'E':
                 mapped_id = KEY_CTRL('e');
@@ -276,22 +312,12 @@ breadcrumb_curses::handle_key(const ncinput& ch)
         case NCKEY_TAB:
         case NCKEY_RIGHT:
             if (this->bc_selected_crumb) {
-                this->perform_selection(perform_behavior_t::if_different);
-                this->blur();
-                this->focus();
-                this->reload_data();
-                if (this->bc_selected_crumb.value()
-                    < this->bc_focused_crumbs.size() - 1)
+                if (!this->perform_selection(perform_behavior_t::if_different))
                 {
-                    this->bc_selected_crumb
-                        = this->bc_selected_crumb.value() + 1;
-                    retval = true;
+                    this->focus_next();
                 }
-                this->bc_current_search.clear();
-                this->reload_data();
-            } else {
-                retval = true;
             }
+            retval = true;
             break;
         case NCKEY_HOME:
             this->bc_match_view.set_selection(0_vl);
@@ -337,7 +363,7 @@ breadcrumb_curses::handle_key(const ncinput& ch)
         case KEY_ESCAPE:
             break;
         default:
-            if (isprint(ch.id)) {
+            if (ch.id < 0x7f && isprint(ch.id)) {
                 this->bc_current_search.push_back(ch.id);
                 this->reload_data();
                 retval = true;
@@ -352,34 +378,41 @@ breadcrumb_curses::handle_key(const ncinput& ch)
     return retval;
 }
 
-void
-breadcrumb_curses::perform_selection(
-    breadcrumb_curses::perform_behavior_t behavior)
+bool
+breadcrumb_curses::perform_selection(perform_behavior_t behavior)
 {
+    auto retval = false;
+
     if (!this->bc_selected_crumb) {
-        return;
+        return retval;
     }
 
     auto& selected_crumb_ref
         = this->bc_focused_crumbs[this->bc_selected_crumb.value()];
     auto match_sel = this->bc_match_view.get_selection();
-    if (match_sel >= 0
-        && match_sel < vis_line_t(this->bc_similar_values.size()))
+    if (match_sel.has_value()
+        && match_sel.value() < vis_line_t(this->bc_similar_values.size()))
     {
-        const auto& new_value = this->bc_similar_values[match_sel].p_key;
+        const auto& new_value
+            = this->bc_similar_values[match_sel.value()].p_key;
 
         switch (behavior) {
             case perform_behavior_t::if_different:
                 if (breadcrumb::crumb::key_t{new_value}
                     == selected_crumb_ref.c_key)
                 {
-                    return;
+                    return retval;
                 }
                 break;
             case perform_behavior_t::always:
                 break;
         }
-        selected_crumb_ref.c_performer(new_value);
+        if (this->bc_perform_handler) {
+            this->bc_perform_handler(
+                *this, selected_crumb_ref.c_performer, new_value);
+        }
+
+        retval = true;
     } else if (!this->bc_current_search.empty()) {
         switch (selected_crumb_ref.c_expected_input) {
             case breadcrumb::crumb::expected_input_t::exact:
@@ -395,10 +428,17 @@ breadcrumb_curses::perform_selection(
                 break;
             }
             case breadcrumb::crumb::expected_input_t::anything:
-                selected_crumb_ref.c_performer(this->bc_current_search);
+                if (this->bc_perform_handler) {
+                    this->bc_perform_handler(*this,
+                                             selected_crumb_ref.c_performer,
+                                             this->bc_current_search);
+                }
                 break;
         }
+        retval = true;
     }
+
+    return retval;
 }
 
 bool
