@@ -76,16 +76,32 @@ hier_node::child_index(const hier_node* hn) const
     return std::nullopt;
 }
 
+std::optional<section_key_t>
+hier_node::child_key(const hier_node* hn) const
+{
+    for (const auto& named_pair : this->hn_named_children) {
+        if (named_pair.second == hn) {
+            return named_pair.first;
+        }
+    }
+
+    auto index_opt = this->child_index(hn);
+    if (index_opt) {
+        return section_key_t{index_opt.value()};
+    }
+
+    return std::nullopt;
+}
+
 std::optional<hier_node::child_neighbors_result>
-hier_node::child_neighbors(const lnav::document::hier_node* hn,
-                           file_off_t offset) const
+hier_node::child_neighbors(const hier_node* hn, file_off_t offset) const
 {
     auto index_opt = this->child_index(hn);
     if (!index_opt) {
         return std::nullopt;
     }
 
-    hier_node::child_neighbors_result retval;
+    child_neighbors_result retval;
 
     if (index_opt.value() == 0) {
         if (this->hn_parent != nullptr) {
@@ -159,7 +175,7 @@ hier_node::line_neighbors(size_t ln) const
         return std::nullopt;
     }
 
-    hier_node::child_neighbors_result retval;
+    child_neighbors_result retval;
 
     for (const auto& child : this->hn_children) {
         if (child->hn_line_number > ln) {
@@ -232,6 +248,13 @@ discover_metadata_int(const attr_line_t& al, metadata_builder& mb)
     const auto& orig_attrs = al.get_attrs();
     auto headers = orig_attrs
         | lnav::itertools::filter_in([](const string_attr& attr) {
+                       if (!attr.sa_range.is_valid()) {
+                           return false;
+                       }
+
+                       if (attr.sa_type == &VC_ANCHOR) {
+                           return true;
+                       }
                        if (attr.sa_type != &VC_ROLE) {
                            return false;
                        }
@@ -275,17 +298,20 @@ discover_metadata_int(const attr_line_t& al, metadata_builder& mb)
     };
     std::vector<open_interval_t> open_intervals;
     auto root_node = std::make_unique<hier_node>();
+    const auto sf = string_fragment::from_str(al.get_string());
 
     for (const auto& hdr_attr : headers) {
-        const auto role = hdr_attr.sa_value.get<role_t>();
-        auto role_num = lnav::enums::to_underlying(role)
-            - lnav::enums::to_underlying(role_t::VCR_H1);
+        auto role_num = 0;
+        if (hdr_attr.sa_type == &VC_ROLE) {
+            const auto role = hdr_attr.sa_value.get<role_t>();
+            role_num = lnav::enums::to_underlying(role)
+                - lnav::enums::to_underlying(role_t::VCR_H1);
+        }
         std::vector<open_interval_t> new_open_intervals;
 
         for (auto& oi : open_intervals) {
             if (oi.oi_level >= role_num) {
                 // close out this section
-                auto sf = string_fragment::from_str(al.get_string());
                 auto left_sf = sf.find_left_boundary(
                     hdr_attr.sa_range.lr_start, string_fragment::tag1{'\n'});
                 if (left_sf.sf_begin > 0) {
@@ -306,17 +332,16 @@ discover_metadata_int(const attr_line_t& al, metadata_builder& mb)
                 new_open_intervals.emplace_back(std::move(oi));
             }
         }
-        if (!hdr_attr.sa_range.empty()) {
+        if (hdr_attr.sa_type == &VC_ANCHOR || !hdr_attr.sa_range.empty()) {
             auto* parent_node = new_open_intervals.empty()
                 ? root_node.get()
                 : new_open_intervals.back().oi_node.get();
-            auto sf = string_fragment::from_str(al.get_string());
             auto left_sf = sf.find_left_boundary(hdr_attr.sa_range.lr_start,
                                                  string_fragment::tag1{'\n'});
-            new_open_intervals.emplace_back(
-                role_num,
-                left_sf.sf_begin,
-                al.get_substring(hdr_attr.sa_range));
+            auto key = section_key_t{hdr_attr.sa_type == &VC_ANCHOR
+                                         ? hdr_attr.sa_value.get<std::string>()
+                                         : al.get_substring(hdr_attr.sa_range)};
+            new_open_intervals.emplace_back(role_num, left_sf.sf_begin, key);
             new_open_intervals.back().oi_node->hn_parent = parent_node;
             new_open_intervals.back().oi_node->hn_start = left_sf.sf_begin;
         }
@@ -403,7 +428,7 @@ discover_metadata(const attr_line_t& al)
 class structure_walker {
 public:
     explicit structure_walker(discover_builder& db)
-        : sw_discover_builder(db), sw_line(db.db_line), sw_range(db.db_range),
+        : sw_discover_builder(db), sw_line(db.db_line),
           sw_scanner(string_fragment::from_str_range(db.db_line.get_string(),
                                                      db.db_range.lr_start,
                                                      db.db_range.lr_end))
@@ -418,6 +443,7 @@ public:
             case text_format_t::TF_JSON:
             case text_format_t::TF_YAML:
             case text_format_t::TF_TOML:
+            case text_format_t::TF_INI:
             case text_format_t::TF_LOG:
             case text_format_t::TF_UNKNOWN:
                 return true;
@@ -432,6 +458,8 @@ public:
 
         mb.mb_text_format = this->sw_discover_builder.db_text_format;
         while (true) {
+            require(this->sw_depth == this->sw_container_tokens.size());
+
             auto tokenize_res = this->sw_scanner.tokenize2(
                 this->sw_discover_builder.db_text_format);
             if (!tokenize_res) {
@@ -463,8 +491,8 @@ public:
                         section_types_t::comment);
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start + el.e_capture.c_begin,
-                            this->sw_range.lr_start + el.e_capture.c_end,
+                            el.e_capture.c_begin,
+                            el.e_capture.c_end,
                         },
                         VC_ROLE.value(role_t::VCR_COMMENT));
                     break;
@@ -498,6 +526,9 @@ public:
                                 term = std::nullopt;
                             }
                             this->sw_interval_state.pop_back();
+                            if (!found && this->sw_depth > 0) {
+                                this->sw_depth -= 1;
+                            }
                             this->sw_hier_stage
                                 = std::move(this->sw_hier_nodes.back());
                             this->sw_hier_nodes.pop_back();
@@ -514,8 +545,8 @@ public:
                 case DT_H1: {
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start + inner_cap.c_begin,
-                            this->sw_range.lr_start + inner_cap.c_end,
+                            inner_cap.c_begin,
+                            inner_cap.c_end,
                         },
                         VC_ROLE.value(role_t::VCR_H1));
                     this->sw_line_number += 1;
@@ -538,24 +569,22 @@ public:
                     }
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start
-                                + tokenize_res->tr_capture.c_begin,
-                            this->sw_range.lr_start
-                                + tokenize_res->tr_capture.c_begin,
+                            tokenize_res->tr_capture.c_begin,
+                            tokenize_res->tr_capture.c_begin,
                         },
                         VC_ROLE.value(role_t::VCR_H1));
                     if (file1 == "/dev/null" || file1 == file2) {
                         this->sw_line.get_attrs().emplace_back(
                             line_range{
-                                this->sw_range.lr_start + file2.sf_begin,
-                                this->sw_range.lr_start + file2.sf_end,
+                                file2.sf_begin,
+                                file2.sf_end,
                             },
                             VC_ROLE.value(role_t::VCR_H1));
                     } else {
                         this->sw_line.get_attrs().emplace_back(
                             line_range{
-                                this->sw_range.lr_start + inner_cap.c_begin,
-                                this->sw_range.lr_start + inner_cap.c_end,
+                                inner_cap.c_begin,
+                                inner_cap.c_end,
                             },
                             VC_ROLE.value(role_t::VCR_H1));
                     }
@@ -566,16 +595,14 @@ public:
                     this->drop_open_children();
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start
-                                + tokenize_res->tr_capture.c_begin,
-                            this->sw_range.lr_start
-                                + tokenize_res->tr_capture.c_begin,
+                            tokenize_res->tr_capture.c_begin,
+                            tokenize_res->tr_capture.c_begin,
                         },
                         VC_ROLE.value(role_t::VCR_H2));
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start + inner_cap.c_begin,
-                            this->sw_range.lr_start + inner_cap.c_end,
+                            inner_cap.c_begin,
+                            inner_cap.c_end,
                         },
                         VC_ROLE.value(role_t::VCR_H2));
                     this->sw_line_number += 1;
@@ -700,10 +727,8 @@ public:
                                 section_types_t::multiline_string);
                             this->sw_line.get_attrs().emplace_back(
                                 line_range{
-                                    this->sw_range.lr_start
-                                        + el.e_capture.c_begin,
-                                    this->sw_range.lr_start
-                                        + el.e_capture.c_end,
+                                    el.e_capture.c_begin,
+                                    el.e_capture.c_end,
                                 },
                                 VC_ROLE.value(role_t::VCR_STRING));
                         }
@@ -719,6 +744,8 @@ public:
                     break;
                 }
             }
+
+            ensure(this->sw_depth == this->sw_container_tokens.size());
         }
         this->flush_values();
 
@@ -886,7 +913,6 @@ private:
 
     discover_builder& sw_discover_builder;
     attr_line_t& sw_line;
-    line_range sw_range;
     data_scanner sw_scanner;
     int sw_depth{0};
     size_t sw_line_number{0};
