@@ -37,6 +37,7 @@
 #include "log_format_ext.hh"
 #include "log_vtab_impl.hh"
 #include "md2attr_line.hh"
+#include "msg.text.hh"
 #include "ptimec.hh"
 #include "readline_highlighters.hh"
 #include "sql_util.hh"
@@ -64,8 +65,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         return;
     }
 
-    content_line_t cl = lss.at(row);
-    std::shared_ptr<logfile> file = lss.find(cl);
+    auto cl = lss.at(row);
+    auto file = lss.find(cl);
     auto ll = file->begin() + cl;
     auto format = file->get_format();
     bool display = false;
@@ -109,10 +110,10 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
     }
 
     char old_timestamp[64], curr_timestamp[64], orig_timestamp[64];
-    struct timeval curr_tv, offset_tv, orig_tv, diff_tv = {0, 0};
+    timeval curr_tv, offset_tv, orig_tv, diff_tv = {0, 0};
     attr_line_t time_line;
     auto& time_str = time_line.get_string();
-    struct line_range time_lr;
+    line_range time_lr;
     off_t ts_len = sql_strftime(
         curr_timestamp, sizeof(curr_timestamp), ll->get_timeval(), 'T');
     {
@@ -248,6 +249,16 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         return;
     }
 
+    auto anchor_opt = this->fos_lss.anchor_for_row(row);
+    if (anchor_opt) {
+        auto permalink
+            = attr_line_t(" Permalink: ")
+                  .append(lnav::roles::hyperlink(anchor_opt.value()));
+        this->fos_row_to_field_meta.emplace(
+            this->fos_lines.size(), row_info{std::nullopt, anchor_opt.value()});
+        this->fos_lines.emplace_back(permalink);
+    }
+
     this->fos_known_key_size = LOG_BODY.length();
     if (!this->fos_contexts.empty()) {
         this->fos_known_key_size += this->fos_contexts.top().c_prefix.length();
@@ -389,7 +400,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             this->fos_row_to_field_meta.emplace(
                 this->fos_lines.size(), row_info{std::nullopt, value_str});
         }
-        readline_sqlite_highlighter_int(al, std::nullopt, hl_range);
+        readline_sql_highlighter_int(
+            al, lnav::sql::dialect::sqlite, std::nullopt, hl_range);
 
         al.append(" = ").append(scrub_ws(value_str.c_str()));
 
@@ -425,7 +437,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
                             .append(qname.in())
                             .append(")")
                             .move();
-        readline_sqlite_highlighter(key_line, std::nullopt);
+        readline_sql_highlighter(
+            key_line, lnav::sql::dialect::sqlite, std::nullopt);
         auto key_size = key_line.length();
         key_line.append(" = ").append(scrub_ws(extra_pair.second));
         this->fos_row_to_field_meta.emplace(this->fos_lines.size(),
@@ -445,7 +458,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
                                 .append(this->fos_log_helper.format_json_getter(
                                     jpairs_map.first, lpc))
                                 .move();
-            readline_sqlite_highlighter(key_line, std::nullopt);
+            readline_sql_highlighter(
+                key_line, lnav::sql::dialect::sqlite, std::nullopt);
             auto key_size = key_line.length();
             key_line.append(" = ").append(scrub_ws(jpairs[lpc].wt_value));
             this->fos_row_to_field_meta.emplace(
@@ -468,7 +482,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             this->fos_log_helper.ldh_file->get_format()->get_name().c_str(),
             qname.in());
         auto key_line = attr_line_t("   ").append(xp_call.in()).move();
-        readline_sqlite_highlighter(key_line, std::nullopt);
+        readline_sql_highlighter(
+            key_line, lnav::sql::dialect::sqlite, std::nullopt);
         auto key_size = key_line.length();
         key_line.append(" = ").append(scrub_ws(xml_pair.second));
         this->fos_row_to_field_meta.emplace(
@@ -526,29 +541,48 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
     if (!this->fos_contexts.empty()
         && this->fos_contexts.top().c_show_applicable_annotations)
     {
+        if (this->fos_index_generation != this->fos_lss.lss_index_generation) {
+            this->fos_anno_cache.clear();
+            this->fos_index_generation = this->fos_lss.lss_index_generation;
+        }
+
         auto file_and_line = this->fos_lss.find_line_with_file(row);
 
         if (file_and_line && !file_and_line->second->is_continued()) {
-            auto applicable_anno = lnav::log::annotate::applicable(row);
-            if (!applicable_anno.empty()
-                && (!line_meta_opt
-                    || line_meta_opt.value()->bm_annotations.la_pairs.empty()))
-            {
-                auto anno_msg
-                    = attr_line_t(" ")
-                          .append(":memo:"_emoji)
-                          .append(" Annotations available, ")
-                          .append(lv.get_selection() == row
-                                      ? "use "
-                                      : "focus on this line and use ")
-                          .append(":annotate"_quoted_code)
-                          .append(" to apply them")
-                          .append(lv.get_selection() == row ? " to this line"
-                                                            : "")
-                          .with_attr_for_all(VC_ROLE.value(role_t::VCR_COMMENT))
-                          .move();
+            auto get_res = this->fos_anno_cache.get(row);
+            if (get_res) {
+                auto anno_val = get_res.value();
+                if (anno_val) {
+                    dst.emplace_back(anno_val.value());
+                }
+            } else {
+                auto applicable_anno = lnav::log::annotate::applicable(row);
+                if (!applicable_anno.empty()
+                    && (!line_meta_opt
+                        || line_meta_opt.value()
+                               ->bm_annotations.la_pairs.empty()))
+                {
+                    auto anno_msg
+                        = attr_line_t(" ")
+                              .append(":memo:"_emoji)
+                              .append(" Annotations available, ")
+                              .append(lv.get_selection() == row
+                                          ? "use "
+                                          : "focus on this line and use ")
+                              .append(":annotate"_quoted_code)
+                              .append(" to apply them")
+                              .append(lv.get_selection() == row
+                                          ? " to this line"
+                                          : "")
+                              .with_attr_for_all(
+                                  VC_ROLE.value(role_t::VCR_COMMENT))
+                              .move();
 
-                dst.emplace_back(anno_msg);
+                    this->fos_anno_cache.put(row, anno_msg);
+                    dst.emplace_back(anno_msg);
+                } else {
+                    this->fos_anno_cache.put(row, std::nullopt);
+                }
             }
         }
     }
@@ -588,6 +622,15 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
         md2attr_line mdal;
         attr_line_t al;
 
+        auto comment_id = intern_string::lookup(fmt::format(
+            FMT_STRING("{}-line{}-comment"),
+            file_and_line->first->get_filename().filename().string(),
+            std::distance(file_and_line->first->begin(),
+                          file_and_line->second)));
+        mdal.with_source_id(comment_id);
+        if (tc->tc_interactive) {
+            mdal.add_lnav_script_icons();
+        }
         auto parse_res = md4cpp::parse(line_meta.bm_comment, mdal);
         if (parse_res.isOk()) {
             al = parse_res.unwrap();
@@ -617,7 +660,8 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
                 auto hl_iter = hl.find({highlight_source_t::PREVIEW, "search"});
 
                 if (hl_iter != hl.end()) {
-                    hl_iter->second.annotate(comment_line, filename_width);
+                    hl_iter->second.annotate(comment_line,
+                                             line_range{(int) filename_width});
                 }
             }
 
@@ -633,21 +677,13 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
                                      VC_STYLE.value(vc.attrs_for_ident(str)));
         }
 
-        if (tc != nullptr) {
-            const auto& hm = tc->get_highlights();
-            auto hl_iter = hm.find({highlight_source_t::PREVIEW, "search"});
-
-            if (hl_iter != hm.end()) {
-                hl_iter->second.annotate(al, 2);
-            }
-        }
         al.insert(0, filename_width, ' ');
         if (tc != nullptr) {
             auto hl = tc->get_highlights();
             auto hl_iter = hl.find({highlight_source_t::PREVIEW, "search"});
 
             if (hl_iter != hl.end()) {
-                hl_iter->second.annotate(al, filename_width);
+                hl_iter->second.annotate(al, line_range{(int) filename_width});
             }
         }
         dst.emplace_back(al);
@@ -657,6 +693,7 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
             attr_line_t al;
             md2attr_line mdal;
 
+            mdal.add_lnav_script_icons();
             dst.push_back(
                 attr_line_t()
                     .append(filename_width, ' ')
@@ -695,7 +732,8 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
                         = hl.find({highlight_source_t::PREVIEW, "search"});
 
                     if (hl_iter != hl.end()) {
-                        hl_iter->second.annotate(anno_line, filename_width);
+                        hl_iter->second.annotate(
+                            anno_line, line_range{(int) filename_width});
                     }
                 }
 
@@ -725,6 +763,7 @@ field_overlay_source::list_value_for_overlay(
     vis_line_t row,
     std::vector<attr_line_t>& value_out)
 {
+    // log_debug("value for overlay %d", row);
     if (row == lv.get_selection()) {
         this->build_field_lines(lv, row);
         value_out = this->fos_lines;
@@ -732,6 +771,119 @@ field_overlay_source::list_value_for_overlay(
     this->build_meta_line(lv, value_out, row);
 }
 
+bool
+field_overlay_source::list_static_overlay(const listview_curses& lv,
+                                          int y,
+                                          int bottom,
+                                          attr_line_t& value_out)
+{
+    const std::vector<attr_line_t>* lines = nullptr;
+    if (this->fos_lss.text_line_count() == 0) {
+        if (this->fos_lss.is_indexing_in_progress()
+            || this->fos_lss.is_rebuild_forced())
+        {
+            auto msg = lnav::console::user_message::info(
+                "Log messages are being indexed...");
+            this->fos_static_lines = msg.to_attr_line().split_lines();
+            this->fos_static_lines_state.clear();
+            lines = &this->fos_static_lines;
+        } else if (this->fos_lss.file_count() > 0) {
+            hasher h;
+            this->fos_lss.update_filter_hash_state(h);
+            auto curr_state = h.to_array();
+            if (this->fos_static_lines.empty()
+                || curr_state != this->fos_static_lines_state)
+            {
+                auto msg = lnav::console::user_message::info(
+                    "All log messages are currently hidden");
+                auto hidden_file_count = size_t{0};
+                for (const auto& ld : this->fos_lss) {
+                    if (ld->get_file_ptr() == nullptr) {
+                        continue;
+                    }
+                    if (!ld->is_visible()) {
+                        hidden_file_count += 1;
+                    }
+                }
+                if (hidden_file_count > 0) {
+                    msg.with_note(attr_line_t()
+                                      .append(lnav::roles::number(
+                                          fmt::to_string(hidden_file_count)))
+                                      .append(" file(s) are hidden"));
+                }
+                auto min_time = this->fos_lss.get_min_row_time();
+                if (min_time) {
+                    msg.with_note(attr_line_t("Logs before ")
+                                      .append_quoted(lnav::to_rfc3339_string(
+                                          min_time.value()))
+                                      .append(" are not being shown"));
+                }
+                auto max_time = this->fos_lss.get_max_row_time();
+                if (max_time) {
+                    msg.with_note(attr_line_t("Logs after ")
+                                      .append_quoted(lnav::to_rfc3339_string(
+                                          max_time.value()))
+                                      .append(" are not being shown"));
+                }
+                if (this->fos_lss.get_min_log_level()
+                    > log_level_t::LEVEL_UNKNOWN)
+                {
+                    msg.with_note(
+                        attr_line_t("Logs with a level below ")
+                            .append_quoted(
+                                level_names[this->fos_lss.get_min_log_level()])
+                            .append(" are not being shown"));
+                }
+                auto& fs = this->fos_lss.get_filters();
+                for (const auto& filt : fs) {
+                    auto hits = this->fos_lss.get_filtered_count_for(
+                        filt->get_index());
+                    if (filt->get_type() == text_filter::EXCLUDE && hits == 0) {
+                        continue;
+                    }
+                    auto cmd = attr_line_t(":" + filt->to_command());
+                    readline_command_highlighter(cmd, std::nullopt);
+                    msg.with_note(
+                        attr_line_t("Filter ")
+                            .append_quoted(cmd)
+                            .append(" matched ")
+                            .append(lnav::roles::number(fmt::to_string(hits)))
+                            .append(" message(s) "));
+                }
+                if (this->fos_lss.get_marked_only()) {
+                    msg.with_note(attr_line_t("The ")
+                                      .append_quoted(lnav::roles::keyword(
+                                          ":hide-unmarked-lines"))
+                                      .append(" command was used and no "
+                                              "unfiltered lines are marked"));
+                }
+                this->fos_static_lines = msg.to_attr_line().split_lines();
+                this->fos_static_lines_state = curr_state;
+            }
+
+            lines = &this->fos_static_lines;
+        } else if (this->fos_tss.empty()) {
+            lines = lnav::messages::view::no_files();
+        } else {
+            lines = lnav::messages::view::only_text_files();
+        }
+    } else {
+        this->fos_static_lines.clear();
+    }
+
+    if (lines != nullptr && y < (ssize_t) lines->size()) {
+        value_out = lines->at(y);
+        value_out.with_attr_for_all(VC_ROLE.value(role_t::VCR_STATUS));
+        if (y == (ssize_t) lines->size() - 1) {
+            value_out.with_attr_for_all(
+                VC_STYLE.value(text_attrs::with_underline()));
+        }
+        return true;
+    }
+
+    return false;
+}
+
 std::optional<attr_line_t>
 field_overlay_source::list_header_for_overlay(const listview_curses& lv,
                                               vis_line_t vl)
