@@ -473,7 +473,12 @@ rcFilter(sqlite3_vtab_cursor* pVtabCursor,
 #endif
 
     log_debug("doing glob %s", pattern);
+#if defined(__MSYS__)
+    auto win_path = lnav::filesystem::escape_glob_for_win(pattern);
+    switch (glob(win_path.c_str(), glob_flags, nullptr, pCur->c_glob.inout())) {
+#else
     switch (glob(pattern, glob_flags, nullptr, pCur->c_glob.inout())) {
+#endif
         case GLOB_NOSPACE:
             pVtabCursor->pVtab->zErrMsg
                 = sqlite3_mprintf("No space to perform glob()");
@@ -507,13 +512,13 @@ register_fstat_vtab(sqlite3* db)
               .with_result({"st_ino", "The inode number"})
               .with_result(help_text{"st_type", "The type of the entry"}
                                .with_enum_values({
-                                   "reg",
-                                   "blk",
-                                   "chr",
-                                   "dir",
-                                   "fifo",
-                                   "lnk",
-                                   "sock",
+                                   "reg"_frag,
+                                   "blk"_frag,
+                                   "chr"_frag,
+                                   "dir"_frag,
+                                   "fifo"_frag,
+                                   "lnk"_frag,
+                                   "sock"_frag,
                                }))
               .with_result({"st_mode", "The protection mode"})
               .with_result(
