@@ -41,6 +41,7 @@
 #include "config.h"
 #include "field_overlay_source.hh"
 #include "lnav.hh"
+#include "lnav.prompt.hh"
 #include "lnav_config.hh"
 #include "shlex.hh"
 #include "sql_util.hh"
@@ -52,6 +53,8 @@ using namespace lnav::roles::literals;
 bool
 handle_keyseq(const char* keyseq)
 {
+    static auto& prompt = lnav::prompt::get();
+
     const auto& km = lnav_config.lc_active_keymap;
     const auto& iter = km.km_seq_to_cmd.find(keyseq);
     if (iter == km.km_seq_to_cmd.end()) {
@@ -80,7 +83,7 @@ handle_keyseq(const char* keyseq)
                               kc.kc_cmd.pp_value);
     auto result = execute_any(ec, kc.kc_cmd.pp_value);
     if (result.isOk()) {
-        lnav_data.ld_rl_view->set_value(result.unwrap());
+        prompt.p_editor.set_inactive_value(result.unwrap());
     } else {
         auto um = result.unwrapErr();
 
@@ -97,7 +100,7 @@ handle_keyseq(const char* keyseq)
                            &ec.ec_global_vars,
                        }))
         {
-            lnav_data.ld_rl_view->set_alt_value(expanded_msg);
+            prompt.p_editor.set_alt_value(expanded_msg);
         }
     }
 
@@ -107,6 +110,7 @@ handle_keyseq(const char* keyseq)
 void
 handle_paste_content(notcurses* nc, const ncinput& ch)
 {
+    static auto& prompt = lnav::prompt::get();
     auto& ec = lnav_data.ld_exec_context;
 
     switch (ch.paste_content[0]) {
@@ -120,11 +124,9 @@ handle_paste_content(notcurses* nc, const ncinput& ch)
 
             auto paste_sf = string_fragment::from_c_str(ch.paste_content);
             auto cmdline = lf_re.replace(paste_sf, "\n");
-            auto sg = ec.enter_source(SRC, 0, cmdline);
-
-            auto exec_res = ec.execute(cmdline);
+            auto exec_res = ec.execute(source_location{SRC, 1}, cmdline);
             if (exec_res.isOk()) {
-                lnav_data.ld_rl_view->set_value(exec_res.unwrap());
+                prompt.p_editor.set_inactive_value(exec_res.unwrap());
             } else {
                 auto um = exec_res.unwrapErr();
 
@@ -161,7 +163,10 @@ handle_paging_key(notcurses* nc, const ncinput& ch, const char* keyseq)
         return false;
     }
 
+    static auto& prompt = lnav::prompt::get();
+
     auto* tc = *lnav_data.ld_view_stack.top();
+    auto sel = tc->get_selection();
     auto& ec = lnav_data.ld_exec_context;
     auto* tc_tss = tc->get_sub_source();
     auto& bm = tc->get_bookmarks();
@@ -171,17 +176,22 @@ handle_paging_key(notcurses* nc, const ncinput& ch, const char* keyseq)
         return true;
     }
 
+    if (ch.id == NCKEY_EOF) {
+        lnav_data.ld_looping = false;
+        return true;
+    }
+
     if (tc->get_overlay_selection()) {
         if (tc->handle_key(ch)) {
             return true;
         }
     }
 
-    if (handle_keyseq(keyseq)) {
+    if (tc->handle_key(ch)) {
         return true;
     }
 
-    if (tc->handle_key(ch)) {
+    if (handle_keyseq(keyseq)) {
         return true;
     }
 
@@ -217,8 +227,8 @@ handle_paging_key(notcurses* nc, const ncinput& ch, const char* keyseq)
                     top_tc->get_sub_source());
 
                 lnav_data.ld_last_view = nullptr;
-                if (src_view != nullptr && dst_view != nullptr) {
-                    src_view->time_for_row(top_tc->get_selection()) |
+                if (src_view != nullptr && dst_view != nullptr && sel) {
+                    src_view->time_for_row(sel.value()) |
                         [dst_view, tc](auto top_ri) {
                             dst_view->row_for_time(top_ri.ri_time) |
                                 [tc](auto row) { tc->set_selection(row); };
@@ -248,7 +258,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                 clear_note.unwrap().execute();
             }
             auto um = lnav::console::user_message::ok(al);
-            lnav_data.ld_rl_view->set_attr_value(um.to_attr_line());
+            prompt.p_editor.set_inactive_value(um.to_attr_line());
             break;
         }
 
@@ -262,7 +272,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
             tc->get_bookmarks()[&textview_curses::BM_USER].clear();
             tc->reload_data();
 
-            lnav_data.ld_rl_view->set_attr_value(
+            prompt.p_editor.set_inactive_value(
                 lnav::console::user_message::ok("Cleared bookmarks")
                     .to_attr_line());
             break;
@@ -272,7 +282,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                 tc->get_top(), tc->get_bottom(), tc->get_left());
             if (range_opt && range_opt.value().second != INT_MAX) {
                 tc->set_left(range_opt.value().second);
-                lnav_data.ld_rl_view->set_alt_value(
+                prompt.p_editor.set_alt_value(
                     HELP_MSG_1(m, "to bookmark a line"));
             } else {
                 alerter::singleton().chime("no more search hits to the right");
@@ -291,15 +301,16 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                 } else {
                     tc->set_left(0);
                 }
-                lnav_data.ld_rl_view->set_alt_value(
+                prompt.p_editor.set_alt_value(
                     HELP_MSG_1(m, "to bookmark a line"));
             }
             break;
 
         case 'f':
             if (tc == &lnav_data.ld_views[LNV_LOG]) {
-                bm[&logfile_sub_source::BM_FILES].next(tc->get_selection()) |
-                    [&tc](auto vl) { tc->set_selection(vl); };
+                bm[&logfile_sub_source::BM_FILES].next(
+                    tc->get_selection().value_or(0_vl))
+                    | [&tc](auto vl) { tc->set_selection(vl); };
             } else if (tc == &lnav_data.ld_views[LNV_TEXT]) {
                 textfile_sub_source& tss = lnav_data.ld_text_source;
 
@@ -312,13 +323,14 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
 
         case 'F':
             if (tc == &lnav_data.ld_views[LNV_LOG]) {
-                bm[&logfile_sub_source::BM_FILES].prev(tc->get_selection()) |
-                    [&tc](auto vl) {
-                        // setting the selection for movement to previous file
-                        // marker instead of the top will move the cursor, too,
-                        // if needed.
-                        tc->set_selection(vl);
-                    };
+                bm[&logfile_sub_source::BM_FILES].prev(
+                    tc->get_selection().value_or(0_vl))
+                    | [&tc](auto vl) {
+                          // setting the selection for movement to previous file
+                          // marker instead of the top will move the cursor,
+                          // too, if needed.
+                          tc->set_selection(vl);
+                      };
             } else if (tc == &lnav_data.ld_views[LNV_TEXT]) {
                 textfile_sub_source& tss = lnav_data.ld_text_source;
 
@@ -333,10 +345,10 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
             if ((lnav_data.ld_zoom_level - 1) < 0) {
                 alerter::singleton().chime("maximum zoom-in level reached");
             } else {
-                execute_command(
-                    ec,
-                    "zoom-to "
-                        + lnav_zoom_strings[lnav_data.ld_zoom_level - 1]);
+                ec.execute(INTERNAL_SRC_LOC,
+                           fmt::format(
+                               FMT_STRING(":zoom-to {}"),
+                               lnav_zoom_strings[lnav_data.ld_zoom_level - 1]));
             }
             break;
 
@@ -344,22 +356,22 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
             if ((lnav_data.ld_zoom_level + 1) >= ZOOM_COUNT) {
                 alerter::singleton().chime("maximum zoom-out level reached");
             } else {
-                execute_command(
-                    ec,
-                    "zoom-to "
-                        + lnav_zoom_strings[lnav_data.ld_zoom_level + 1]);
+                ec.execute(INTERNAL_SRC_LOC,
+                           fmt::format(
+                               FMT_STRING(":zoom-to {}"),
+                               lnav_zoom_strings[lnav_data.ld_zoom_level + 1]));
             }
             break;
 
         case 'J':
             if (tc->is_selectable()) {
-                if (tc->get_selection() >= 0_vl) {
+                if (sel) {
                     tc->toggle_user_mark(&textview_curses::BM_USER,
-                                         tc->get_selection());
-                    lnav_data.ld_select_start[tc] = tc->get_selection();
-                    lnav_data.ld_last_user_mark[tc] = tc->get_selection();
-                    if (tc->get_selection() + 1_vl < tc->get_inner_height()) {
-                        tc->set_selection(tc->get_selection() + 1_vl);
+                                         sel.value());
+                    lnav_data.ld_select_start[tc] = sel.value();
+                    lnav_data.ld_last_user_mark[tc] = sel.value();
+                    if (sel.value() + 1_vl < tc->get_inner_height()) {
+                        tc->set_selection(sel.value() + 1_vl);
                     }
                 }
             } else {
@@ -368,16 +380,15 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                     || !tc->is_line_visible(
                         vis_line_t(lnav_data.ld_last_user_mark[tc])))
                 {
-                    lnav_data.ld_select_start[tc] = tc->get_selection();
-                    lnav_data.ld_last_user_mark[tc] = tc->get_selection();
+                    lnav_data.ld_select_start[tc] = tc->get_top();
+                    lnav_data.ld_last_user_mark[tc] = tc->get_top();
                 } else {
                     vis_line_t height;
                     unsigned long width;
 
                     tc->get_dimensions(height, width);
                     if (lnav_data.ld_last_user_mark[tc] > (tc->get_bottom() - 2)
-                        && tc->get_selection() + height
-                            < tc->get_inner_height())
+                        && tc->get_top() + height < tc->get_inner_height())
                     {
                         tc->shift_top(1_vl);
                     }
@@ -394,54 +405,61 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
             }
             tc->reload_data();
 
-            lnav_data.ld_rl_view->set_alt_value(
+            prompt.p_editor.set_alt_value(
                 HELP_MSG_1(c, "to copy marked lines to the clipboard"));
             break;
 
         case 'K': {
-            int new_mark;
-
-            if (lnav_data.ld_last_user_mark.find(tc)
-                    == lnav_data.ld_last_user_mark.end()
-                || !tc->is_line_visible(
-                    vis_line_t(lnav_data.ld_last_user_mark[tc])))
-            {
-                new_mark = tc->get_selection();
+            if (tc->is_selectable()) {
+                tc->toggle_user_mark(&textview_curses::BM_USER,
+                                     tc->get_selection().value_or(0_vl));
+                tc->shift_selection(listview_curses::shift_amount_t::up_line);
             } else {
-                new_mark = lnav_data.ld_last_user_mark[tc];
-            }
+                int new_mark;
 
-            tc->toggle_user_mark(&textview_curses::BM_USER,
-                                 vis_line_t(new_mark));
-            if (new_mark == tc->get_selection() && tc->get_top() > 0_vl) {
-                tc->shift_top(-1_vl);
-            }
-            if (new_mark > 0) {
-                lnav_data.ld_last_user_mark[tc] = new_mark - 1;
-            } else {
-                lnav_data.ld_last_user_mark[tc] = new_mark;
-                alerter::singleton().chime("no more lines to mark");
-            }
-            lnav_data.ld_select_start[tc] = tc->get_selection();
-            if (tc->is_selectable() && tc->get_selection() > 0_vl) {
-                tc->set_selection(tc->get_selection() - 1_vl);
+                if (lnav_data.ld_last_user_mark.find(tc)
+                        == lnav_data.ld_last_user_mark.end()
+                    || !tc->is_line_visible(
+                        vis_line_t(lnav_data.ld_last_user_mark[tc])))
+                {
+                    new_mark = tc->get_top();
+                } else {
+                    new_mark = lnav_data.ld_last_user_mark[tc];
+                }
+
+                tc->toggle_user_mark(&textview_curses::BM_USER,
+                                     vis_line_t(new_mark));
+                if (new_mark == tc->get_top() && tc->get_top() > 0_vl) {
+                    tc->shift_top(-1_vl);
+                }
+                if (new_mark > 0) {
+                    lnav_data.ld_last_user_mark[tc] = new_mark - 1;
+                } else {
+                    lnav_data.ld_last_user_mark[tc] = new_mark;
+                    alerter::singleton().chime("no more lines to mark");
+                }
+                lnav_data.ld_select_start[tc] = tc->get_top();
+                if (tc->is_selectable() && tc->get_top() > 0_vl) {
+                    tc->set_selection(tc->get_top() - 1_vl);
+                }
             }
             tc->reload_data();
 
-            lnav_data.ld_rl_view->set_alt_value(
+            prompt.p_editor.set_alt_value(
                 HELP_MSG_1(c, "to copy marked lines to the clipboard"));
             break;
         }
 
-        case 'M':
-            if (lnav_data.ld_last_user_mark.find(tc)
-                == lnav_data.ld_last_user_mark.end())
+        case 'M': {
+            if (!sel) {
+            } else if (lnav_data.ld_last_user_mark.find(tc)
+                       == lnav_data.ld_last_user_mark.end())
             {
                 alerter::singleton().chime("no lines have been marked");
             } else {
-                int start_line = std::min((int) tc->get_selection(),
+                int start_line = std::min((int) tc->get_selection().value(),
                                           lnav_data.ld_last_user_mark[tc] + 1);
-                int end_line = std::max((int) tc->get_selection(),
+                int end_line = std::max((int) tc->get_selection().value(),
                                         lnav_data.ld_last_user_mark[tc] - 1);
 
                 tc->toggle_user_mark(&textview_curses::BM_USER,
@@ -450,6 +468,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                 tc->reload_data();
             }
             break;
+        }
 
 #if 0
             case 'S':
@@ -469,15 +488,16 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
 #endif
 
         case 's':
-            if (text_accel_p && text_accel_p->is_time_offset_supported()) {
-                auto next_top = tc->get_selection() + 1_vl;
+            if (sel && text_accel_p && text_accel_p->is_time_offset_supported())
+            {
+                auto next_top = sel.value() + 1_vl;
 
                 if (!tc->is_selectable()) {
                     next_top += 1_vl;
                 }
 
                 if (!text_accel_p->is_time_offset_enabled()) {
-                    lnav_data.ld_rl_view->set_alt_value(
+                    prompt.p_editor.set_alt_value(
                         HELP_MSG_1(T, "to disable elapsed-time mode"));
                 }
                 text_accel_p->set_time_offset(true);
@@ -502,13 +522,13 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
 
         case 'S':
             if (text_accel_p && text_accel_p->is_time_offset_supported()) {
-                auto next_top = tc->get_selection();
+                auto next_top = tc->get_selection().value_or(0_vl);
 
                 if (tc->is_selectable() && next_top > 0_vl) {
                     next_top -= 1_vl;
                 }
                 if (!text_accel_p->is_time_offset_enabled()) {
-                    lnav_data.ld_rl_view->set_alt_value(
+                    prompt.p_editor.set_alt_value(
                         HELP_MSG_1(T, "to disable elapsed-time mode"));
                 }
                 text_accel_p->set_time_offset(true);
@@ -550,7 +570,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
         case '0':
             if (lss) {
                 const int step = 24 * 60 * 60;
-                lss->time_for_row(tc->get_selection()) |
+                lss->time_for_row(tc->get_selection().value_or(0_vl)) |
                     [lss, tc](auto first_ri) {
                         lss->find_from_time(
                             roundup_size(first_ri.ri_time.tv_sec, step))
@@ -561,7 +581,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
 
         case ')':
             if (lss) {
-                lss->time_for_row(tc->get_selection()) |
+                lss->time_for_row(tc->get_selection().value_or(0_vl)) |
                     [lss, tc](auto first_ri) {
                         time_t day
                             = rounddown(first_ri.ri_time.tv_sec, 24 * 60 * 60);
@@ -580,7 +600,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                 alerter::singleton().chime(
                     "the top of the log has been reached");
             } else if (lss) {
-                lss->time_for_row(tc->get_selection()) |
+                lss->time_for_row(tc->get_selection().value_or(0_vl)) |
                     [lss, ch, tc](auto first_ri) {
                         int step = ch.id == 'D' ? (24 * 60 * 60) : (60 * 60);
                         time_t top_time = first_ri.ri_time.tv_sec;
@@ -592,34 +612,34 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                         };
                     };
 
-                lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(/, "to search"));
+                prompt.p_editor.set_alt_value(HELP_MSG_1(/, "to search"));
             }
             break;
 
         case 'd':
             if (lss) {
-                lss->time_for_row(tc->get_selection()) |
+                lss->time_for_row(tc->get_selection().value_or(0_vl)) |
                     [ch, lss, tc](auto first_ri) {
                         int step = ch.id == 'd' ? (24 * 60 * 60) : (60 * 60);
                         lss->find_from_time(first_ri.ri_time.tv_sec + step) |
                             [tc](auto line) { tc->set_selection(line); };
                     };
 
-                lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(/, "to search"));
+                prompt.p_editor.set_alt_value(HELP_MSG_1(/, "to search"));
             }
             break;
 
         case 'o':
         case 'O':
-            if (lss != nullptr && lss->text_line_count() > 0) {
-                auto start_win = lss->window_at(tc->get_selection());
+            if (sel && lss != nullptr && lss->text_line_count() > 0) {
+                auto start_win = lss->window_at(sel.value());
                 auto start_win_iter = start_win.begin();
                 const auto& opid_opt
                     = start_win_iter->get_values().lvv_opid_value;
                 if (!opid_opt) {
                     alerter::singleton().chime(
                         "Log message does not contain an opid");
-                    lnav_data.ld_rl_view->set_attr_value(
+                    prompt.p_editor.set_inactive_value(
                         lnav::console::user_message::error(
                             "Log message does not contain an opid")
                             .to_attr_line());
@@ -658,10 +678,10 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                         break;
                     }
                     if (found) {
-                        lnav_data.ld_rl_view->set_value("");
+                        prompt.p_editor.clear_inactive_value();
                         tc->set_selection(next_win_iter->get_vis_line());
                     } else {
-                        lnav_data.ld_rl_view->set_attr_value(
+                        prompt.p_editor.set_inactive_value(
                             lnav::console::user_message::error(
                                 attr_line_t(
                                     "No more messages found with opid: ")
@@ -676,13 +696,22 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
             break;
 
         case 't':
-            if (lnav_data.ld_text_source.current_file() == nullptr) {
+            if (lnav_data.ld_view_stack.size() == 2
+                && lnav_data.ld_log_source.file_count() == 0
+                && lnav_data.ld_view_stack.top().value_or(nullptr)
+                    == &lnav_data.ld_views[LNV_TEXT])
+            {
+                prompt.p_editor.set_inactive_value(
+                    lnav::console::user_message::info(
+                        "Already in the TEXT view")
+                        .to_attr_line());
+            } else if (lnav_data.ld_text_source.current_file() == nullptr) {
                 alerter::singleton().chime("No text files loaded");
-                lnav_data.ld_rl_view->set_attr_value(
+                prompt.p_editor.set_inactive_value(
                     lnav::console::user_message::error("No text files loaded")
                         .to_attr_line());
             } else if (toggle_view(&lnav_data.ld_views[LNV_TEXT])) {
-                lnav_data.ld_rl_view->set_alt_value(
+                prompt.p_editor.set_alt_value(
                     HELP_MSG_2(f, F, "to switch to the next/previous file"));
             }
             break;
@@ -695,7 +724,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                     = dynamic_cast<text_time_translator*>(tc->get_sub_source());
 
                 if (src_view != nullptr) {
-                    src_view->time_for_row(tc->get_selection()) |
+                    src_view->time_for_row(tc->get_selection().value_or(0_vl)) |
                         [](auto log_top_ri) {
                             lnav_data.ld_hist_source2.row_for_time(
                                 log_top_ri.ri_time)
@@ -712,10 +741,10 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
 
                     if (dst_view != nullptr) {
                         auto& hs = lnav_data.ld_hist_source2;
-                        auto hist_top_time_opt
-                            = hs.time_for_row(hist_tc.get_selection());
-                        auto curr_top_time_opt
-                            = dst_view->time_for_row(top_tc->get_selection());
+                        auto hist_top_time_opt = hs.time_for_row(
+                            hist_tc.get_selection().value_or(0_vl));
+                        auto curr_top_time_opt = dst_view->time_for_row(
+                            top_tc->get_selection().value_or(0_vl));
                         if (hist_top_time_opt && curr_top_time_opt
                             && hs.row_for_time(hist_top_time_opt->ri_time)
                                 != hs.row_for_time(curr_top_time_opt->ri_time))
@@ -767,9 +796,10 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                     log_line_col = dls.column_name_to_index("min(log_line)");
                 }
 
-                if (log_line_col) {
-                    auto line_number
-                        = dls.get_cell_as_int64(db_row, log_line_col.value());
+                if (!db_row) {
+                } else if (log_line_col) {
+                    auto line_number = dls.get_cell_as_int64(
+                        db_row.value(), log_line_col.value());
 
                     if (line_number < tc->listview_rows(*tc)) {
                         tc->set_selection(vis_line_t(line_number.value()));
@@ -781,7 +811,7 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                         timeval tv;
                         exttm tm;
                         const auto col_value
-                            = dls.get_cell_as_string(db_row, lpc);
+                            = dls.get_cell_as_string(db_row.value(), lpc);
 
                         if (dts.scan(col_value.c_str(),
                                      col_value.length(),
@@ -838,19 +868,21 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
 
         case 'r':
         case 'R':
-            if (lss != nullptr) {
+            if (sel && lss != nullptr) {
                 const auto& last_time = injector::get<const relative_time&,
                                                       last_relative_time_tag>();
 
                 if (last_time.empty()) {
-                    lnav_data.ld_rl_view->set_value(
-                        "Use the 'goto' command to set the relative time to "
-                        "move by");
+                    prompt.p_editor.set_inactive_value(
+                        attr_line_t("Use the ")
+                            .append(":goto"_keyword)
+                            .append(" command to set the relative time to "
+                                    "move by"));
                 } else {
-                    vis_line_t vl = tc->get_selection(), new_vl;
+                    vis_line_t vl = sel.value(), new_vl;
                     relative_time rt = last_time;
                     content_line_t cl;
-                    struct exttm tm;
+                    exttm tm;
                     bool done = false;
 
                     if (ch.id == 'r') {
@@ -882,12 +914,14 @@ DELETE FROM lnav_user_notifications WHERE id = 'org.lnav.mouse-support'
                         }
                     } while (!done);
                     tc->set_selection(vl);
-                    lnav_data.ld_rl_view->set_value(" " + rt.to_string());
+                    prompt.p_editor.set_inactive_value(" " + rt.to_string());
                 }
             }
             break;
 
-        case KEY_CTRL('p'):
+        case NCKEY_F03:
+            lnav_data.ld_preview_status_source->update_toggle_msg(
+                lnav_data.ld_preview_hidden);
             lnav_data.ld_preview_hidden = !lnav_data.ld_preview_hidden;
             break;
 
