@@ -47,14 +47,16 @@ using namespace ww898;
 
 template<typename A>
 static void
-to_key_seq(A& dst, const char* src)
+to_key_seq(A& dst, uint32_t* src)
 {
     dst[0] = '\0';
     for (size_t lpc = 0; src[lpc]; lpc++) {
-        snprintf(dst.data() + strlen(dst.data()),
-                 dst.size() - strlen(dst.data()),
-                 "x%02x",
-                 src[lpc] & 0xff);
+        ww898::utf::utf8::write(src[lpc], [&dst](uint8_t ch) {
+            snprintf(dst.data() + strlen(dst.data()),
+                     dst.size() - strlen(dst.data()),
+                     "x%02x",
+                     ch & 0xff);
+        });
     }
 }
 
@@ -68,14 +70,15 @@ input_dispatcher::new_input(const struct timeval& current_time,
     std::string eff_str;
 
     for (size_t lpc = 0; ch.eff_text[lpc]; lpc++) {
-        fmt::format_to(
-            std::back_inserter(eff_str), FMT_STRING("{:02x}"), ch.eff_text[lpc]);
+        fmt::format_to(std::back_inserter(eff_str),
+                       FMT_STRING("{:02x}"),
+                       ch.eff_text[lpc]);
     }
     log_debug("new input %x %d/%x(%c)/%s/%s evtype=%d",
               ch.modifiers,
               ch.id,
               ch.id,
-              isprint(ch.id) ? ch.id : ' ',
+              ch.id < 0x7f && isprint(ch.id) ? ch.id : ' ',
               ch.utf8,
               eff_str.c_str(),
               ch.evtype);
@@ -110,19 +113,11 @@ input_dispatcher::new_input(const struct timeval& current_time,
         log_debug("nckey %s", keyseq.data());
         handled = this->id_key_handler(nc, ch, keyseq.data());
     } else {
-        auto seq_size = utf::utf8::char_size([&ch]() {
-                            return std::make_pair(ch.eff_text[0], 16);
-                        }).unwrapOr(size_t{1});
-        log_debug("seq_size %d", seq_size);
-        if (seq_size == 1) {
-            snprintf(
-                keyseq.data(), keyseq.size(), "x%02x", ch.eff_text[0] & 0xff);
-            log_debug("key %s", keyseq.data());
-            handled = this->id_key_handler(nc, ch, keyseq.data());
-        }
+        to_key_seq(keyseq, ch.eff_text);
+        handled = this->id_key_handler(nc, ch, keyseq.data());
     }
 
     if (handled && !handled.value()) {
         this->id_unhandled_handler(keyseq.data());
     }
-}
+}
\ No newline at end of file
