@@ -49,7 +49,7 @@ namespace {
 
 class sql_json_op : public json_op {
 public:
-    explicit sql_json_op(json_ptr& ptr) : json_op(ptr){};
+    explicit sql_json_op(json_ptr& ptr) : json_op(ptr) {};
 
     int sjo_type{-1};
     std::string sjo_str;
@@ -75,7 +75,10 @@ struct contains_userdata {
 };
 
 int
-contains_string(void* ctx, const unsigned char* str, size_t len, yajl_string_props_t*)
+contains_string(void* ctx,
+                const unsigned char* str,
+                size_t len,
+                yajl_string_props_t*)
 {
     auto sf = string_fragment::from_bytes(str, len);
     auto& cu = *((contains_userdata*) ctx);
@@ -211,7 +214,10 @@ gen_handle_boolean(void* ctx, int boolVal)
 }
 
 static int
-gen_handle_string(void* ctx, const unsigned char* stringVal, size_t len, yajl_string_props_t* props)
+gen_handle_string(void* ctx,
+                  const unsigned char* stringVal,
+                  size_t len,
+                  yajl_string_props_t* props)
 {
     sql_json_op* sjo = (sql_json_op*) ctx;
     yajl_gen gen = (yajl_gen) sjo->jo_ptr_data;
@@ -405,7 +411,10 @@ concat_gen_number(void* ctx, const char* val, size_t len)
 }
 
 static int
-concat_gen_string(void* ctx, const unsigned char* val, size_t len, yajl_string_props_t*)
+concat_gen_string(void* ctx,
+                  const unsigned char* val,
+                  size_t len,
+                  yajl_string_props_t*)
 {
     auto* cc = static_cast<concat_context*>(ctx);
 
@@ -431,7 +440,10 @@ concat_gen_end_map(void* ctx)
 }
 
 static int
-concat_gen_map_key(void* ctx, const unsigned char* key, size_t len)
+concat_gen_map_key(void* ctx,
+                   const unsigned char* key,
+                   size_t len,
+                   yajl_string_props_t* props)
 {
     auto* cc = static_cast<concat_context*>(ctx);
 
