@@ -31,6 +31,7 @@
 
 #include <chrono>
 #include <cmath>
+#include <optional>
 
 #include "listview_curses.hh"
 
@@ -38,6 +39,8 @@
 #include <time.h>
 
 #include "base/func_util.hh"
+#include "base/itertools.enumerate.hh"
+#include "base/itertools.hh"
 #include "base/keycodes.hh"
 #include "base/lnav_log.hh"
 #include "config.h"
@@ -48,25 +51,26 @@ list_gutter_source listview_curses::DEFAULT_GUTTER_SOURCE;
 
 listview_curses::listview_curses() : lv_scroll(noop_func{}) {}
 
-bool
-listview_curses::contains(int x, int y) const
+std::optional<view_curses*>
+listview_curses::contains(int x, int y)
 {
     if (!this->vc_visible) {
-        return false;
+        return std::nullopt;
     }
 
-    if (view_curses::contains(x, y)) {
-        return true;
+    auto child = view_curses::contains(x, y);
+    if (child) {
+        return child;
     }
 
-    auto dim = this->get_dimensions();
+    auto [height, width] = this->get_dimensions();
 
-    if (this->vc_x <= x && x < this->vc_x + dim.second && this->vc_y <= y
-        && y < this->vc_y + dim.first)
+    if (this->vc_x <= x && x < this->vc_x + (int) width && this->vc_y <= y
+        && y < this->vc_y + height)
     {
-        return true;
+        return this;
     }
-    return false;
+    return std::nullopt;
 }
 
 void
@@ -92,9 +96,7 @@ listview_curses::update_top_from_selection()
         return;
     }
 
-    if (this->lv_sync_selection_and_top || height <= this->lv_tail_space
-        || this->lv_top > this->lv_selection)
-    {
+    if (this->lv_sync_selection_and_top || height <= this->lv_tail_space) {
         this->set_top(this->lv_selection);
         return;
     }
@@ -102,13 +104,22 @@ listview_curses::update_top_from_selection()
     auto layout = this->layout_for_row(this->lv_selection);
     auto min_top_for_sel
         = vis_line_t(this->lv_selection - layout.lr_above_line_heights.size());
-    if (this->lv_top < min_top_for_sel) {
+    if (this->lv_top > this->lv_selection) {
+        if (this->lv_head_space > 0_vl) {
+            this->set_top(this->lv_selection
+                          - vis_line_t(layout.lr_above_line_heights.size())
+                              / 2_vl);
+        } else {
+            this->lv_top = this->lv_selection;
+        }
+    } else if (this->lv_top < min_top_for_sel) {
         this->set_top(min_top_for_sel);
-    } else if (this->lv_top == this->lv_selection) {
+    } else if (this->lv_top == this->lv_selection && this->lv_head_space > 0_vl)
+    {
         if (!layout.lr_above_line_heights.empty()) {
             auto avail_height = height - layout.lr_desired_row_height;
             if (layout.lr_above_line_heights.front() < avail_height) {
-                this->lv_top -= 1_vl;
+                this->lv_top -= this->lv_head_space;
             }
         }
     }
@@ -120,7 +131,7 @@ listview_curses::get_dimensions(vis_line_t& height_out,
 {
     unsigned int height;
 
-    if (this->lv_window == nullptr) {
+    if (this->vc_window == nullptr) {
         height_out = std::max(this->lv_height, 1_vl);
         if (this->lv_source) {
             width_out = this->lv_source->listview_width(*this);
@@ -130,7 +141,7 @@ listview_curses::get_dimensions(vis_line_t& height_out,
     } else {
         unsigned int width_tmp;
 
-        ncplane_dim_yx(this->lv_window, &height, &width_tmp);
+        ncplane_dim_yx(this->vc_window, &height, &width_tmp);
         width_out = width_tmp;
         if (this->lv_height < 0) {
             height_out
@@ -142,7 +153,7 @@ listview_curses::get_dimensions(vis_line_t& height_out,
             height_out = this->lv_height;
         }
     }
-    if (this->vc_x < width_out) {
+    if (this->vc_x < (int) width_out) {
         width_out -= this->vc_x;
     } else {
         width_out = 0;
@@ -171,14 +182,6 @@ listview_curses::reload_data()
             } else if (this->lv_selection >= this->get_inner_height()) {
                 this->set_selection_without_context(this->get_inner_height()
                                                     - 1_vl);
-            } else {
-                auto curr_sel = this->get_selection();
-
-                if (curr_sel == -1_vl) {
-                    curr_sel = 0_vl;
-                }
-                this->lv_selection = -1_vl;
-                this->set_selection_without_context(curr_sel);
             }
 
             this->update_top_from_selection();
@@ -230,12 +233,14 @@ listview_curses::handle_key(const ncinput& ch)
 
         case '\r':
         case 'j':
+        case KEY_CTRL('N'):
         case NCKEY_DOWN:
         case NCKEY_ENTER:
             this->shift_selection(shift_amount_t::down_line);
             break;
 
         case 'k':
+        case KEY_CTRL('P'):
         case NCKEY_UP:
             this->shift_selection(shift_amount_t::up_line);
             break;
@@ -251,12 +256,16 @@ listview_curses::handle_key(const ncinput& ch)
             }
             break;
 
-        case KEY_CTRL(']'):
-            if (this->lv_overlay_source != nullptr && !this->lv_overlay_focused)
+        case KEY_CTRL(']'): {
+            auto sel = this->get_selection();
+
+            if (sel && sel.value() < this->get_inner_height()
+                && this->lv_overlay_source != nullptr
+                && !this->lv_overlay_focused)
             {
                 std::vector<attr_line_t> overlay_content;
                 this->lv_overlay_source->list_value_for_overlay(
-                    *this, this->get_selection(), overlay_content);
+                    *this, this->lv_selection, overlay_content);
                 if (!overlay_content.empty()) {
                     auto bot = this->get_bottom();
                     this->lv_overlay_focused = !this->lv_overlay_focused;
@@ -264,7 +273,7 @@ listview_curses::handle_key(const ncinput& ch)
                         overlay_content.size(), height));
 
                     if (this->lv_selection + overlay_height >= bot) {
-                        this->shift_top(overlay_height, true);
+                        this->set_top(this->lv_selection, true);
                     }
                     this->lv_source->listview_selection_changed(*this);
                     this->set_needs_update();
@@ -273,6 +282,7 @@ listview_curses::handle_key(const ncinput& ch)
                 retval = false;
             }
             break;
+        }
 
         case 'b':
         case NCKEY_BACKSPACE:
@@ -287,50 +297,52 @@ listview_curses::handle_key(const ncinput& ch)
             {
                 this->set_selection(0_vl);
             } else {
-                auto shift_amount
-                    = -(this->rows_available(this->lv_top, RD_UP) - 1_vl);
+                auto rows_avail = this->rows_available(this->lv_top, RD_UP);
+                if (rows_avail <= 1_vl) {
+                    // may happen in case of jumping over very long lines
+                    // and we need to garantee at least some movement
+                    rows_avail = 2_vl;
+                }
+                auto shift_amount = -(rows_avail - 1_vl);
                 this->shift_top(shift_amount);
             }
             break;
 
         case ' ':
         case NCKEY_PGDOWN: {
-            if (this->lv_overlay_source != nullptr) {
+            auto inner_height = this->get_inner_height();
+            auto sel = this->get_selection();
+            if (this->lv_overlay_source != nullptr && sel
+                && sel.value() < inner_height)
+            {
                 std::vector<attr_line_t> overlay_content;
                 this->lv_overlay_source->list_value_for_overlay(
-                    *this, this->get_selection(), overlay_content);
+                    *this, sel.value(), overlay_content);
                 if (!overlay_content.empty()) {
                     this->shift_selection(shift_amount_t::down_page);
                     break;
                 }
             }
 
-            auto rows_avail = this->rows_available(this->lv_top, RD_DOWN);
-            if (rows_avail == 0_vl) {
-                rows_avail = 2_vl;
-            } else if (rows_avail > 2_vl) {
-                rows_avail -= 1_vl;
-            }
-            auto top_for_last = this->get_top_for_last_row();
+            if (this->lv_top + height * 2 > inner_height) {
+                // NB: getting the last row can read from the file, which can
+                // be expensive.  Use sparingly.
+                auto top_for_last = this->get_top_for_last_row();
 
-            if ((this->lv_top < top_for_last)
-                && (this->lv_top + rows_avail > top_for_last))
-            {
-                this->set_top(top_for_last);
-                if (this->lv_selection <= top_for_last) {
-                    this->set_selection(top_for_last + 1_vl);
-                }
-            } else if (this->lv_top > top_for_last) {
-                this->set_top(top_for_last);
-            } else {
-                this->shift_top(rows_avail);
-
-                auto inner_height = this->get_inner_height();
-                if (this->lv_selectable && this->lv_top >= top_for_last
-                    && inner_height > 0_vl)
-                {
+                if (this->lv_top + height > inner_height) {
                     this->set_selection(inner_height - 1_vl);
+                } else {
+                    this->set_top(top_for_last);
                 }
+            } else {
+                auto rows_avail = this->rows_available(this->lv_top, RD_DOWN);
+                if (rows_avail == 0_vl) {
+                    rows_avail = 2_vl;
+                } else if (rows_avail > 2_vl) {
+                    rows_avail -= 1_vl;
+                }
+
+                this->shift_top(rows_avail);
             }
             break;
         }
@@ -357,7 +369,7 @@ listview_curses::handle_key(const ncinput& ch)
             if (this->lv_overlay_focused) {
                 std::vector<attr_line_t> overlay_content;
                 this->lv_overlay_source->list_value_for_overlay(
-                    *this, this->get_selection(), overlay_content);
+                    *this, this->get_selection().value(), overlay_content);
                 auto overlay_height
                     = this->get_overlay_height(overlay_content.size(), height);
                 auto ov_top_for_last = vis_line_t{
@@ -410,16 +422,20 @@ listview_curses::handle_key(const ncinput& ch)
 }
 
 vis_line_t
-listview_curses::get_overlay_top(vis_line_t row, size_t count, size_t total)
+listview_curses::get_overlay_top(
+    vis_line_t row,
+    size_t count,
+    const std::vector<attr_line_t>& overlay_content)
 {
-    if (row == this->get_selection()) {
-        if (this->lv_focused_overlay_selection >= total) {
+    const auto total = overlay_content.size();
+    auto max_top = vis_line_t(total - count);
+    if (row == this->get_selection() && this->lv_overlay_focused) {
+        if (this->lv_focused_overlay_selection >= (ssize_t) total) {
             this->lv_focused_overlay_selection = vis_line_t(total) - 1_vl;
         }
         if (this->lv_focused_overlay_selection < 0_vl) {
             this->lv_focused_overlay_selection = 0_vl;
         }
-        auto max_top = vis_line_t(total - count);
         if (this->lv_focused_overlay_selection <= this->lv_focused_overlay_top)
         {
             this->lv_focused_overlay_top = this->lv_focused_overlay_selection;
@@ -440,7 +456,27 @@ listview_curses::get_overlay_top(vis_line_t row, size_t count, size_t total)
         return this->lv_focused_overlay_top;
     }
 
-    return 0_vl;
+    auto retval = 0_vl;
+
+    for (size_t lpc = 0; lpc < total; lpc++) {
+        const auto& oc_line = overlay_content[lpc];
+        auto saw_opt = get_string_attr(oc_line.al_attrs, VC_ANCHOR);
+
+        if (saw_opt && saw_opt.value().get() == "default-focus"_frag) {
+            auto ov_sel = vis_line_t(lpc);
+
+            if (ov_sel >= count) {
+                retval = ov_sel - vis_line_t(count) + 2_vl;
+            }
+
+            break;
+        }
+    }
+    if (retval > max_top) {
+        retval = max_top;
+    }
+
+    return retval;
 }
 
 bool
@@ -449,7 +485,7 @@ listview_curses::do_update()
     static auto& vc = view_colors::singleton();
     bool retval = false;
 
-    if (this->lv_window == nullptr || this->lv_height == 0 || !this->vc_visible)
+    if (this->vc_window == nullptr || this->lv_height == 0 || !this->vc_visible)
     {
         return view_curses::do_update();
     }
@@ -467,7 +503,7 @@ listview_curses::do_update()
     while (this->vc_needs_update) {
         vis_line_t row;
         attr_line_t overlay_line;
-        struct line_range lr;
+        line_range lr;
         unsigned long wrap_width;
         int y = this->vc_y, bottom;
         auto role_attrs = vc.attrs_for_role(this->vc_default_role);
@@ -490,6 +526,55 @@ listview_curses::do_update()
         this->lv_source->listview_value_for_rows(*this, row, rows);
         this->lv_display_lines.clear();
         this->lv_display_lines_row = row;
+        auto x = this->vc_x;
+        uint64_t border_channels = 0;
+        if (this->lv_border_left_role) {
+            this->lv_display_lines.emplace_back(empty_space{});
+            border_channels = vc.to_channels(
+                vc.attrs_for_role(this->lv_border_left_role.value()));
+
+            auto al = attr_line_t("  ");
+            if (!this->vc_title.empty()) {
+                al.append(this->vc_title,
+                          VC_STYLE.value(text_attrs::with_bold()));
+            }
+            al.al_attrs.emplace_back(line_range{0, 1},
+                                     VC_GRAPHIC.value(NCACS_ULCORNER));
+            auto hline_lr = line_range{1, (int) width - 1};
+            if (!this->vc_title.empty()) {
+                al.al_attrs.emplace_back(line_range{1, 2},
+                                         VC_GRAPHIC.value(NCACS_RTEE));
+                al.al_attrs.emplace_back(
+                    line_range{
+                        2 + (int) this->vc_title.length(),
+                        2 + (int) this->vc_title.length() + 1,
+                    },
+                    VC_GRAPHIC.value(NCACS_LTEE));
+                hline_lr.lr_start += 1 + this->vc_title.length() + 1;
+            }
+            al.al_attrs.emplace_back(hline_lr, VC_GRAPHIC.value(NCACS_HLINE));
+            al.al_attrs.emplace_back(line_range{(int) width - 1, (int) width},
+                                     VC_GRAPHIC.value(NCACS_URCORNER));
+            mvwattrline(this->vc_window,
+                        y,
+                        x,
+                        al,
+                        line_range{0, (int) width},
+                        this->lv_border_left_role.value());
+
+            y += 1;
+            for (auto border_y = y; border_y < bottom; border_y++) {
+                ncplane_putstr_yx(
+                    this->vc_window, border_y, this->vc_x, NCACS_VLINE);
+                ncplane_set_cell_yx(this->vc_window,
+                                    border_y,
+                                    x,
+                                    NCSTYLE_ALTCHARSET,
+                                    border_channels);
+            }
+            x += 1;
+            width -= 1;
+        }
         while (y < bottom) {
             lr.lr_start = this->lv_left;
             lr.lr_end = this->lv_left + wrap_width;
@@ -498,7 +583,7 @@ listview_curses::do_update()
                     *this, y - this->vc_y, bottom - this->vc_y, overlay_line))
             {
                 this->lv_display_lines.push_back(static_overlay_content{});
-                mvwattrline(this->lv_window, y, this->vc_x, overlay_line, lr);
+                mvwattrline(this->vc_window, y, x, overlay_line, lr);
                 overlay_line.clear();
                 ++y;
             } else if (row < (int) row_count) {
@@ -509,20 +594,19 @@ listview_curses::do_update()
                 }
 
                 mvwattrline_result write_res;
+                auto wrapped_line = size_t{0};
                 do {
-                    this->lv_display_lines.push_back(main_content{row});
+                    this->lv_display_lines.push_back(
+                        main_content{row, wrapped_line, lr});
                     if (this->lv_word_wrap) {
-                        // XXX mvwhline(this->lv_window, y, this->vc_x, ' ',
+                        // XXX mvwhline(this->vc_window, y, this->vc_x, ' ',
                         // width);
                     }
-                    write_res = mvwattrline(this->lv_window,
-                                            y,
-                                            this->vc_x,
-                                            al,
-                                            lr,
-                                            this->vc_default_role);
+                    write_res = mvwattrline(
+                        this->vc_window, y, x, al, lr, this->vc_default_role);
                     lr.lr_start = write_res.mr_chars_out;
                     lr.lr_end = write_res.mr_chars_out + width - 1;
+                    wrapped_line += 1;
                     ++y;
                 } while (this->lv_word_wrap && y < bottom
                          && write_res.mr_bytes_remaining > 0);
@@ -541,12 +625,12 @@ listview_curses::do_update()
                             break;
                         }
 
-                        this->lv_display_lines.push_back(overlay_menu{
+                        this->lv_display_lines.emplace_back(overlay_menu{
                             ov_menu_row,
                         });
-                        mvwattrline(this->lv_window,
+                        mvwattrline(this->vc_window,
                                     y,
-                                    this->vc_x,
+                                    x,
                                     ov_menu_line,
                                     line_range{0, (int) wrap_width},
                                     role_t::VCR_ALT_ROW);
@@ -560,7 +644,7 @@ listview_curses::do_update()
                         row_overlay_content.size(), height);
                     auto ov_height_remaining = overlay_height;
                     auto overlay_top = this->get_overlay_top(
-                        row, overlay_height, row_overlay_content.size());
+                        row, overlay_height, row_overlay_content);
                     auto overlay_row = overlay_top;
                     if (row_overlay_content.size() > 1) {
                         auto hdr
@@ -573,9 +657,9 @@ listview_curses::do_update()
                                 VC_STYLE.value(ov_hdr_attrs));
                             this->lv_display_lines.emplace_back(
                                 static_overlay_content{});
-                            mvwattrline(this->lv_window,
+                            mvwattrline(this->vc_window,
                                         y,
-                                        this->vc_x,
+                                        x,
                                         ov_hdr,
                                         lr,
                                         role_t::VCR_STATUS_INFO);
@@ -593,23 +677,25 @@ listview_curses::do_update()
                                 VC_ROLE.value(role_t::VCR_CURSOR_LINE));
                         }
 
-                        this->lv_display_lines.emplace_back(
-                            overlay_content{row,
-                                            overlay_row,
-                                            overlay_height,
-                                            row_overlay_content.size()});
-                        mvwattrline(this->lv_window,
+                        this->lv_display_lines.emplace_back(overlay_content{
+                            row,
+                            overlay_row,
+                            overlay_height,
+                            vis_line_t{(int) row_overlay_content.size()},
+                        });
+                        mvwattrline(this->vc_window,
                                     y,
-                                    this->vc_x,
+                                    x,
                                     row_overlay_content[overlay_row],
                                     lr,
                                     role_t::VCR_ALT_ROW);
-                        ov_height_remaining -= 1;
+                        ov_height_remaining -= 1_vl;
                         ++overlay_row;
                         ++y;
                     }
 
-                    if (overlay_height != row_overlay_content.size()) {
+                    if (overlay_height != (ssize_t) row_overlay_content.size())
+                    {
                         double progress = 1.0;
                         double coverage = 1.0;
                         vis_line_t lines;
@@ -629,7 +715,7 @@ listview_curses::do_update()
                                 (int) overlay_height,
                                 (int) (coverage * (double) overlay_height)));
 
-                        for (unsigned int gutter_y = overlay_y;
+                        for (auto gutter_y = overlay_y;
                              gutter_y < (overlay_y + overlay_height);
                              gutter_y++)
                         {
@@ -642,20 +728,16 @@ listview_curses::do_update()
                                        ? NCACS_LRCORNER
                                        : NCACS_VLINE);
 
-                            if (gutter_y >= (unsigned int) scroll_y
-                                && gutter_y <= (unsigned int) lines)
-                            {
+                            if (gutter_y >= scroll_y && gutter_y <= lines) {
                                 role = bar_role;
                             }
                             attrs = vc.attrs_for_role(role);
-                            ncplane_putstr_yx(this->lv_window,
-                                              gutter_y,
-                                              this->vc_x + width - 2,
-                                              ch);
+                            ncplane_putstr_yx(
+                                this->vc_window, gutter_y, x + width - 2, ch);
                             ncplane_set_cell_yx(
-                                this->lv_window,
+                                this->vc_window,
                                 gutter_y,
-                                this->vc_x + width - 2,
+                                x + width - 2,
                                 attrs.ta_attrs | NCSTYLE_ALTCHARSET,
                                 view_colors::to_channels(attrs));
                         }
@@ -666,14 +748,14 @@ listview_curses::do_update()
             } else {
                 nccell clear_cell;
                 nccell_init(&clear_cell);
-                nccell_prime(this->lv_window,
+                nccell_prime(this->vc_window,
                              &clear_cell,
                              " ",
                              0,
                              view_colors::to_channels(role_attrs));
-                ncplane_cursor_move_yx(this->lv_window, y, this->vc_x);
-                ncplane_hline(this->lv_window, &clear_cell, width);
-                nccell_release(this->lv_window, &clear_cell);
+                ncplane_cursor_move_yx(this->vc_window, y, x);
+                ncplane_hline(this->vc_window, &clear_cell, width);
+                nccell_release(this->vc_window, &clear_cell);
 
                 this->lv_display_lines.push_back(empty_space{});
                 ++y;
@@ -688,20 +770,24 @@ listview_curses::do_update()
         }
 
         if (this->lv_show_scrollbar) {
+            auto scroll_offset = (this->lv_border_left_role ? 1 : 0);
+            auto avail_height = height - scroll_offset;
             double progress = 1.0;
             double coverage = 1.0;
-            double adjusted_height = (double) row_count / (double) height;
+            double adjusted_height = (double) row_count / (double) avail_height;
 
             if (row_count > 0) {
                 progress = (double) this->lv_top / (double) row_count;
-                coverage = (double) height / (double) row_count;
+                coverage = (double) avail_height / (double) row_count;
             }
 
-            this->lv_scroll_top = (int) (progress * (double) height);
+            this->lv_scroll_top
+                = scroll_offset + (int) (progress * (double) avail_height);
             this->lv_scroll_bottom = this->lv_scroll_top
-                + std::min((int) height, (int) (coverage * (double) height));
+                + std::min((int) avail_height,
+                           (int) (coverage * (double) avail_height));
 
-            for (unsigned int gutter_y = this->vc_y;
+            for (auto gutter_y = this->vc_y + scroll_offset;
                  gutter_y < (this->vc_y + height);
                  gutter_y++)
             {
@@ -725,23 +811,20 @@ listview_curses::do_update()
                     role = bar_role;
                 }
                 attrs = vc.attrs_for_role(role);
-                ncplane_putstr_yx(
-                    this->lv_window, gutter_y, this->vc_x + width - 1, ch);
-                ncplane_set_cell_yx(this->lv_window,
+                ncplane_putstr_yx(this->vc_window, gutter_y, x + width - 1, ch);
+                ncplane_set_cell_yx(this->vc_window,
                                     gutter_y,
-                                    this->vc_x + width - 1,
+                                    x + width - 1,
                                     attrs.ta_attrs | NCSTYLE_ALTCHARSET,
                                     view_colors::to_channels(attrs));
             }
         }
 
         if (this->lv_show_bottom_border) {
-            int bottom_y = this->vc_y + height - 1;
-            for (int lpc = 0; lpc < width - 1; lpc++) {
-                ncplane_on_styles_yx(this->lv_window,
-                                     bottom_y,
-                                     this->vc_x + lpc,
-                                     NCSTYLE_UNDERLINE);
+            auto bottom_y = this->vc_y + height - 1;
+            for (size_t lpc = 0; lpc < width - 1; lpc++) {
+                ncplane_on_styles_yx(
+                    this->vc_window, bottom_y, x + lpc, NCSTYLE_UNDERLINE);
             }
         }
 
@@ -774,7 +857,7 @@ listview_curses::shift_selection(shift_amount_t sa)
 
     this->get_dimensions(height, width);
     if (this->lv_overlay_focused) {
-        const auto focused = this->get_selection();
+        const auto focused = this->get_selection().value();
         std::vector<attr_line_t> overlay_content;
 
         this->lv_overlay_source->list_value_for_overlay(
@@ -806,7 +889,7 @@ listview_curses::shift_selection(shift_amount_t sa)
                 }
                 case shift_amount_t::down_line:
                     if (this->lv_focused_overlay_selection + 1
-                        < overlay_content.size())
+                        < (ssize_t) overlay_content.size())
                     {
                         this->lv_focused_overlay_selection += 1_vl;
                     }
@@ -907,7 +990,7 @@ listview_curses::handle_mouse(mouse_event& me)
     switch (me.me_button) {
         case mouse_button_t::BUTTON_SCROLL_UP:
         case mouse_button_t::BUTTON_SCROLL_DOWN: {
-            if (me.me_y < this->lv_display_lines.size()) {
+            if (me.me_y < (ssize_t) this->lv_display_lines.size()) {
                 auto display_content = this->lv_display_lines[me.me_y];
 
                 if (display_content.is<overlay_content>()
@@ -921,6 +1004,14 @@ listview_curses::handle_mouse(mouse_event& me)
                         {
                             this->lv_mouse_time = me.me_time;
                             this->lv_focused_overlay_top -= 1_vl;
+                            if (this->lv_focused_overlay_selection
+                                >= this->lv_focused_overlay_top
+                                    + (oc.oc_height - 1_vl))
+                            {
+                                this->lv_focused_overlay_selection
+                                    = this->lv_focused_overlay_top
+                                    + oc.oc_height - 2_vl;
+                            }
                             this->set_needs_update();
                         }
 
@@ -938,8 +1029,8 @@ listview_curses::handle_mouse(mouse_event& me)
                             }
                             this->set_needs_update();
                         }
-                        return true;
                     }
+                    return true;
                 }
             }
             this->shift_top(vis_line_t(scroll_polarity(me.me_button) * 2_vl),
@@ -1062,50 +1153,46 @@ listview_curses::set_top(vis_line_t top, bool suppress_flash)
         }
     } else if (this->lv_top != top) {
         this->lv_top = top;
-        if (this->lv_selectable) {
-            if (this->lv_selection < 0_vl) {
-                this->set_selection_without_context(top);
+        if (this->lv_selectable && this->lv_selection >= 0_vl) {
+            auto layout = this->layout_for_row(this->lv_top);
+            auto last_row = this->lv_top
+                + vis_line_t(layout.lr_below_line_heights.size());
+
+            if (this->lv_top <= this->lv_selection
+                && this->lv_selection <= last_row)
+            {
+                // selection is already in view, nothing to do
+            } else if (layout.lr_below_line_heights.size() < 2) {
+                this->set_selection_without_context(this->lv_top);
             } else {
-                auto layout = this->layout_for_row(this->lv_top);
-                auto last_row = this->lv_top
-                    + vis_line_t(layout.lr_below_line_heights.size());
+                auto sel_location = selection_location_t::middle;
 
-                if (this->lv_top <= this->lv_selection
-                    && this->lv_selection <= last_row)
+                if (this->lv_top - 5_vl <= this->lv_selection
+                    && this->lv_selection < this->lv_top)
                 {
-                    // selection is already in view, nothing to do
-                } else if (layout.lr_below_line_heights.size() < 2) {
-                    this->set_selection_without_context(this->lv_top);
-                } else {
-                    auto sel_location = selection_location_t::middle;
+                    sel_location = selection_location_t::upper;
+                } else if (last_row < this->lv_selection
+                           && this->lv_selection <= last_row + 5_vl)
+                {
+                    sel_location = selection_location_t::lower;
+                }
 
-                    if (this->lv_top - 5_vl <= this->lv_selection
-                        && this->lv_selection < this->lv_top)
-                    {
-                        sel_location = selection_location_t::upper;
-                    } else if (last_row < this->lv_selection
-                               && this->lv_selection <= last_row + 5_vl)
-                    {
-                        sel_location = selection_location_t::lower;
+                switch (sel_location) {
+                    case selection_location_t::upper: {
+                        this->set_selection_without_context(this->lv_top
+                                                            + 1_vl);
+                        break;
                     }
-
-                    switch (sel_location) {
-                        case selection_location_t::upper: {
-                            this->set_selection_without_context(this->lv_top
-                                                                + 1_vl);
-                            break;
-                        }
-                        case selection_location_t::middle: {
-                            auto middle_of_below = vis_line_t(
-                                layout.lr_below_line_heights.size() / 2);
-                            this->set_selection_without_context(
-                                this->lv_top + middle_of_below);
-                            break;
-                        }
-                        case selection_location_t::lower: {
-                            this->set_selection_without_context(last_row);
-                            break;
-                        }
+                    case selection_location_t::middle: {
+                        auto middle_of_below = vis_line_t(
+                            layout.lr_below_line_heights.size() / 2);
+                        this->set_selection_without_context(this->lv_top
+                                                            + middle_of_below);
+                        break;
+                    }
+                    case selection_location_t::lower: {
+                        this->set_selection_without_context(last_row);
+                        break;
                     }
                 }
             }
@@ -1168,17 +1255,29 @@ listview_curses::layout_for_row(vis_line_t row) const
     {
         auto above_height_avail
             = height - retval.lr_desired_row_height - this->lv_tail_space;
-        auto curr_above_row = row - 1_vl;
-        while (curr_above_row >= 0_vl && above_height_avail > 0_vl) {
+        // scan forwards since that will be nicer to the line_buffer cache
+        auto highest_above_row = std::max(0_vl, row - above_height_avail);
+        for (auto curr_above_row = highest_above_row; curr_above_row < row;
+             ++curr_above_row)
+        {
             auto curr_above_height
                 = this->height_for_row(curr_above_row, height, width);
-
+            retval.lr_above_line_heights.emplace_back(curr_above_height);
+        }
+        std::reverse(retval.lr_above_line_heights.begin(),
+                     retval.lr_above_line_heights.end());
+        auto above_limit = std::optional<size_t>();
+        for (const auto& [index, curr_above_height] :
+             lnav::itertools::enumerate(retval.lr_above_line_heights))
+        {
             above_height_avail -= curr_above_height;
             if (above_height_avail < 0_vl) {
+                above_limit = index;
                 break;
             }
-            curr_above_row -= 1_vl;
-            retval.lr_above_line_heights.emplace_back(curr_above_height);
+        }
+        if (above_limit) {
+            retval.lr_above_line_heights.resize(above_limit.value());
         }
     }
     {
@@ -1311,10 +1410,6 @@ listview_curses::set_selection(vis_line_t sel)
         return;
     }
 
-    if (sel < 0_vl) {
-        return;
-    }
-
     this->set_selection_without_context(sel);
     this->update_top_from_selection();
 }
@@ -1376,33 +1471,51 @@ listview_curses::set_left(int left)
     this->set_needs_update();
 }
 
-size_t
+vis_line_t
 listview_curses::get_overlay_height(size_t total, vis_line_t view_height) const
 {
-    return std::min(total, static_cast<size_t>(2 * (view_height / 3)));
+    return std::min(vis_line_t{(int) total}, (2_vl * (view_height / 3_vl)));
 }
 
 void
 listview_curses::set_overlay_selection(std::optional<vis_line_t> sel)
 {
+    if (this->lv_overlay_source == nullptr) {
+        return;
+    }
     if (sel) {
-        if (sel.value() == this->lv_focused_overlay_selection) {
+        if (this->lv_overlay_focused
+            && sel.value() == this->lv_focused_overlay_selection)
+        {
             return;
         }
 
         std::vector<attr_line_t> overlay_content;
         this->lv_overlay_source->list_value_for_overlay(
-            *this, this->get_selection(), overlay_content);
+            *this, this->get_selection().value(), overlay_content);
         if (!overlay_content.empty()) {
-            this->lv_overlay_focused = true;
             if (sel.value() < 0) {
                 this->lv_focused_overlay_selection = 0_vl;
-            } else if (sel.value() >= overlay_content.size()) {
+            } else if (sel.value() >= (ssize_t) overlay_content.size()) {
                 this->lv_focused_overlay_selection
-                    = vis_line_t(overlay_content.size());
+                    = vis_line_t(overlay_content.size()) - 1_vl;
             } else {
                 this->lv_focused_overlay_selection = sel.value();
             }
+
+            const auto [height, width] = this->get_dimensions();
+            auto bot = this->get_bottom();
+            auto overlay_height = vis_line_t(
+                this->get_overlay_height(overlay_content.size(), height));
+
+            if (!this->lv_overlay_focused) {
+                this->lv_focused_overlay_top = this->get_overlay_top(
+                    this->lv_selection, overlay_height, overlay_content);
+                this->lv_overlay_focused = true;
+            }
+            if (this->lv_selection + overlay_height >= bot) {
+                this->set_top(this->lv_selection, true);
+            }
         }
     } else {
         this->lv_overlay_focused = false;
@@ -1412,3 +1525,10 @@ listview_curses::set_overlay_selection(std::optional<vis_line_t> sel)
     this->lv_source->listview_selection_changed(*this);
     this->set_needs_update();
 }
+
+int
+listview_curses::get_y_for_selection() const
+{
+    return this->get_y()
+        + (int) (this->get_selection().value_or(0_vl) - this->get_top());
+}
