@@ -27,6 +27,8 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <unordered_map>
+
 #include "lnav.indexing.hh"
 
 #include "bound_tags.hh"
@@ -35,6 +37,7 @@
 #include "service_tags.hh"
 #include "session_data.hh"
 #include "sql_util.hh"
+#include "yajlpp/yajlpp_def.hh"
 
 using namespace std::chrono_literals;
 using namespace lnav::roles::literals;
@@ -44,20 +47,18 @@ using namespace lnav::roles::literals;
  */
 class loading_observer : public logfile_observer {
 public:
-    loading_observer() : lo_last_offset(0) {}
-
-    indexing_result logfile_indexing(const logfile* lf,
-                                     file_off_t off,
-                                     file_size_t total) override
+    lnav::progress_result_t logfile_indexing(const logfile* lf,
+                                             file_off_t off,
+                                             file_ssize_t total) override
     {
         static sig_atomic_t index_counter = 0;
 
         if (lnav_data.ld_window == nullptr) {
-            return indexing_result::CONTINUE;
+            return lnav::progress_result_t::ok;
         }
 
         if (lnav_data.ld_sigint_count.load() > 0) {
-            return indexing_result::BREAK;
+            return lnav::progress_result_t::interrupt;
         }
 
         /* XXX require(off <= total); */
@@ -65,7 +66,8 @@ public:
             off = total;
         }
 
-        if ((((size_t) off == total) && (this->lo_last_offset != off))
+        auto retval = lnav::progress_result_t::ok;
+        if (((off == total) && (this->lo_last_offset != off))
             || ui_periodic_timer::singleton().time_to_update(index_counter))
         {
             if (off == total) {
@@ -73,26 +75,29 @@ public:
             } else {
                 lnav_data.ld_bottom_source.update_loading(off, total);
             }
-            do_observer_update(lf);
+            lnav_data.ld_status[LNS_BOTTOM].set_needs_update();
+            if (do_observer_update(lf) == lnav::progress_result_t::interrupt) {
+                retval = lnav::progress_result_t::interrupt;
+            }
             this->lo_last_offset = off;
         }
 
         if (!lnav_data.ld_looping) {
-            return indexing_result::BREAK;
+            retval = lnav::progress_result_t::interrupt;
         }
-        return indexing_result::CONTINUE;
+        return retval;
     }
 
-    off_t lo_last_offset;
+    off_t lo_last_offset{0};
 };
 
-void
+lnav::progress_result_t
 do_observer_update(const logfile* lf)
 {
     if (lf != nullptr && lnav_data.ld_mode == ln_mode_t::FILES
         && lnav_data.ld_exec_phase < lnav_exec_phase::INTERACTIVE)
     {
-        auto& fc = lnav_data.ld_active_files;
+        const auto& fc = lnav_data.ld_active_files;
         size_t index = 0;
 
         for (const auto& curr_file : fc.fc_files) {
@@ -106,19 +111,15 @@ do_observer_update(const logfile* lf)
             lnav_data.ld_files_view.do_update();
         }
     }
-    if (handle_winch(nullptr)) {
-        layout_views();
-        lnav_data.ld_view_stack.do_update();
-    }
-    lnav_data.ld_status_refresher();
+    return lnav_data.ld_status_refresher(lnav::func::op_type::interactive);
 }
 
 void
 rebuild_hist()
 {
-    logfile_sub_source& lss = lnav_data.ld_log_source;
-    hist_source2& hs = lnav_data.ld_hist_source2;
-    int zoom = lnav_data.ld_zoom_level;
+    auto& lss = lnav_data.ld_log_source;
+    auto& hs = lnav_data.ld_hist_source2;
+    const auto zoom = lnav_data.ld_zoom_level;
 
     hs.set_time_slice(ZOOM_LEVELS[zoom]);
     lss.reload_index_delegate();
@@ -140,8 +141,8 @@ public:
         auto& ftf = lnav_data.ld_files_to_front;
 
         ftf.remove_if([&lf](const auto& elem) {
-            return elem.first == lf->get_filename()
-                || elem.first == lf->get_open_options().loo_filename;
+            return elem == lf->get_filename()
+                || elem == lf->get_open_options().loo_filename;
         });
         if (lnav_data.ld_log_source.insert_file(lf)) {
             this->did_promotion = true;
@@ -156,6 +157,10 @@ public:
                     lnav_data.ld_vtab_manager->register_vtab(vt);
                 }
             }
+            if (lf->get_open_options().loo_source == logfile_name_source::USER)
+            {
+                lf->set_include_in_session(true);
+            }
 
             auto iter = session_data.sd_file_states.find(lf->get_filename());
             if (iter != session_data.sd_file_states.end()) {
@@ -182,11 +187,11 @@ public:
         const auto& ftf = lnav_data.ld_files_to_front;
 
         if (!ftf.empty()
-            && (ftf.front().first == lf->get_filename()
-                || ftf.front().first == lf->get_open_options().loo_filename))
+            && (ftf.front() == lf->get_filename()
+                || ftf.front() == lf->get_open_options().loo_filename))
         {
             this->front_file = lf;
-            this->front_top = lnav_data.ld_files_to_front.front().second;
+            this->front_top = lf->get_open_options().loo_init_location;
 
             lnav_data.ld_files_to_front.pop_front();
         }
@@ -252,36 +257,55 @@ rebuild_indexes(std::optional<ui_clock::time_point> deadline)
             }
 
             std::optional<vis_line_t> new_top_opt;
-            cb.front_top.match(
-                [&new_top_opt](vis_line_t vl) {
-                    log_info("file open request to jump to line: %d", (int) vl);
-                    if (vl < 0_vl) {
-                        vl += lnav_data.ld_views[LNV_TEXT].get_inner_height();
-                    }
-                    if (vl < lnav_data.ld_views[LNV_TEXT].get_inner_height()) {
-                        new_top_opt = vl;
-                    }
-                },
-                [&new_top_opt](const std::string& loc) {
-                    log_info("file open request to jump to anchor: %s",
-                             loc.c_str());
-                    auto* ta = dynamic_cast<text_anchors*>(
-                        lnav_data.ld_views[LNV_TEXT].get_sub_source());
+            if (cb.front_top.valid()) {
+                cb.front_top.match(
+                    [&new_top_opt, &cb](file_location_tail tail) {
+                        switch (cb.front_file->get_text_format()) {
+                            case text_format_t::TF_UNKNOWN:
+                            case text_format_t::TF_LOG:
+                                log_info("file open request to tail");
+                                break;
+                            default:
+                                log_info("file open is %s, moving to top",
+                                         fmt::to_string(
+                                             cb.front_file->get_text_format())
+                                             .c_str());
+                                new_top_opt = 0_vl;
+                                break;
+                        }
+                    },
+                    [&new_top_opt](int vl) {
+                        log_info("file open request to jump to line: %d", vl);
+                        if (vl < 0) {
+                            vl += lnav_data.ld_views[LNV_TEXT]
+                                      .get_inner_height();
+                        }
+                        if (vl
+                            < lnav_data.ld_views[LNV_TEXT].get_inner_height()) {
+                            new_top_opt = vis_line_t(vl);
+                        }
+                    },
+                    [&new_top_opt](const std::string& loc) {
+                        log_info("file open request to jump to anchor: %s",
+                                 loc.c_str());
+                        auto* ta = dynamic_cast<text_anchors*>(
+                            lnav_data.ld_views[LNV_TEXT].get_sub_source());
 
-                    if (ta != nullptr) {
-                        new_top_opt = ta->row_for_anchor(loc);
-                    }
-                });
+                        if (ta != nullptr) {
+                            new_top_opt = ta->row_for_anchor(loc);
+                        }
+                    });
+            }
             if (new_top_opt) {
                 log_info("  setting requested top line: %d",
                          (int) new_top_opt.value());
                 text_view.set_selection(new_top_opt.value());
                 log_info("  actual top is now: %d", (int) text_view.get_top());
                 log_info("  actual selection is now: %d",
-                         (int) text_view.get_selection());
+                         (int) text_view.get_selection().value_or(-1_vl));
                 scroll_downs[LNV_TEXT] = false;
             } else {
-                log_warning("could not jump to requested line");
+                log_info("no line requested");
             }
         }
         if (cb.did_promotion && deadline) {
@@ -295,10 +319,13 @@ rebuild_indexes(std::optional<ui_clock::time_point> deadline)
     std::vector<std::shared_ptr<logfile>> closed_files;
     for (auto& lf : lnav_data.ld_active_files.fc_files) {
         if (!lf->exists() || lf->is_closed()) {
-            log_info("closed log file: %s", lf->get_filename().c_str());
+            log_info("%s file: %s",
+                     !lf->exists() ? "deleted" : "closed",
+                     lf->get_filename().c_str());
             lnav_data.ld_text_source.remove(lf);
             lnav_data.ld_log_source.remove_file(lf);
             closed_files.emplace_back(lf);
+            retval.rir_rescan_needed = true;
         }
     }
     if (!closed_files.empty()) {
@@ -309,7 +336,7 @@ rebuild_indexes(std::optional<ui_clock::time_point> deadline)
     auto result = lss.rebuild_index(deadline);
     if (result != logfile_sub_source::rebuild_result::rr_no_change) {
         size_t new_count = lss.text_line_count();
-        bool force
+        auto force
             = result == logfile_sub_source::rebuild_result::rr_full_rebuild;
 
         if ((!scroll_downs[LNV_LOG]
@@ -319,41 +346,57 @@ rebuild_indexes(std::optional<ui_clock::time_point> deadline)
             scroll_downs[LNV_LOG] = false;
         }
 
-        {
+        if (retval.rir_completed && !retval.rir_rescan_needed) {
             std::unordered_map<std::string, std::list<std::shared_ptr<logfile>>>
                 id_to_files;
-            bool reload = false;
+            auto reload = false;
 
             for (const auto& lf : lnav_data.ld_active_files.fc_files) {
+                if (lf->get_format_ptr() == nullptr) {
+                    continue;
+                }
                 id_to_files[lf->get_content_id()].push_back(lf);
             }
 
-            for (auto& pair : id_to_files) {
-                if (pair.second.size() == 1) {
+            for (auto& [name, lf] : id_to_files) {
+                if (lf.size() == 1) {
                     continue;
                 }
 
-                pair.second.sort([](const auto& left, const auto& right) {
-                    return right->get_stat().st_size < left->get_stat().st_size;
+                lf.sort([](const auto& left, const auto& right) {
+                    const auto& lst = left->get_stat();
+                    const auto& rst = right->get_stat();
+                    return rst.st_size < lst.st_size
+                        || (rst.st_size == lst.st_size
+                            && rst.st_mtime < lst.st_mtime);
                 });
 
-                auto dupe_name = pair.second.front()->get_unique_path();
-                pair.second.pop_front();
-                for_each(pair.second.begin(),
-                         pair.second.end(),
-                         [&dupe_name](auto& lf) {
-                             if (lf->mark_as_duplicate(dupe_name)) {
-                                 log_info("Hiding duplicate file: %s",
-                                          lf->get_filename().c_str());
-                                 lnav_data.ld_log_source.find_data(lf) |
-                                     [](auto ld) { ld->set_visibility(false); };
-                             }
-                         });
-                reload = true;
+                const auto& dupe_name = lf.front()->get_unique_path();
+                log_info(
+                    "Keeping duplicated file: %s; size=%lld; mtime=%d; path=%s",
+                    lf.front()->get_content_id().c_str(),
+                    lf.front()->get_stat().st_size,
+                    lf.front()->get_stat().st_mtime,
+                    lf.front()->get_filename_as_string().c_str());
+                lf.pop_front();
+                std::for_each(
+                    lf.begin(), lf.end(), [&dupe_name, &reload](auto& lf) {
+                        if (lf->mark_as_duplicate(dupe_name)) {
+                            log_info(
+                                "  Hiding copy: size=%lld; mtime=%d; path=%s",
+                                lf->get_stat().st_size,
+                                lf->get_stat().st_mtime,
+                                lf->get_filename_as_string().c_str());
+                            lnav_data.ld_log_source.find_data(lf) |
+                                [](auto ld) { ld->set_visibility(false); };
+                            reload = true;
+                        }
+                    });
             }
 
             if (reload) {
-                log_trace("text filters changed");
+                log_trace(
+                    "file visibility changed, calling text_filters_changed");
                 lss.text_filters_changed();
             }
         }
@@ -390,7 +433,9 @@ rebuild_indexes(std::optional<ui_clock::time_point> deadline)
         }
 
         auto* tss = tc->get_sub_source();
-        lnav_data.ld_filter_status_source.update_filtered(tss);
+        if (lnav_data.ld_filter_status_source.update_filtered(tss)) {
+            lnav_data.ld_status[LNS_FILTER].set_needs_update();
+        }
         if (retval.rir_changes > 0) {
             lnav_data.ld_scroll_broadcaster(tc);
         }
@@ -421,7 +466,7 @@ update_active_files(file_collection& new_files)
         return true;
     }
 
-    bool was_below_open_file_limit
+    const auto was_below_open_file_limit
         = lnav_data.ld_active_files.is_below_open_file_limit();
 
     for (const auto& lf : new_files.fc_files) {
@@ -517,7 +562,7 @@ rescan_files(bool req)
         if (!done && !(lnav_data.ld_flags & LNF_HEADLESS)) {
             lnav_data.ld_files_view.set_needs_update();
             lnav_data.ld_files_view.do_update();
-            lnav_data.ld_status_refresher();
+            lnav_data.ld_status_refresher(lnav::func::op_type::interactive);
         }
     } while (!done && lnav_data.ld_looping);
     return true;
