@@ -86,19 +86,22 @@ is_dev_null(int fd)
     return is_dev_null(fd_stat);
 }
 
-void
+size_t
 write_line_to(FILE* outfile, const attr_line_t& al)
 {
     const auto& al_attrs = al.get_attrs();
-    auto lr = find_string_attr_range(al_attrs, &SA_ORIGINAL_LINE);
+    const auto lr = find_string_attr_range(al_attrs, &SA_ORIGINAL_LINE);
 
     if (lr.empty() || !lr.is_valid() || lr.lr_start > 1) {
         // If the line is prefixed with some extra information, include that
         // in the output.  For example, the log file name or time offset.
         lnav::console::println(outfile, al);
-    } else {
-        lnav::console::println(outfile, al.subline(lr.lr_start, lr.length()));
+        return al.column_width();
     }
+    const auto sub_al = al.subline(lr.lr_start, lr.length());
+    lnav::console::println(outfile, sub_al);
+
+    return sub_al.column_width();
 }
 
 namespace lnav {
@@ -153,7 +156,9 @@ to_json(yajlpp_gen& gen, const attr_line_t& al)
                     [&](const string_fragment& sf) { elem_map.gen(sf); },
                     [&](const block_elem_t& be) { elem_map.gen(""); },
                     [&](const styling::color_unit& rgb) { elem_map.gen(""); },
-                    [&](const ui_icon_t& ic) { elem_map.gen(""); });
+                    [&](const ui_icon_t& ic) { elem_map.gen(""); },
+                    [&](const ui_command& uc) { elem_map.gen(""); },
+                    [&](const text_format_t tf) { elem_map.gen(""); });
             }
         }
     }
@@ -305,12 +310,12 @@ static const json_path_container snippet_handlers = {
         .with_children(attr_line_handlers),
 };
 
-static const json_path_handler_base::enum_value_t LEVEL_ENUM[] = {
-    {"raw", lnav::console::user_message::level::raw},
-    {"ok", lnav::console::user_message::level::ok},
-    {"info", lnav::console::user_message::level::info},
-    {"warning", lnav::console::user_message::level::warning},
-    {"error", lnav::console::user_message::level::error},
+static constexpr json_path_handler_base::enum_value_t LEVEL_ENUM[] = {
+    {"raw"_frag, lnav::console::user_message::level::raw},
+    {"ok"_frag, lnav::console::user_message::level::ok},
+    {"info"_frag, lnav::console::user_message::level::info},
+    {"warning"_frag, lnav::console::user_message::level::warning},
+    {"error"_frag, lnav::console::user_message::level::error},
 
     json_path_handler_base::ENUM_TERMINATOR,
 };
