@@ -30,6 +30,8 @@
  */
 
 #include <algorithm>
+#include <chrono>
+#include <memory>
 #include <utility>
 
 #include "log_format.hh"
@@ -73,22 +75,24 @@ public:
         return scan_no_match{"not a piper capture"};
     }
 
+    static constexpr int TIMESTAMP_SIZE = 28;
+
     void annotate(logfile* lf,
                   uint64_t line_number,
                   string_attrs_t& sa,
                   logline_value_vector& values,
                   bool annotate_module) const override
     {
-        auto lr = line_range{0, 0};
+        auto lr = line_range{0, TIMESTAMP_SIZE};
         sa.emplace_back(lr, L_TIMESTAMP.value());
         log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
     void get_subline(const logline& ll,
                      shared_buffer_ref& sbr,
-                     bool full_message) override
+                     subline_options opts) override
     {
-        this->plf_cached_line.resize(32);
+        this->plf_cached_line.resize(TIMESTAMP_SIZE);
         auto tlen = sql_strftime(this->plf_cached_line.data(),
                                  this->plf_cached_line.size(),
                                  ll.get_timeval(),
@@ -126,7 +130,7 @@ public:
         auto retval = std::make_shared<piper_log_format>(*this);
 
         retval->lf_specialized = true;
-        retval->lf_timestamp_flags |= ETF_ZONE_SET;
+        retval->lf_timestamp_flags |= ETF_ZONE_SET | ETF_MICROS_SET;
         return retval;
     }
 
@@ -192,8 +196,8 @@ public:
                        shared_buffer_ref& sbr,
                        scan_batch_context& sbc) override
     {
-        struct exttm log_time;
-        struct timeval log_tv;
+        exttm log_time;
+        timeval log_tv;
         string_fragment ts;
         std::optional<string_fragment> level;
         const char* last_pos;
@@ -220,7 +224,7 @@ public:
                                         &level))
             != nullptr)
         {
-            log_level_t level_val = log_level_t::LEVEL_UNKNOWN;
+            auto level_val = log_level_t::LEVEL_UNKNOWN;
             if (level) {
                 level_val = string2level(level->data(), level->length());
             }
@@ -257,7 +261,7 @@ public:
             }
 
             dst.emplace_back(li.li_file_range.fr_offset, log_tv, level_val);
-            return scan_match{0};
+            return scan_match{5};
         }
 
         return scan_no_match{"no patterns matched"};
@@ -301,6 +305,11 @@ public:
                     LEVEL_META, line, to_line_range(level_cap->trim()));
                 values.lvv_values.back().lv_meta.lvm_format
                     = (log_format*) this;
+
+                lr = to_line_range(level_cap->trim());
+                if (lr.lr_end != (ssize_t) line.length()) {
+                    sa.emplace_back(lr, L_LEVEL.value());
+                }
             }
         }
 
@@ -620,10 +629,11 @@ public:
             const auto& fd = this->blf_field_defs[iter.index()];
 
             if (TS == fd.fd_meta.lvm_name) {
-                string_fragment sf = *iter;
+                static const char* const TIME_FMT[] = {"%s.%f"};
+                const auto sf = *iter;
 
                 if (this->lf_date_time.scan(
-                        sf.data(), sf.length(), nullptr, &tm, tv))
+                        sf.data(), sf.length(), TIME_FMT, &tm, tv))
                 {
                     this->lf_timestamp_flags = tm.et_flags;
                     found_ts = true;
@@ -1035,7 +1045,7 @@ public:
 
     void get_subline(const logline& ll,
                      shared_buffer_ref& sbr,
-                     bool full_message) override
+                     subline_options opts) override
     {
     }
 
@@ -1221,8 +1231,129 @@ public:
         intern_string_t fs_struct_name;
     };
 
-    static const std::vector<field_def> KNOWN_FIELDS;
-    const static std::vector<field_to_struct_t> KNOWN_STRUCT_FIELDS;
+    static const std::array<field_def, 16>& get_known_fields()
+    {
+        static size_t KNOWN_FIELD_INDEX = 0;
+        static const std::array<field_def, 16> RETVAL = {
+            field_def{
+                KNOWN_FIELD_INDEX++,
+                "cs-method",
+                value_kind_t::VALUE_TEXT,
+                true,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "c-ip",
+                value_kind_t::VALUE_TEXT,
+                true,
+                false,
+                "ipaddress",
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "cs-bytes",
+                value_kind_t::VALUE_INTEGER,
+                false,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "cs-host",
+                value_kind_t::VALUE_TEXT,
+                true,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "cs-uri-stem",
+                value_kind_t::VALUE_TEXT,
+                true,
+                false,
+                "naturalnocase",
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "cs-uri-query",
+                value_kind_t::VALUE_TEXT,
+                false,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "cs-username",
+                value_kind_t::VALUE_TEXT,
+                false,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "cs-version",
+                value_kind_t::VALUE_TEXT,
+                true,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "s-ip",
+                value_kind_t::VALUE_TEXT,
+                true,
+                false,
+                "ipaddress",
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "s-port",
+                value_kind_t::VALUE_INTEGER,
+                true,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "s-computername",
+                value_kind_t::VALUE_TEXT,
+                true,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "s-sitename",
+                value_kind_t::VALUE_TEXT,
+                true,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "sc-bytes",
+                value_kind_t::VALUE_INTEGER,
+                false,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "sc-status",
+                value_kind_t::VALUE_INTEGER,
+                false,
+                true,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "sc-substatus",
+                value_kind_t::VALUE_INTEGER,
+                false,
+            },
+            {
+                KNOWN_FIELD_INDEX++,
+                "time-taken",
+                value_kind_t::VALUE_FLOAT,
+                false,
+            },
+        };
+
+        return RETVAL;
+    }
+
+    static const std::array<field_to_struct_t, 4>& get_known_struct_fields()
+    {
+        static const std::array<field_to_struct_t, 4> RETVAL = {
+            field_to_struct_t{"cs(", "cs_headers"},
+            {"sc(", "sc_headers"},
+            {"rs(", "rs_headers"},
+            {"sr(", "sr_headers"},
+        };
+
+        return RETVAL;
+    }
 
     w3c_log_format()
     {
@@ -1233,7 +1364,7 @@ public:
 
     const intern_string_t get_name() const override
     {
-        static const intern_string_t name(intern_string::lookup("w3c"));
+        static const intern_string_t name(intern_string::lookup("w3c_log"));
 
         return this->wlf_format_name.empty() ? name : this->wlf_format_name;
     }
@@ -1284,8 +1415,8 @@ public:
                     if (sbr_sf_opt) {
                         auto sbr_sf = sbr_sf_opt.value().trim();
                         date_time_scanner dts;
-                        struct exttm tm;
-                        struct timeval tv;
+                        exttm tm;
+                        timeval tv;
 
                         if (dts.scan(sbr_sf.data(),
                                      sbr_sf.length(),
@@ -1352,8 +1483,7 @@ public:
         }
 
         if (found_time) {
-            exttm tm = time_tm;
-            timeval tv;
+            auto tm = time_tm;
 
             if (found_date) {
                 tm.et_tm.tm_year = date_tm.et_tm.tm_year;
@@ -1363,7 +1493,7 @@ public:
                 tm.et_tm.tm_yday = date_tm.et_tm.tm_yday;
             }
 
-            tv = tm.to_timeval();
+            auto tv = tm.to_timeval();
             if (!this->lf_specialized) {
                 for (auto& ll : dst) {
                     ll.set_ignore(true);
@@ -1384,6 +1514,8 @@ public:
     {
         static const auto* W3C_LOG_NAME = intern_string::lookup("w3c_log");
         static const auto* X_FIELDS_NAME = intern_string::lookup("x_fields");
+        static const auto& KNOWN_FIELDS = get_known_fields();
+        static const auto& KNOWN_STRUCT_FIELDS = get_known_struct_fields();
         static auto X_FIELDS_IDX = 0;
 
         if (li.li_partial) {
@@ -1470,7 +1602,7 @@ public:
                             common_iter = emp_res.first;
                         }
                         fd.fd_root_meta = &common_iter->second;
-                    } else if (sf == "date" || sf == "time") {
+                    } else if (sf.is_one_of("date", "time")) {
                         this->wlf_field_defs.emplace_back(
                             intern_string::lookup(sf));
                         auto& fd = this->wlf_field_defs.back();
@@ -1555,9 +1687,11 @@ public:
     {
         auto& sbr = values.lvv_sbr;
         ws_separated_string ss(sbr.get_data(), sbr.length());
+        std::optional<line_range> date_lr;
+        std::optional<line_range> time_lr;
 
         for (auto iter = ss.begin(); iter != ss.end(); ++iter) {
-            string_fragment sf = *iter;
+            auto sf = *iter;
 
             if (iter.index() >= this->wlf_field_defs.size()) {
                 sa.emplace_back(line_range{sf.sf_begin, -1},
@@ -1574,6 +1708,11 @@ public:
             auto lr = line_range(sf.sf_begin, sf.sf_end);
 
             if (lr.is_valid()) {
+                if (fd.fd_meta.lvm_name == F_DATE) {
+                    date_lr = lr;
+                } else if (fd.fd_meta.lvm_name == F_TIME) {
+                    time_lr = lr;
+                }
                 values.lvv_values.emplace_back(fd.fd_meta, sbr, lr);
                 if (sf.startswith("\"")) {
                     auto& meta = values.lvv_values.back().lv_meta;
@@ -1592,6 +1731,17 @@ public:
                     = fd.fd_root_meta->lvm_user_hidden;
             }
         }
+        if (time_lr) {
+            auto ts_lr = time_lr.value();
+            if (date_lr) {
+                if (date_lr->lr_end + 1 == time_lr->lr_start) {
+                    ts_lr.lr_start = date_lr->lr_start;
+                    ts_lr.lr_end = time_lr->lr_end;
+                }
+            }
+
+            sa.emplace_back(ts_lr, L_TIMESTAMP.value());
+        }
         log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
@@ -1666,7 +1816,7 @@ public:
 
         void get_columns(std::vector<vtab_column>& cols) const override
         {
-            for (const auto& fd : KNOWN_FIELDS) {
+            for (const auto& fd : get_known_fields()) {
                 auto type_pair = log_vtab_impl::logline_value_to_sqlite_type(
                     fd.fd_meta.lvm_kind);
 
@@ -1681,7 +1831,7 @@ public:
             cols.back().with_comment(
                 "A JSON-object that contains fields that are not first-class "
                 "columns");
-            for (const auto& fs : KNOWN_STRUCT_FIELDS) {
+            for (const auto& fs : get_known_struct_fields()) {
                 cols.emplace_back(fs.fs_struct_name.to_string());
             }
         };
@@ -1691,7 +1841,7 @@ public:
         {
             this->log_vtab_impl::get_foreign_keys(keys_inout);
 
-            for (const auto& fd : KNOWN_FIELDS) {
+            for (const auto& fd : get_known_fields()) {
                 if (fd.fd_meta.lvm_identifier || fd.fd_meta.lvm_foreign_key) {
                     keys_inout.emplace(fd.fd_meta.lvm_name.to_string());
                 }
@@ -1729,7 +1879,7 @@ public:
 
     void get_subline(const logline& ll,
                      shared_buffer_ref& sbr,
-                     bool full_message) override
+                     subline_options opts) override
     {
     }
 
@@ -1744,149 +1894,40 @@ std::unordered_map<const intern_string_t, logline_value_meta>
 const intern_string_t w3c_log_format::F_DATE = intern_string::lookup("date");
 const intern_string_t w3c_log_format::F_TIME = intern_string::lookup("time");
 
-static size_t KNOWN_FIELD_INDEX = 0;
-const std::vector<w3c_log_format::field_def> w3c_log_format::KNOWN_FIELDS = {
-    {
-        KNOWN_FIELD_INDEX++,
-        "cs-method",
-        value_kind_t::VALUE_TEXT,
-        true,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "c-ip",
-        value_kind_t::VALUE_TEXT,
-        true,
-        false,
-        "ipaddress",
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "cs-bytes",
-        value_kind_t::VALUE_INTEGER,
-        false,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "cs-host",
-        value_kind_t::VALUE_TEXT,
-        true,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "cs-uri-stem",
-        value_kind_t::VALUE_TEXT,
-        true,
-        false,
-        "naturalnocase",
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "cs-uri-query",
-        value_kind_t::VALUE_TEXT,
-        false,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "cs-username",
-        value_kind_t::VALUE_TEXT,
-        false,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "cs-version",
-        value_kind_t::VALUE_TEXT,
-        true,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "s-ip",
-        value_kind_t::VALUE_TEXT,
-        true,
-        false,
-        "ipaddress",
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "s-port",
-        value_kind_t::VALUE_INTEGER,
-        true,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "s-computername",
-        value_kind_t::VALUE_TEXT,
-        true,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "s-sitename",
-        value_kind_t::VALUE_TEXT,
-        true,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "sc-bytes",
-        value_kind_t::VALUE_INTEGER,
-        false,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "sc-status",
-        value_kind_t::VALUE_INTEGER,
-        false,
-        true,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "sc-substatus",
-        value_kind_t::VALUE_INTEGER,
-        false,
-    },
-    {
-        KNOWN_FIELD_INDEX++,
-        "time-taken",
-        value_kind_t::VALUE_FLOAT,
-        false,
-    },
-};
-
-const std::vector<w3c_log_format::field_to_struct_t>
-    w3c_log_format::KNOWN_STRUCT_FIELDS = {
-        {"cs(", "cs_headers"},
-        {"sc(", "sc_headers"},
-        {"rs(", "rs_headers"},
-        {"sr(", "sr_headers"},
-};
-
 struct logfmt_pair_handler {
     explicit logfmt_pair_handler(date_time_scanner& dts) : lph_dt_scanner(dts)
     {
     }
 
-    bool process_value(const string_fragment& value_frag)
+    log_format::scan_result_t process_value(const string_fragment& value_frag)
     {
-        if (this->lph_key_frag == "time" || this->lph_key_frag == "ts") {
+        if (this->lph_key_frag.is_one_of(
+                "timestamp"_frag, "time"_frag, "ts"_frag, "t"_frag))
+        {
             if (!this->lph_dt_scanner.scan(value_frag.data(),
                                            value_frag.length(),
                                            nullptr,
                                            &this->lph_time_tm,
                                            this->lph_tv))
             {
-                return false;
+                return log_format::scan_no_match{
+                    "timestamp value did not parse correctly"};
             }
+            char buf[1024];
+            this->lph_dt_scanner.ftime(
+                buf, sizeof(buf), nullptr, this->lph_time_tm);
             this->lph_found_time = true;
-        } else if (this->lph_key_frag == "level") {
+        } else if (this->lph_key_frag.is_one_of("level"_frag, "lvl"_frag)) {
             this->lph_level
                 = string2level(value_frag.data(), value_frag.length());
         }
-        return true;
+        return log_format::scan_match{};
     }
 
     date_time_scanner& lph_dt_scanner;
     bool lph_found_time{false};
-    struct exttm lph_time_tm{};
-    struct timeval lph_tv{0, 0};
+    exttm lph_time_tm;
+    timeval lph_tv{0, 0};
     log_level_t lph_level{log_level_t::LEVEL_INFO};
     string_fragment lph_key_frag{""};
 };
@@ -1947,24 +1988,30 @@ public:
         while (!done) {
             auto parse_result = p.step();
 
-            done = parse_result.match(
-                [](const logfmt::parser::end_of_input&) { return true; },
-                [&lph](const logfmt::parser::kvpair& kvp) {
+            auto value_res = parse_result.match(
+                [&done](const logfmt::parser::end_of_input&) -> scan_result_t {
+                    done = true;
+                    return scan_match{};
+                },
+                [&lph](const logfmt::parser::kvpair& kvp) -> scan_result_t {
                     lph.lph_key_frag = kvp.first;
 
                     return kvp.second.match(
-                        [](const logfmt::parser::bool_value& bv) {
-                            return false;
-                        },
-                        [&lph](const logfmt::parser::float_value& fv) {
+                        [](const logfmt::parser::bool_value& bv)
+                            -> scan_result_t { return scan_match{}; },
+                        [&lph](const logfmt::parser::float_value& fv)
+                            -> scan_result_t {
                             return lph.process_value(fv.fv_str_value);
                         },
-                        [&lph](const logfmt::parser::int_value& iv) {
+                        [&lph](const logfmt::parser::int_value& iv)
+                            -> scan_result_t {
                             return lph.process_value(iv.iv_str_value);
                         },
-                        [&lph](const logfmt::parser::quoted_value& qv) {
+                        [&lph](const logfmt::parser::quoted_value& qv)
+                            -> scan_result_t {
                             auto_mem<yajl_handle_t> handle(yajl_free);
                             yajl_callbacks cb;
+                            scan_result_t retval;
 
                             memset(&cb, 0, sizeof(cb));
                             handle = yajl_alloc(&cb, nullptr, &lph);
@@ -1975,7 +2022,8 @@ public:
                                 auto& lph = *((logfmt_pair_handler*) ctx);
                                 string_fragment value_frag{str, 0, (int) len};
 
-                                return lph.process_value(value_frag);
+                                auto value_res = lph.process_value(value_frag);
+                                return value_res.is<scan_match>();
                             };
 
                             if (yajl_parse(
@@ -1996,19 +2044,25 @@ public:
                                 return lph.process_value(unq_frag);
                             }
 
-                            return false;
+                            return scan_match{};
                         },
-                        [&lph](const logfmt::parser::unquoted_value& uv) {
+                        [&lph](const logfmt::parser::unquoted_value& uv)
+                            -> scan_result_t {
                             return lph.process_value(uv.uv_value);
                         });
                 },
-                [](const logfmt::parser::error& err) {
+                [](const logfmt::parser::error& err) -> scan_result_t {
                     // log_error("logfmt parse error: %s", err.e_msg.c_str());
-                    return true;
+                    return scan_no_match{};
                 });
+            if (value_res.is<scan_no_match>()) {
+                retval = value_res;
+                done = true;
+            }
         }
 
         if (lph.lph_found_time) {
+            this->lf_timestamp_flags = lph.lph_time_tm.et_flags;
             dst.emplace_back(
                 li.li_file_range.fr_offset, lph.lph_tv, lph.lph_level);
             retval = scan_match{2000};
@@ -2023,18 +2077,21 @@ public:
                   logline_value_vector& values,
                   bool annotate_module) const override
     {
-        static const auto FIELDS_NAME = intern_string::lookup("fields");
+        static const intern_string_t FIELDS_NAME
+            = intern_string::lookup("fields");
 
         auto& sbr = values.lvv_sbr;
         auto p = logfmt::parser(sbr.to_string_fragment());
-        bool done = false;
+        auto done = false;
+        auto found_body = false;
 
         while (!done) {
             auto parse_result = p.step();
 
             done = parse_result.match(
                 [](const logfmt::parser::end_of_input&) { return true; },
-                [this, &sa, &values](const logfmt::parser::kvpair& kvp) {
+                [this, &sa, &values, &found_body](
+                    const logfmt::parser::kvpair& kvp) {
                     auto value_frag = kvp.second.match(
                         [this, &kvp, &values](
                             const logfmt::parser::bool_value& bv) {
@@ -2087,11 +2144,19 @@ public:
                     auto value_lr
                         = line_range{value_frag.sf_begin, value_frag.sf_end};
 
-                    if (kvp.first == "time" || kvp.first == "ts") {
+                    auto known_field = false;
+                    if (kvp.first.is_one_of(
+                            "timestamp"_frag, "time"_frag, "ts"_frag, "t"_frag))
+                    {
                         sa.emplace_back(value_lr, L_TIMESTAMP.value());
-                    } else if (kvp.first == "level") {
-                    } else if (kvp.first == "msg") {
+                        known_field = true;
+                    } else if (kvp.first.is_one_of("level"_frag, "lvl"_frag)) {
+                        sa.emplace_back(value_lr, L_LEVEL.value());
+                        known_field = true;
+                    } else if (kvp.first.is_one_of("msg"_frag, "message"_frag))
+                    {
                         sa.emplace_back(value_lr, SA_BODY.value());
+                        found_body = true;
                     } else if (kvp.second.is<logfmt::parser::quoted_value>()
                                || kvp.second
                                       .is<logfmt::parser::unquoted_value>())
@@ -2108,7 +2173,15 @@ public:
                                   .with_struct_name(FIELDS_NAME);
                         values.lvv_values.emplace_back(lvm, value_frag);
                     }
-
+                    if (known_field) {
+                        auto key_with_eq = kvp.first;
+                        key_with_eq.sf_end += 1;
+                        sa.emplace_back(to_line_range(key_with_eq),
+                                        SA_REPLACED.value());
+                    } else {
+                        sa.emplace_back(to_line_range(kvp.first),
+                                        VC_ROLE.value(role_t::VCR_OBJECT_KEY));
+                    }
                     return false;
                 },
                 [line_number, &sbr](const logfmt::parser::error& err) {
@@ -2120,6 +2193,11 @@ public:
                 });
         }
 
+        if (!found_body) {
+            sa.emplace_back(line_range::empty_at(sbr.length()),
+                            SA_BODY.value());
+        }
+
         log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
