@@ -43,7 +43,7 @@ match_data::to_string() const
 
     if (this->get_count() == 1) {
         auto cap = (*this)[0];
-        retval.append(cap->data(), cap->length());
+        retval += cap.value();
     } else {
         for (size_t lpc = 1; lpc < this->get_count(); lpc++) {
             auto cap = (*this)[lpc];
@@ -52,7 +52,7 @@ match_data::to_string() const
                 continue;
             }
 
-            retval.append(cap->data(), cap->length());
+            retval += cap.value();
         }
     }
 
@@ -173,6 +173,7 @@ code::get_capture_count() const
 std::vector<string_fragment>
 code::get_captures() const
 {
+    int class_start = 0;
     bool in_class = false, in_escape = false, in_literal = false;
     auto pat_frag = string_fragment::from_str(this->p_pattern);
     std::vector<string_fragment> cap_in_progress;
@@ -185,7 +186,9 @@ code::get_captures() const
                 in_literal = true;
             }
         } else if (in_class) {
-            if (this->p_pattern[lpc] == ']') {
+            if (lpc == class_start + 1 && this->p_pattern[lpc] == '^') {
+                class_start = lpc;
+            } else if (lpc > class_start + 1 && this->p_pattern[lpc] == ']') {
                 in_class = false;
             }
             if (this->p_pattern[lpc] == '\\') {
@@ -204,6 +207,7 @@ code::get_captures() const
                     break;
                 case '[':
                     in_class = true;
+                    class_start = lpc;
                     break;
                 case '(':
                     cap_in_progress.emplace_back(pat_frag.sub_range(lpc, lpc));
@@ -287,7 +291,7 @@ code::replace(string_fragment str, const char* repl) const
         remaining = find_res->f_remaining;
         bool in_escape = false;
 
-        retval.append(str.data(), start, (all.sf_begin - start));
+        retval.append(&str.data()[start],(all.sf_begin - start));
         start = all.sf_end;
         for (int lpc = 0; repl[lpc]; lpc++) {
             auto ch = repl[lpc];
@@ -299,7 +303,7 @@ code::replace(string_fragment str, const char* repl) const
                     if (capture_index < md.get_count()) {
                         auto cap = md[capture_index];
                         if (cap) {
-                            retval.append(cap->data(), cap->length());
+                            retval += cap.value();
                         }
                     } else if (capture_index > this->get_capture_count()) {
                         retval.push_back('\\');
@@ -325,7 +329,7 @@ code::replace(string_fragment str, const char* repl) const
         }
     }
     if (remaining.is_valid()) {
-        retval.append(remaining.data(), 0, remaining.length());
+        retval += remaining;
     }
 
     return retval;
