@@ -39,6 +39,7 @@
 #include "shlex.hh"
 #include "sql_help.hh"
 #include "sql_util.hh"
+#include "textfile_highlighters.hh"
 #include "view_curses.hh"
 
 static bool
@@ -87,7 +88,7 @@ find_matching_bracket(
     }
 
     if (line[x] == left && is_bracket(line, x, is_lit)) {
-        for (size_t lpc = x + 1; lpc < sub.lr_end; lpc++) {
+        for (auto lpc = x + 1; lpc < sub.lr_end; lpc++) {
             if (line[lpc] == left && is_bracket(line, lpc, is_lit)) {
                 depth += 1;
             } else if (line[lpc] == right && is_bracket(line, lpc, is_lit)) {
@@ -110,7 +111,7 @@ find_matching_bracket(
 
     depth = 0;
 
-    for (size_t lpc = sub.lr_start; lpc < sub.lr_end; lpc++) {
+    for (auto lpc = sub.lr_start; lpc < sub.lr_end; lpc++) {
         if (line[lpc] == left && is_bracket(line, lpc, is_lit)) {
             depth += 1;
             if (!first_left) {
@@ -149,17 +150,29 @@ readline_regex_highlighter(attr_line_t& al, std::optional<int> x)
         al, x, line_range{1, (int) al.get_string().size()});
 }
 
+static highlight_map_t
+get_hl_map()
+{
+    highlight_map_t retval;
+
+    setup_highlights(retval);
+    return retval;
+}
+
 void
 readline_command_highlighter_int(attr_line_t& al,
                                  std::optional<int> x,
                                  line_range sub)
 {
+    static const auto TEXT_HIGHLIGHTERS = get_hl_map();
     static const auto RE_PREFIXES = lnav::pcre2pp::code::from_const(
         R"(^:(filter-in|filter-out|delete-filter|enable-filter|disable-filter|highlight|clear-highlight|create-search-table\s+[^\s]+\s+))");
     static const auto SH_PREFIXES = lnav::pcre2pp::code::from_const(
         "^:(eval|open|append-to|write-to|write-csv-to|write-json-to)");
     static const auto SQL_PREFIXES
         = lnav::pcre2pp::code::from_const("^:(filter-expr|mark-expr)");
+    static const auto MD_PREFIXES
+        = lnav::pcre2pp::code::from_const("^:comment");
     static const auto IDENT_PREFIXES
         = lnav::pcre2pp::code::from_const("^:(tag|untag|delete-tags)");
     static const auto COLOR_PREFIXES
@@ -176,8 +189,10 @@ readline_command_highlighter_int(attr_line_t& al,
     ws_index = line.find(' ', sub.lr_start);
     auto command = line.substr(sub.lr_start, ws_index);
     if (ws_index != std::string::npos) {
-        alb.overlay_attr(line_range(sub.lr_start + 1, ws_index),
-                         VC_ROLE.value(role_t::VCR_KEYWORD));
+        auto has_prefix = al.al_string[sub.lr_start] == ':';
+        alb.overlay_attr(
+            line_range(sub.lr_start + (has_prefix ? 1 : 0), ws_index),
+            VC_ROLE.value(role_t::VCR_KEYWORD));
 
         if (RE_PREFIXES.find_in(in_frag).ignore_error()) {
             lnav::snippets::regex_highlighter(
@@ -188,8 +203,11 @@ readline_command_highlighter_int(attr_line_t& al,
                 al, x, line_range{(int) ws_index, sub.lr_end});
         }
         if (SQL_PREFIXES.find_in(in_frag).ignore_error()) {
-            readline_sqlite_highlighter_int(
-                al, x, line_range{(int) ws_index, sub.lr_end});
+            readline_sql_highlighter_int(
+                al,
+                lnav::sql::dialect::sqlite,
+                x,
+                line_range{(int) ws_index, sub.lr_end});
         }
     }
     if (COLOR_PREFIXES.find_in(in_frag).ignore_error()) {
@@ -209,7 +227,7 @@ readline_command_highlighter_int(attr_line_t& al,
     if (IDENT_PREFIXES.find_in(in_frag).ignore_error()
         && ws_index != std::string::npos)
     {
-        size_t start = ws_index, last;
+        ssize_t start = ws_index, last;
 
         do {
             for (; start < sub.length() && isspace(line[start]); start++)
@@ -236,6 +254,19 @@ readline_command_highlighter_int(attr_line_t& al,
             start = last;
         } while (start < sub.length());
     }
+    if (MD_PREFIXES.find_in(in_frag).ignore_error()) {
+        for (const auto& [src, hl] : TEXT_HIGHLIGHTERS) {
+            if (!hl.applies_to_format(text_format_t::TF_MARKDOWN)) {
+                continue;
+            }
+
+            hl.annotate(al,
+                        line_range{
+                            (int) (sub.lr_start + command.length()),
+                            sub.lr_end,
+                        });
+        }
+    }
 }
 
 void
@@ -246,9 +277,10 @@ readline_command_highlighter(attr_line_t& al, std::optional<int> x)
 }
 
 void
-readline_sqlite_highlighter_int(attr_line_t& al,
-                                std::optional<int> x,
-                                line_range sub)
+readline_sql_highlighter_int(attr_line_t& al,
+                             lnav::sql::dialect dia,
+                             std::optional<int> x,
+                             line_range sub)
 {
     static const char* brackets[] = {
         "[]",
@@ -261,10 +293,10 @@ readline_sqlite_highlighter_int(attr_line_t& al,
 
     auto anno_sql = al.subline(sub.lr_start, sub.length());
     anno_sql.get_attrs().clear();
-    annotate_sql_statement(anno_sql);
+    annotate_sql_statement(anno_sql, dia);
 
     for (const auto& attr : anno_sql.al_attrs) {
-        line_range lr{
+        auto lr = line_range{
             sub.lr_start + attr.sa_range.lr_start,
             sub.lr_start + attr.sa_range.lr_end,
         };
@@ -275,10 +307,11 @@ readline_sqlite_highlighter_int(attr_line_t& al,
         {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_KEYWORD));
         } else if (attr.sa_type == &SQL_IDENTIFIER_ATTR
-                   || attr.sa_type == &lnav::sql::PRQL_IDENTIFIER_ATTR)
+                   || attr.sa_type == &lnav::sql::PRQL_FQID_ATTR)
         {
-            if (x && !attr.sa_range.contains(x.value())
-                && attr.sa_range.lr_end != x.value())
+            if (!x
+                || (x && !attr.sa_range.contains(x.value())
+                    && attr.sa_range.lr_end != x.value()))
             {
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_IDENTIFIER));
             }
@@ -290,6 +323,15 @@ readline_sqlite_highlighter_int(attr_line_t& al,
                    || attr.sa_type == &lnav::sql::PRQL_NUMBER_ATTR)
         {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_NUMBER));
+        } else if (attr.sa_type == &SQL_HEX_LIT_ATTR) {
+            if (lr.length() > 1 && al.al_string[lr.lr_end - 1] == '\'') {
+                alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_STRING));
+            } else {
+                alb.overlay_attr_for_char(
+                    lr.lr_start, VC_STYLE.value(text_attrs::with_reverse()));
+                alb.overlay_attr_for_char(lr.lr_start,
+                                          VC_ROLE.value(role_t::VCR_ERROR));
+            }
         } else if (attr.sa_type == &SQL_STRING_ATTR) {
             if (lr.length() > 1 && al.al_string[lr.lr_end - 1] == '\'') {
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_STRING));
@@ -319,10 +361,12 @@ readline_sqlite_highlighter_int(attr_line_t& al,
 }
 
 void
-readline_sqlite_highlighter(attr_line_t& al, std::optional<int> x)
+readline_sql_highlighter(attr_line_t& al,
+                         lnav::sql::dialect dia,
+                         std::optional<int> x)
 {
-    readline_sqlite_highlighter_int(
-        al, x, line_range{0, (int) al.get_string().length()});
+    readline_sql_highlighter_int(
+        al, dia, x, line_range{0, (int) al.get_string().length()});
 }
 
 void
@@ -430,12 +474,13 @@ readline_lnav_highlighter_int(attr_line_t& al,
             readline_command_highlighter_int(al, x, sub);
             break;
         case ';':
-            readline_sqlite_highlighter_int(al,
-                                            x,
-                                            line_range{
-                                                sub.lr_start + 1,
-                                                sub.lr_end,
-                                            });
+            readline_sql_highlighter_int(al,
+                                         lnav::sql::dialect::sqlite,
+                                         x,
+                                         line_range{
+                                             sub.lr_start + 1,
+                                             sub.lr_end,
+                                         });
             break;
         case '|':
             break;
@@ -522,19 +567,23 @@ readline_lnav_highlighter(attr_line_t& al, std::optional<int> x)
 }
 
 void
-highlight_syntax(text_format_t tf, attr_line_t& al)
+highlight_syntax(text_format_t tf, attr_line_t& al, std::optional<int> x)
 {
-     switch (tf) {
+    switch (tf) {
         case text_format_t::TF_SQL: {
-            readline_sqlite_highlighter(al, std::nullopt);
+            readline_sql_highlighter(al, lnav::sql::dialect::sqlite, x);
             break;
         }
-         case text_format_t::TF_SHELL_SCRIPT: {
-            readline_shlex_highlighter(al, std::nullopt);
+        case text_format_t::TF_PCRE: {
+            readline_regex_highlighter(al, x);
             break;
         }
-         case text_format_t::TF_LNAV_SCRIPT: {
-            readline_lnav_highlighter(al, std::nullopt);
+        case text_format_t::TF_SHELL_SCRIPT: {
+            readline_shlex_highlighter(al, x);
+            break;
+        }
+        case text_format_t::TF_LNAV_SCRIPT: {
+            readline_lnav_highlighter(al, x);
             break;
         }
         default:
