@@ -249,6 +249,12 @@ SELECT content_id, format, time_offset FROM lnav_file
 # The following field visibility commands were run by the
 # original user during this session.  Uncomment them if
 # desired.
+)";
+
+    static const char* HIGHLIGHT_HEADER = R"(
+# The following highlight commands were run by the
+# original user during this session.  Uncomment them if
+# desired.
 )";
 
     static constexpr char FILE_FOOTER[] = R"(
@@ -279,6 +285,7 @@ SELECT content_id, format, time_offset FROM lnav_file
         const auto& open_opts = name_pair.second;
 
         if (!open_opts.loo_is_visible || !open_opts.loo_include_in_session
+            || !open_opts.loo_filename.empty()
             || open_opts.loo_source != logfile_name_source::USER)
         {
             continue;
@@ -298,6 +305,19 @@ SELECT content_id, format, time_offset FROM lnav_file
             raw_files.insert(file_path_str);
         }
     }
+    for (const auto& lf : lnav_data.ld_active_files.fc_files) {
+        if (lf->is_valid_filename()) {
+            continue;
+        }
+        if (!lf->get_open_options().loo_include_in_session) {
+            continue;
+        }
+
+        const auto& open_options = lf->get_open_options();
+        if (open_options.loo_piper) {
+            raw_files.emplace(open_options.loo_piper->get_url());
+        }
+    }
     for (const auto& file_path_str : raw_files) {
         fmt::print(
             file, FMT_STRING(":open {}\n"), replace_home_dir(file_path_str));
@@ -469,7 +489,45 @@ SELECT content_id, format, time_offset FROM lnav_file
             }
         }
 
+        {
+            std::vector<std::string> hl_cmds;
+            const auto& hm = tc.get_highlights();
+            for (const auto& hl_pair : hm) {
+                if (hl_pair.first.first != highlight_source_t::INTERACTIVE) {
+                    continue;
+                }
+
+                auto cmd = fmt::format(FMT_STRING("# :highlight {}\n"),
+                                       hl_pair.second.h_regex->get_pattern());
+                hl_cmds.emplace_back(cmd);
+            }
+
+            if (!hl_cmds.empty()) {
+                fmt::print(file, FMT_STRING("{}"), HIGHLIGHT_HEADER);
+                hl_cmds
+                    | lnav::itertools::for_each([&file](const auto& cmd) {
+                          fmt::print(file, FMT_STRING("{}"), cmd);
+                      });
+                fmt::println(file, FMT_STRING(""));
+            }
+        }
+
         auto* tss = tc.get_sub_source();
+        auto* ttt = dynamic_cast<text_time_translator*>(tss);
+        if (ttt != nullptr) {
+            char tsbuf[128];
+            auto min_time_opt = ttt->get_min_row_time();
+            if (min_time_opt) {
+                sql_strftime(tsbuf, sizeof(tsbuf), min_time_opt.value(), 'T');
+                fmt::print(file, FMT_STRING(":hide-lines-before {}\n"), tsbuf);
+            }
+            auto max_time_opt = ttt->get_max_row_time();
+            if (max_time_opt) {
+                sql_strftime(tsbuf, sizeof(tsbuf), max_time_opt.value(), 'T');
+                fmt::print(file, FMT_STRING(":hide-lines-after {}\n"), tsbuf);
+            }
+        }
+
         auto* lss = dynamic_cast<logfile_sub_source*>(tss);
         if (lss != nullptr) {
             auto min_level = lss->get_min_log_level();
@@ -480,17 +538,6 @@ SELECT content_id, format, time_offset FROM lnav_file
                            level_names[min_level]);
             }
 
-            char tsbuf[128];
-            auto min_time_opt = lss->get_min_log_time();
-            if (min_time_opt) {
-                sql_strftime(tsbuf, sizeof(tsbuf), min_time_opt.value(), 'T');
-                fmt::print(file, FMT_STRING(":hide-lines-before {}\n"), tsbuf);
-            }
-            auto max_time_opt = lss->get_max_log_time();
-            if (max_time_opt) {
-                sql_strftime(tsbuf, sizeof(tsbuf), max_time_opt.value(), 'T');
-                fmt::print(file, FMT_STRING(":hide-lines-after {}\n"), tsbuf);
-            }
             for (const auto& ld : *lss) {
                 if (ld->is_visible()) {
                     continue;
