@@ -27,12 +27,15 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <memory>
+#include <optional>
+
 #include "spectro_impls.hh"
 
 #include "base/itertools.hh"
 #include "lnav.hh"
 #include "logfile_sub_source.hh"
-#include "scn/scan.h"
+#include "textview_curses.hh"
 
 using namespace lnav::roles::literals;
 
@@ -41,6 +44,8 @@ class filtered_sub_source
     , public text_time_translator
     , public list_overlay_source {
 public:
+    bool empty() const override { return this->fss_delegate->empty(); }
+
     size_t text_line_count() override { return this->fss_lines.size(); }
 
     line_info text_value_for_line(textview_curses& tc,
@@ -295,11 +300,11 @@ log_spectro_value_source::spectro_mark(textview_curses& tc,
     logline_value_vector values;
     string_attrs_t sa;
 
-    for (vis_line_t curr_line = begin_line; curr_line < end_line; ++curr_line) {
-        content_line_t cl = lss.at(curr_line);
-        std::shared_ptr<logfile> lf = lss.find(cl);
+    for (auto curr_line = begin_line; curr_line < end_line; ++curr_line) {
+        auto cl = lss.at(curr_line);
+        const auto lf = lss.find(cl);
         auto ll = lf->begin() + cl;
-        auto format = lf->get_format();
+        const auto* format = lf->get_format_ptr();
 
         if (!ll->is_message()) {
             continue;
