@@ -119,9 +119,9 @@ sql_cmd_read(exec_context& ec,
         auto split_err = split_args_res.unwrapErr();
         auto um
             = lnav::console::user_message::error("unable to parse file name")
-                  .with_reason(split_err.te_msg)
+                  .with_reason(split_err.se_error.te_msg)
                   .with_snippet(lnav::console::snippet::from(
-                      SRC, lexer.to_attr_line(split_err)))
+                      SRC, lexer.to_attr_line(split_err.se_error)))
                   .move();
 
         return Err(um);
@@ -264,12 +264,13 @@ prql_cmd_from_prompt(exec_context& ec, const std::string& cmdline)
 
     auto* tc = *lnav_data.ld_view_stack.top();
     auto* lss = dynamic_cast<logfile_sub_source*>(tc->get_sub_source());
+    auto sel = tc->get_selection();
 
-    if (lss == nullptr || lss->text_line_count() == 0) {
+    if (!sel || lss == nullptr || lss->text_line_count() == 0) {
         return {};
     }
 
-    auto line_pair = lss->find_line_with_file(lss->at(tc->get_selection()));
+    auto line_pair = lss->find_line_with_file(lss->at(sel.value()));
     if (!line_pair) {
         return {};
     }
@@ -612,11 +613,15 @@ static readline_context::command_t sql_commands[] = {
             .prql_transform()
             .with_tags({"prql"})
             .with_summary("PRQL transform to add columns from another table")
-            .with_parameter(
-                help_text{"side", "Specifies which rows to include"}
-                    .with_enum_values({"inner", "left", "right", "full"})
-                    .with_default_value("inner")
-                    .optional())
+            .with_parameter(help_text{"side", "Specifies which rows to include"}
+                                .with_enum_values({
+                                    "inner"_frag,
+                                    "left"_frag,
+                                    "right"_frag,
+                                    "full"_frag,
+                                })
+                                .with_default_value("inner")
+                                .optional())
             .with_parameter(
                 {"table", "The other table to join with the current rows"})
             .with_parameter(
