@@ -29,6 +29,7 @@
  * @file sql_util.cc
  */
 
+#include <array>
 #include <regex>
 #include <vector>
 
@@ -39,6 +40,7 @@
 #include <string.h>
 
 #include "base/auto_mem.hh"
+#include "base/from_trait.hh"
 #include "base/injector.hh"
 #include "base/lnav_log.hh"
 #include "base/string_util.hh"
@@ -54,10 +56,577 @@
 
 using namespace lnav::roles::literals;
 
+constexpr std::array<const char*, 564> pg_sql_keywords = {
+    "ABORT",
+    "ABS",
+    "ACCESSIBLE",
+    "ACTION",
+    "ADD",
+    "ADMIN",
+    "AFTER",
+    "AGGREGATE",
+    "ALIAS",
+    "ALL",
+    "ALLOCATE",
+    "ALTER",
+    "ALWAYS",
+    "ANALYSE",
+    "ANALYZE",
+    "AND",
+    "ANY",
+    "ARE",
+    "ARRAY",
+    "AS",
+    "ASC",
+    "ASENSITIVE",
+    "ASSERTION",
+    "ASSIGNMENT",
+    "ASYMMETRIC",
+    "AT",
+    "ATOMIC",
+    "ATTRIBUTE",
+    "ATTRIBUTES",
+    "AUDIT",
+    "AUTHORIZATION",
+    "AVG",
+    "BEFORE",
+    "BEGIN",
+    "BERNOULLI",
+    "BETWEEN",
+    "BIGINT",
+    "BINARY",
+    "BIT",
+    "BLOB",
+    "BOOLEAN",
+    "BOTH",
+    "BREADTH",
+    "BY",
+    "CALL",
+    "CALLED",
+    "CARDINALITY",
+    "CASCADE",
+    "CASCADED",
+    "CASE",
+    "CAST",
+    "CATALOG",
+    "CATALOG_NAME",
+    "CEIL",
+    "CEILING",
+    "CHAR",
+    "CHARACTER",
+    "CHARACTER_LENGTH",
+    "CHARACTER_SET_CATALOG",
+    "CHARACTER_SET_NAME",
+    "CHARACTER_SET_SCHEMA",
+    "CHAR_LENGTH",
+    "CHECK",
+    "CLASS",
+    "CLASS_ORIGIN",
+    "CLOB",
+    "CLOSE",
+    "COALESCE",
+    "COLLATE",
+    "COLLATION",
+    "COLLATION_CATALOG",
+    "COLLATION_NAME",
+    "COLLATION_SCHEMA",
+    "COLLECT",
+    "COLUMN",
+    "COLUMN_NAME",
+    "COMMAND_FUNCTION",
+    "COMMAND_FUNCTION_CODE",
+    "COMMIT",
+    "COMMITTED",
+    "CONDITION",
+    "CONDITION_NUMBER",
+    "CONNECT",
+    "CONNECTION",
+    "CONNECTION_NAME",
+    "CONSTRAINT",
+    "CONSTRAINT_CATALOG",
+    "CONSTRAINT_NAME",
+    "CONSTRAINT_SCHEMA",
+    "CONSTRUCTORS",
+    "CONTAINS",
+    "CONTINUE",
+    "CONVERT",
+    "CORR",
+    "CORRESPONDING",
+    "COUNT",
+    "COVAR_POP",
+    "COVAR_SAMP",
+    "CREATE",
+    "CROSS",
+    "CUBE",
+    "CUME_DIST",
+    "CURRENT",
+    "CURRENT_CATALOG",
+    "CURRENT_DATE",
+    "CURRENT_DEFAULT_TRANSFORM_GROUP",
+    "CURRENT_PATH",
+    "CURRENT_ROLE",
+    "CURRENT_ROW",
+    "CURRENT_SCHEMA",
+    "CURRENT_TIME",
+    "CURRENT_TIMESTAMP",
+    "CURRENT_TRANSFORM_GROUP_FOR_TYPE",
+    "CURRENT_USER",
+    "CURSOR",
+    "CURSOR_NAME",
+    "CYCLE",
+    "DATA",
+    "DATABASE",
+    "DATE",
+    "DATETIME_INTERVAL_CODE",
+    "DATETIME_INTERVAL_PRECISION",
+    "DAY",
+    "DEALLOCATE",
+    "DEC",
+    "DECIMAL",
+    "DECLARE",
+    "DEFAULT",
+    "DEFAULTS",
+    "DEFERRABLE",
+    "DEFERRED",
+    "DEFINED",
+    "DEFINER",
+    "DEGREE",
+    "DELETE",
+    "DENSE_RANK",
+    "DEPTH",
+    "DEREF",
+    "DESC",
+    "DESCRIBE",
+    "DESCRIPTOR",
+    "DETERMINISTIC",
+    "DIAGNOSTICS",
+    "DISCONNECT",
+    "DISPATCH",
+    "DISTINCT",
+    "DO",
+    "DOMAIN",
+    "DOUBLE",
+    "DROP",
+    "DYNAMIC",
+    "DYNAMIC_FUNCTION",
+    "DYNAMIC_FUNCTION_CODE",
+    "EACH",
+    "ELEMENT",
+    "ELSE",
+    "END",
+    "END_FRAME",
+    "END_OF_CHAIN",
+    "EQUALS",
+    "ESCAPE",
+    "EVERY",
+    "EXCEPT",
+    "EXCEPTION",
+    "EXCLUDE",
+    "EXCLUDING",
+    "EXEC",
+    "EXECUTE",
+    "EXISTS",
+    "EXP",
+    "EXPLAIN",
+    "EXTEND",
+    "EXTERNAL",
+    "EXTRACT",
+    "FALSE",
+    "FAMILY",
+    "FETCH",
+    "FILE",
+    "FINAL",
+    "FIRST",
+    "FIRST_VALUE",
+    "FLAG",
+    "FLOAT",
+    "FLOOR",
+    "FOLLOWING",
+    "FOR",
+    "FOREIGN",
+    "FORTRAN",
+    "FOUND",
+    "FRAME_ROW",
+    "FREE",
+    "FREEZE",
+    "FROM",
+    "FULL",
+    "FUNCTION",
+    "FUSION",
+    "GENERAL",
+    "GENERATED",
+    "GET",
+    "GLOBAL",
+    "GO",
+    "GOTO",
+    "GRANT",
+    "GRANTED",
+    "GROUP",
+    "GROUPING",
+    "GROUPS",
+    "HAVING",
+    "HOLD",
+    "HOUR",
+    "IDENTITY",
+    "IF",
+    "IGNORE",
+    "ILIKE",
+    "IMMEDIATE",
+    "IMMEDIATELY",
+    "IMPLEMENTATION",
+    "IMPLICIT",
+    "IN",
+    "INCLUDING",
+    "INCREMENT",
+    "INDICATOR",
+    "INHERIT",
+    "INITIALLY",
+    "INNER",
+    "INOUT",
+    "INPUT",
+    "INSENSITIVE",
+    "INSERT",
+    "INSTANCE",
+    "INSTANTIABLE",
+    "INT",
+    "INTEGER",
+    "INTEGRITY",
+    "INTERSECT",
+    "INTERVAL",
+    "INTO",
+    "INVOKER",
+    "IS",
+    "ISNULL",
+    "ISOLATION",
+    "JAVA",
+    "JOIN",
+    "KEY",
+    "KEY_MEMBER",
+    "KEY_TYPE",
+    "LAG",
+    "LANGUAGE",
+    "LARGE",
+    "LAST",
+    "LAST_VALUE",
+    "LATERAL",
+    "LEAD",
+    "LEADING",
+    "LEFT",
+    "LENGTH",
+    "LEVEL",
+    "LIKE",
+    "LIKE_REGEX",
+    "LIMIT",
+    "LN",
+    "LOCAL",
+    "LOCALTIME",
+    "LOCALTIMESTAMP",
+    "LOCATOR",
+    "LOWER",
+    "MAP",
+    "MATCH",
+    "MATCHED",
+    "MAX",
+    "MAX_CARDINALITY",
+    "MEMBER",
+    "MERGE",
+    "MESSAGE_LENGTH",
+    "MESSAGE_OCTET_LENGTH",
+    "MESSAGE_TEXT",
+    "METHOD",
+    "MIN",
+    "MINUTE",
+    "MOD",
+    "MODIFIES",
+    "MODULE",
+    "MONTH",
+    "MULTISET",
+    "NAME",
+    "NAMES",
+    "NATIONAL",
+    "NATURAL",
+    "NCHAR",
+    "NCLOB",
+    "NEW",
+    "NEXT",
+    "NO",
+    "NONE",
+    "NORMALIZE",
+    "NORMALIZED",
+    "NOT",
+    "NOTNULL",
+    "NTH_VALUE",
+    "NULL",
+    "NULLABLE",
+    "NULLIF",
+    "NULLS",
+    "NUMBER",
+    "NUMERIC",
+    "OBJECT",
+    "OCTET_LENGTH",
+    "OF",
+    "OFF",
+    "OFFSET",
+    "OLD",
+    "ON",
+    "ONLY",
+    "OPEN",
+    "OPERATION",
+    "OPTION",
+    "ORDINALITY",
+    "ORDER",
+    "ORDERING",
+    "ORIENTATION",
+    "ORIGIN",
+    "OUT",
+    "OUTER",
+    "OUTPUT",
+    "OVER",
+    "OVERLAPS",
+    "OVERLAY",
+    "PAD",
+    "PARAMETER",
+    "PARAMETER_MODE",
+    "PARAMETER_NAME",
+    "PARAMETER_ORDINAL_POSITION",
+    "PARAMETER_SPECIFIC_CATALOG",
+    "PARAMETER_SPECIFIC_NAME",
+    "PARAMETER_SPECIFIC_SCHEMA",
+    "PARTIAL",
+    "PARTITION",
+    "PASCAL",
+    "PATH",
+    "PERCENT_RANK",
+    "PERCENTILE_CONT",
+    "PERCENTILE_DISC",
+    "PERIOD",
+    "PG_CATALOG",
+    "PG_CLIENT",
+    "PG_EXTENSION",
+    "PG_GET_KEYWORDS",
+    "PG_IS_TEMP_SCHEMA",
+    "PG_IS_TEMP_SCHE",
+    "PG_LAST_WAL_RECEIVE_LOCATION",
+    "PG_LAST_WAL_REPLAY_LOCATION",
+    "PG_LISTEN",
+    "PG_NOTIFY",
+    "PG_RELATION_SIZE",
+    "PG_SLEEP",
+    "PG_TABLE_SIZE",
+    "PG_TRY_ADVISORY_LOCK",
+    "PG_TRY_ADVISORY_UNLOCK",
+    "PG_TYPEOF",
+    "PG_UNLISTEN",
+    "PG_WAL_LSN_DIFF",
+    "PG_WAL_REPLAY_PAUSED",
+    "PG_XLOG_LOCATION_DIFF",
+    "PG_XLOG_REPLAY_PAUSED",
+    "PG_XLOG_REPLAY_WAIT",
+    "PLACING",
+    "PLI",
+    "PORTION",
+    "POSITION",
+    "POWER",
+    "PRECEDES",
+    "PRECISION",
+    "PREPARE",
+    "PRESERVE",
+    "PRIMARY",
+    "PRIOR",
+    "PRIVILEGES",
+    "PROCEDURE",
+    "PUBLIC",
+    "RANGE",
+    "RANK",
+    "READ",
+    "READS",
+    "REAL",
+    "RECURSIVE",
+    "REF",
+    "REFERENCES",
+    "REFERENCING",
+    "REGR_AVGX",
+    "REGR_AVGY",
+    "REGR_COUNT",
+    "REGR_INTERCEPT",
+    "REGR_R2",
+    "REGR_SLOPE",
+    "REGR_SXX",
+    "REGR_SXY",
+    "REGR_SYY",
+    "RELATIVE",
+    "RELEASE",
+    "REPEATABLE",
+    "REPLACE",
+    "RESPECT",
+    "RESTART",
+    "RESTRICT",
+    "RETURN",
+    "RETURNED_CARDINALITY",
+    "RETURNED_LENGTH",
+    "RETURNED_OCTET_LENGTH",
+    "RETURNED_SQLSTATE",
+    "RETURNING",
+    "REVOKE",
+    "RIGHT",
+    "ROLLBACK",
+    "ROLLUP",
+    "ROUTINE",
+    "ROUTINE_CATALOG",
+    "ROUTINE_NAME",
+    "ROUTINE_SCHEMA",
+    "ROW",
+    "ROW_COUNT",
+    "ROW_NUMBER",
+    "ROWS",
+    "SAVEPOINT",
+    "SCHEMA",
+    "SCHEMA_NAME",
+    "SCOPE",
+    "SCOPE_CATALOG",
+    "SCOPE_NAME",
+    "SCOPE_SCHEMA",
+    "SCROLL",
+    "SEARCH",
+    "SECOND",
+    "SECTION",
+    "SECURITY",
+    "SELECT",
+    "SELF",
+    "SENSITIVE",
+    "SEQUENCE",
+    "SERIALIZABLE",
+    "SERVER_NAME",
+    "SESSION",
+    "SESSION_USER",
+    "SET",
+    "SETS",
+    "SIMILAR",
+    "SIZE",
+    "SMALLINT",
+    "SOME",
+    "SPACE",
+    "SPECIFIC",
+    "SPECIFIC_NAME",
+    "SPECIFIC_SCHEMA",
+    "SPECIFICTYPE",
+    "SQL",
+    "SQLCODE",
+    "SQLERROR",
+    "SQLEXCEPTION",
+    "SQLSTATE",
+    "SQLWARNING",
+    "SQRT",
+    "START",
+    "STATE",
+    "STATEMENT",
+    "STATIC",
+    "STDDEV_POP",
+    "STDDEV_SAMP",
+    "STRUCTURE",
+    "STYLE",
+    "SUBCLASS_ORIGIN",
+    "SUBMULTISET",
+    "SUBSTRING",
+    "SUM",
+    "SYMMETRIC",
+    "SYSTEM",
+    "SYSTEM_USER",
+    "TABLE",
+    "TABLE_NAME",
+    "TABLESAMPLE",
+    "TEMPORARY",
+    "THEN",
+    "TIES",
+    "TIME",
+    "TIMESTAMP",
+    "TIMESTAMPADD",
+    "TIMESTAMPDIFF",
+    "TIMEZONE_HOUR",
+    "TIMEZONE_MINUTE",
+    "TO",
+    "TOP",
+    "TRAILING",
+    "TRANSACTION",
+    "TRANSACTIONS_COMMITTED",
+    "TRANSACTIONS_ROLLED_BACK",
+    "TRANSACTION_ACTIVE",
+    "TRANSFORM",
+    "TRANSFORMS",
+    "TRANSLATE",
+    "TRANSLATION",
+    "TREAT",
+    "TRIGGER",
+    "TRIGGER_CATALOG",
+    "TRIGGER_NAME",
+    "TRIGGER_SCHEMA",
+    "TRIM",
+    "TRUE",
+    "TRUNCATE",
+    "UNBOUNDED",
+    "UNCOMMITTED",
+    "UNDER",
+    "UNION",
+    "UNIQUE",
+    "UNKNOWN",
+    "UNLISTEN",
+    "UNNAMED",
+    "UNNEST",
+    "UNTIL",
+    "UPDATE",
+    "UPPER",
+    "USAGE",
+    "USER",
+    "USER_DEFINED_TYPE_CATALOG",
+    "USER_DEFINED_TYPE_CODE",
+    "USER_DEFINED_TYPE_NAME",
+    "USER_DEFINED_TYPE_SCHEMA",
+    "USING",
+    "VACUUM",
+    "VALID",
+    "VALUE",
+    "VALUES",
+    "VAR_POP",
+    "VAR_SAMP",
+    "VARCHAR",
+    "VARYING",
+    "VERBOSE",
+    "VERSION",
+    "VIEW",
+    "WHEN",
+    "WHENEVER",
+    "WHERE",
+    "WIDTH_BUCKET",
+    "WINDOW",
+    "WITH",
+    "WITHIN",
+    "WITHOUT",
+    "WORK",
+    "WRITE",
+    "XML",
+    "XMLAGG",
+    "XMLATTRIBUTES",
+    "XMLCONCAT",
+    "XMLELEMENT",
+    "XMLEXISTS",
+    "XMLFOREST",
+    "XMLPARSE",
+    "XMLPI",
+    "XMLQUERY",
+    "XMLROOT",
+    "XMLSERIALIZE",
+    "XMLTABLE",
+    "XMLTEXT",
+    "XMLVALIDATE",
+    "YEAR",
+    "YES",
+    "ZONE",
+};
+
 /**
  * Copied from -- http://www.sqlite.org/lang_keywords.html
  */
-const std::array<const char*, 145> sql_keywords = {
+constexpr std::array<const char*, 145> sqlite_keywords = {
     "ABORT",
     "ACTION",
     "ADD",
@@ -290,9 +859,7 @@ std::multimap<std::string, const help_text*> sqlite_function_help;
 static int
 handle_db_list(void* ptr, int ncols, char** colvalues, char** colnames)
 {
-    struct sqlite_metadata_callbacks* smc;
-
-    smc = (struct sqlite_metadata_callbacks*) ptr;
+    auto* smc = (struct sqlite_metadata_callbacks*) ptr;
 
     smc->smc_db_list[colvalues[1]] = std::vector<std::string>();
     if (!smc->smc_database_list) {
@@ -303,14 +870,14 @@ handle_db_list(void* ptr, int ncols, char** colvalues, char** colnames)
 }
 
 struct table_list_data {
-    struct sqlite_metadata_callbacks* tld_callbacks;
+    sqlite_metadata_callbacks* tld_callbacks;
     db_table_map_t::iterator* tld_iter;
 };
 
 static int
 handle_table_list(void* ptr, int ncols, char** colvalues, char** colnames)
 {
-    struct table_list_data* tld = (struct table_list_data*) ptr;
+    auto* tld = (struct table_list_data*) ptr;
 
     (*tld->tld_iter)->second.emplace_back(colvalues[0]);
     if (!tld->tld_callbacks->smc_table_list) {
@@ -322,7 +889,7 @@ handle_table_list(void* ptr, int ncols, char** colvalues, char** colnames)
 }
 
 int
-walk_sqlite_metadata(sqlite3* db, struct sqlite_metadata_callbacks& smc)
+walk_sqlite_metadata(sqlite3* db, sqlite_metadata_callbacks& smc)
 {
     auto_mem<char, sqlite3_free> errmsg;
     int retval;
@@ -349,7 +916,7 @@ walk_sqlite_metadata(sqlite3* db, struct sqlite_metadata_callbacks& smc)
     for (auto iter = smc.smc_db_list.begin(); iter != smc.smc_db_list.end();
          ++iter)
     {
-        struct table_list_data tld = {&smc, &iter};
+        table_list_data tld = {&smc, &iter};
         auto_mem<char, sqlite3_free> query;
 
         query = sqlite3_mprintf(
@@ -368,11 +935,11 @@ walk_sqlite_metadata(sqlite3* db, struct sqlite_metadata_callbacks& smc)
              ++table_iter)
         {
             auto_mem<char, sqlite3_free> table_query;
-            std::string& table_name = *table_iter;
+            smc.smc_table_name = *table_iter;
 
             table_query = sqlite3_mprintf("pragma %Q.table_xinfo(%Q)",
                                           iter->first.c_str(),
-                                          table_name.c_str());
+                                          smc.smc_table_name.c_str());
             if (table_query == nullptr) {
                 return SQLITE_NOMEM;
             }
@@ -388,7 +955,7 @@ walk_sqlite_metadata(sqlite3* db, struct sqlite_metadata_callbacks& smc)
 
             table_query = sqlite3_mprintf("pragma %Q.foreign_key_list(%Q)",
                                           iter->first.c_str(),
-                                          table_name.c_str());
+                                          smc.smc_table_name.c_str());
             if (table_query == nullptr) {
                 return SQLITE_NOMEM;
             }
@@ -420,8 +987,8 @@ schema_collation_list(void* ptr, int ncols, char** colvalues, char** colnames)
 static int
 schema_db_list(void* ptr, int ncols, char** colvalues, char** colnames)
 {
-    struct sqlite_metadata_callbacks* smc = (sqlite_metadata_callbacks*) ptr;
-    std::string& schema_out = *((std::string*) smc->smc_userdata);
+    auto smc = (sqlite_metadata_callbacks*) ptr;
+    auto& schema_out = *((std::string*) smc->smc_userdata);
     auto_mem<char, sqlite3_free> attach_sql;
 
     attach_sql = sqlite3_mprintf(
@@ -435,8 +1002,8 @@ schema_db_list(void* ptr, int ncols, char** colvalues, char** colnames)
 static int
 schema_table_list(void* ptr, int ncols, char** colvalues, char** colnames)
 {
-    struct sqlite_metadata_callbacks* smc = (sqlite_metadata_callbacks*) ptr;
-    std::string& schema_out = *((std::string*) smc->smc_userdata);
+    auto smc = (sqlite_metadata_callbacks*) ptr;
+    auto& schema_out = *((std::string*) smc->smc_userdata);
     auto_mem<char, sqlite3_free> create_sql;
 
     create_sql = sqlite3_mprintf("%s;\n", colvalues[1]);
@@ -461,7 +1028,7 @@ schema_foreign_key_list(void* ptr, int ncols, char** colvalues, char** colnames)
 void
 dump_sqlite_schema(sqlite3* db, std::string& schema_out)
 {
-    struct sqlite_metadata_callbacks schema_sql_meta_callbacks = {
+    sqlite_metadata_callbacks schema_sql_meta_callbacks = {
         schema_collation_list,
         schema_db_list,
         schema_table_list,
@@ -574,6 +1141,13 @@ sql_ident_needs_quote(const char* ident)
     return false;
 }
 
+std::string
+sql_quote_text(const std::string& str)
+{
+    auto quoted_token = lnav::sql::mprintf("%Q", str.c_str());
+    return {quoted_token};
+}
+
 auto_mem<char, sqlite3_free>
 sql_quote_ident(const char* ident)
 {
@@ -670,7 +1244,8 @@ annotate_sql_with_error(sqlite3* db, const char* sql, const char* tail)
         retval.append("\n");
     }
     retval.with_attr_for_all(VC_ROLE.value(role_t::VCR_QUOTED_CODE));
-    readline_sqlite_highlighter(retval, retval.length());
+    readline_sql_highlighter(
+        retval, lnav::sql::dialect::sqlite, retval.length());
 
     if (erroff != -1) {
         auto line_with_error
@@ -851,22 +1426,31 @@ sql_execute_script(sqlite3* db,
     sql_compile_script(db, global_vars, src_name, script, errors);
 }
 
-static struct {
+static const struct {
     int sqlite_type;
     const char* collator;
     const char* sample;
 } TYPE_TEST_VALUE[] = {
     {SQLITE3_TEXT, "", "foobar"},
-    {SQLITE_INTEGER, "", "123"},
     {SQLITE_FLOAT, "", "123.0"},
+    {SQLITE_INTEGER, "", "123"},
     {SQLITE_TEXT, "ipaddress", "127.0.0.1"},
+    {SQLITE_TEXT, "measure_with_units", "123ms"},
+    {SQLITE_TEXT, "measure_with_units", "123 ms"},
+    {SQLITE_TEXT, "measure_with_units", "123KB"},
+    {SQLITE_TEXT, "measure_with_units", "123 KB"},
+    {SQLITE_TEXT, "measure_with_units", "123Kbps"},
+    {SQLITE_TEXT, "measure_with_units", "123.0 Kbps"},
+    {SQLITE_TEXT, "measure_with_units", "123.0KB"},
+    {SQLITE_TEXT, "measure_with_units", "123.0 KB"},
+    {SQLITE_TEXT, "measure_with_units", "123.0Kbps"},
+    {SQLITE_TEXT, "measure_with_units", "123.0 Kbps"},
 };
 
 int
 guess_type_from_pcre(const std::string& pattern, std::string& collator)
 {
-    static const std::vector<int> number_matches = {1, 2};
-
+    log_info("guessing SQL type from pattern: %s", pattern.c_str());
     auto compile_res = lnav::pcre2pp::code::from(pattern);
     if (compile_res.isErr()) {
         return SQLITE3_TEXT;
@@ -879,14 +1463,22 @@ guess_type_from_pcre(const std::string& pattern, std::string& collator)
 
     collator.clear();
     for (const auto& test_value : TYPE_TEST_VALUE) {
-        auto find_res
+        log_info("  testing sample: %s", test_value.sample);
+        const auto find_res
             = re.find_in(string_fragment::from_c_str(test_value.sample),
                          PCRE2_ANCHORED)
                   .ignore_error();
         if (find_res && find_res->f_all.sf_begin == 0
             && find_res->f_remaining.empty())
         {
+            log_info("    matched!");
             matches.push_back(index);
+            break;
+        }
+        if (!find_res) {
+            log_info("    mismatch");
+        } else if (!find_res->f_remaining.empty()) {
+            log_info("    incomplete match");
         }
 
         index += 1;
@@ -895,9 +1487,6 @@ guess_type_from_pcre(const std::string& pattern, std::string& collator)
     if (matches.size() == 1) {
         retval = TYPE_TEST_VALUE[matches.front()].sqlite_type;
         collator = TYPE_TEST_VALUE[matches.front()].collator;
-    } else if (matches == number_matches) {
-        retval = SQLITE_FLOAT;
-        collator = "";
     }
 
     return retval;
@@ -919,7 +1508,7 @@ sqlite3_type_to_string(int type)
             return "BLOB";
     }
 
-    ensure("Invalid sqlite type");
+    ensure(!!!"Invalid sqlite type");
 
     return nullptr;
 }
@@ -963,22 +1552,38 @@ sqlite3_errmsg_to_attr_line(sqlite3* db)
     return attr_line_t(errmsg);
 }
 
-std::string
-sql_keyword_re()
+static void
+append_kw(std::string& retval, bool& first, const char* kw)
+{
+    if (!first) {
+        retval.append("|");
+    } else {
+        first = false;
+    }
+    retval.append("\\b");
+    retval.append(kw);
+    retval.append("\\b");
+}
+
+static std::string
+sql_keyword_re(lnav::sql::dialect dia)
 {
     std::string retval = "(?:";
-    bool first = true;
+    auto first = true;
 
-    for (const char* kw : sql_keywords) {
-        if (!first) {
-            retval.append("|");
-        } else {
-            first = false;
-        }
-        retval.append("\\b");
-        retval.append(kw);
-        retval.append("\\b");
+    switch (dia) {
+        case lnav::sql::dialect::sqlite:
+            for (const char* kw : sqlite_keywords) {
+                append_kw(retval, first, kw);
+            }
+            break;
+        default:
+            for (const char* kw : pg_sql_keywords) {
+                append_kw(retval, first, kw);
+            }
+            break;
     }
+
     retval += ")";
 
     return retval;
@@ -987,8 +1592,9 @@ sql_keyword_re()
 constexpr string_attr_type<void> SQL_COMMAND_ATTR("sql_command");
 constexpr string_attr_type<void> SQL_KEYWORD_ATTR("sql_keyword");
 constexpr string_attr_type<void> SQL_IDENTIFIER_ATTR("sql_ident");
-constexpr string_attr_type<void> SQL_FUNCTION_ATTR("sql_func");
+constexpr string_attr_type<std::string> SQL_FUNCTION_ATTR("sql_func");
 constexpr string_attr_type<void> SQL_STRING_ATTR("sql_string");
+constexpr string_attr_type<void> SQL_HEX_LIT_ATTR("sql_hex_lit");
 constexpr string_attr_type<void> SQL_NUMBER_ATTR("sql_number");
 constexpr string_attr_type<void> SQL_OPERATOR_ATTR("sql_oper");
 constexpr string_attr_type<void> SQL_PAREN_ATTR("sql_paren");
@@ -997,9 +1603,18 @@ constexpr string_attr_type<void> SQL_GARBAGE_ATTR("sql_garbage");
 constexpr string_attr_type<void> SQL_COMMENT_ATTR("sql_comment");
 
 void
-annotate_sql_statement(attr_line_t& al)
+annotate_sql_statement(attr_line_t& al, lnav::sql::dialect dia)
 {
-    static const std::string keyword_re_str = R"(\A)" + sql_keyword_re();
+    static const std::string sqlite_keyword_re_str
+        = R"(\A)" + sql_keyword_re(lnav::sql::dialect::sqlite);
+    static const auto sqlite_keyword_re
+        = lnav::pcre2pp::code::from(sqlite_keyword_re_str, PCRE2_CASELESS)
+              .unwrap();
+    static const std::string sql_keyword_re_str
+        = R"(\A)" + sql_keyword_re(lnav::sql::dialect::sql);
+    static const auto sql_keyword_re
+        = lnav::pcre2pp::code::from(sql_keyword_re_str, PCRE2_CASELESS)
+              .unwrap();
 
     static const struct {
         lnav::pcre2pp::code re;
@@ -1014,8 +1629,9 @@ annotate_sql_statement(attr_line_t& al)
             &SQL_PAREN_ATTR,
         },
         {
-            lnav::pcre2pp::code::from(keyword_re_str, PCRE2_CASELESS).unwrap(),
-            &SQL_KEYWORD_ATTR,
+            lnav::pcre2pp::code::from_const(
+                R"(\A(?:x|X)'(?:(?:[0-9a-fA-F]{2})+'|[0-9a-fA-F]*$))"),
+            &SQL_HEX_LIT_ATTR,
         },
         {
             lnav::pcre2pp::code::from_const(R"(\A'[^']*('(?:'[^']*')*|$))"),
@@ -1027,7 +1643,7 @@ annotate_sql_statement(attr_line_t& al)
         },
         {
             lnav::pcre2pp::code::from_const(
-                R"(\A-?\d+(?:\.\d+)?(?:[eE][\-\+]?\d+)?)"),
+                R"(\A-?\d+(?:\.\d+)?(?:[eE][\-\+]?\d+)?\b)"),
             &SQL_NUMBER_ATTR,
         },
         {
@@ -1042,9 +1658,13 @@ annotate_sql_statement(attr_line_t& al)
         },
         {
             lnav::pcre2pp::code::from_const(
-                R"(\A(\*|\->{1,2}|<|>|=|!|\-|\+|\|\|))"),
+                R"(\A(~|%|\*|\->{1,2}|<=|>=|<<|>>|<>|<|>|={1,2}|!=?|\-|\+|\|\|{1,2}|&|::))"),
             &SQL_OPERATOR_ATTR,
         },
+        {
+            lnav::pcre2pp::code::from_const(R"(\A[0-9][a-zA-Z0-9\-\._]+)"),
+            &SQL_GARBAGE_ATTR,
+        },
         {
             lnav::pcre2pp::code::from_const(R"(\A.)"),
             &SQL_GARBAGE_ATTR,
@@ -1079,6 +1699,16 @@ annotate_sql_statement(attr_line_t& al)
             remaining = ws_find_res->f_remaining;
             continue;
         }
+        const auto& kw_pat = dia == lnav::sql::dialect::sqlite
+            ? sqlite_keyword_re
+            : sql_keyword_re;
+        auto kw_pat_find_res = kw_pat.find_in(remaining).ignore_error();
+        if (kw_pat_find_res) {
+            sa.emplace_back(to_line_range(kw_pat_find_res->f_all),
+                            SQL_KEYWORD_ATTR.value());
+            remaining = kw_pat_find_res->f_remaining;
+            continue;
+        }
         for (const auto& pat : PATTERNS) {
             auto pat_find_res = pat.re.find_in(remaining).ignore_error();
             if (pat_find_res) {
@@ -1097,22 +1727,10 @@ annotate_sql_statement(attr_line_t& al)
            != sa.end())
     {
         string_attrs_t::const_iterator piter;
-        bool found_open = false;
-        ssize_t lpc;
 
         start = iter->sa_range.lr_end;
-        for (lpc = iter->sa_range.lr_end; lpc < (int) line.length(); lpc++) {
-            if (line[lpc] == '(') {
-                found_open = true;
-                break;
-            }
-            if (!isspace(line[lpc])) {
-                break;
-            }
-        }
-
-        if (found_open) {
-            ssize_t pstart = lpc + 1;
+        if (start < (ssize_t) line.length() && line[start] == '(') {
+            ssize_t pstart = start + 1;
             int depth = 1;
 
             while (depth > 0
@@ -1131,13 +1749,16 @@ annotate_sql_statement(attr_line_t& al)
             if (piter == sa.end()) {
                 func_range.lr_end = line.length();
             } else {
-                func_range.lr_end = piter->sa_range.lr_end - 1;
+                func_range.lr_end = piter->sa_range.lr_end;
             }
-            sa.emplace_back(func_range, SQL_FUNCTION_ATTR.value());
+            auto func_name = al.to_string_fragment(iter);
+            sa.emplace_back(
+                func_range,
+                SQL_FUNCTION_ATTR.value(tolower(func_name.to_string())));
         }
     }
 
-    remove_string_attr(sa, &SQL_PAREN_ATTR);
+    // remove_string_attr(sa, &SQL_PAREN_ATTR);
     stable_sort(sa.begin(), sa.end());
 }
 
@@ -1202,9 +1823,8 @@ find_sql_help_for_line(const attr_line_t& al, size_t x)
             return false;
         }
 
-        const std::string& str = al.get_string();
-        const line_range& lr = sa.sa_range;
-        int lpc;
+        const auto& str = al.get_string();
+        const auto& lr = sa.sa_range;
 
         if (sa.sa_type == &SQL_FUNCTION_ATTR) {
             if (!sa.sa_range.contains(x)) {
@@ -1212,7 +1832,8 @@ find_sql_help_for_line(const attr_line_t& al, size_t x)
             }
         }
 
-        for (lpc = lr.lr_start; lpc < lr.lr_end; lpc++) {
+        auto lpc = lr.lr_start;
+        for (; lpc < lr.lr_end; lpc++) {
             if (!isalnum(str[lpc]) && str[lpc] != '_') {
                 break;
             }
@@ -1257,6 +1878,25 @@ find_sql_help_for_line(const attr_line_t& al, size_t x)
     return retval;
 }
 
+template<>
+Result<lnav::sql::dialect, std::string>
+from(string_fragment sf)
+{
+    if (sf == "sql"_frag) {
+        return Ok(lnav::sql::dialect::sql);
+    }
+    if (sf == "sqlite"_frag) {
+        return Ok(lnav::sql::dialect::sqlite);
+    }
+    if (sf == "plpgsql"_frag) {
+        return Ok(lnav::sql::dialect::plpgsql);
+    }
+    if (sf == "prql"_frag) {
+        return Ok(lnav::sql::dialect::prql);
+    }
+    return Err(fmt::format(FMT_STRING("unknown SQL dialect: {}"), sf));
+}
+
 namespace lnav {
 namespace sql {
 
@@ -1365,7 +2005,8 @@ constexpr string_attr_type<void> PRQL_STRING_ATTR("prql_string");
 constexpr string_attr_type<void> PRQL_NUMBER_ATTR("prql_number");
 constexpr string_attr_type<void> PRQL_OPERATOR_ATTR("prql_oper");
 constexpr string_attr_type<void> PRQL_PAREN_ATTR("prql_paren");
-constexpr string_attr_type<void> PRQL_UNTERMINATED_PAREN_ATTR("prql_unterminated_paren");
+constexpr string_attr_type<void> PRQL_UNTERMINATED_PAREN_ATTR(
+    "prql_unterminated_paren");
 constexpr string_attr_type<void> PRQL_GARBAGE_ATTR("prql_garbage");
 constexpr string_attr_type<void> PRQL_COMMENT_ATTR("prql_comment");
 
@@ -1423,7 +2064,7 @@ annotate_prql_statement(attr_line_t& al)
             &PRQL_OPERATOR_ATTR,
         },
         {
-            lnav::pcre2pp::code::from_const(R"(\A\|)"),
+            lnav::pcre2pp::code::from_const(R"(\A(?:\||\n))"),
             &PRQL_PIPE_ATTR,
         },
         {
@@ -1436,7 +2077,8 @@ annotate_prql_statement(attr_line_t& al)
         },
     };
 
-    static const auto ws_pattern = lnav::pcre2pp::code::from_const(R"(\A\s+)");
+    static const auto ws_pattern
+        = lnav::pcre2pp::code::from_const(R"(\A[ \t\r]+)");
 
     const auto& line = al.get_string();
     auto& sa = al.get_attrs();
@@ -1452,6 +2094,11 @@ annotate_prql_statement(attr_line_t& al)
             if (pat_find_res) {
                 sa.emplace_back(to_line_range(pat_find_res->f_all),
                                 pat.type->value());
+                if (sa.back().sa_type == &PRQL_PIPE_ATTR
+                    && pat_find_res->f_all == "\n"_frag)
+                {
+                    sa.back().sa_range.lr_start += 1;
+                }
                 remaining = pat_find_res->f_remaining;
                 break;
             }
@@ -1462,11 +2109,15 @@ annotate_prql_statement(attr_line_t& al)
     std::vector<std::pair<char, int>> groups;
     std::vector<line_range> fqids;
     std::optional<line_range> id_start;
+    const string_attr_type_base* last_attr_type = nullptr;
     bool saw_id_dot = false;
     for (const auto& attr : sa) {
-        if (groups.empty() && attr.sa_type == &PRQL_PIPE_ATTR) {
+        if (groups.empty() && attr.sa_type == &PRQL_PIPE_ATTR
+            && last_attr_type != &PRQL_PIPE_ATTR)
+        {
             stages.push_back(attr.sa_range.lr_start);
         }
+        last_attr_type = attr.sa_type;
         if (!id_start) {
             if (attr.sa_type == &PRQL_IDENTIFIER_ATTR) {
                 id_start = attr.sa_range;
@@ -1538,6 +2189,8 @@ annotate_prql_statement(attr_line_t& al)
     for (const auto& fqid_range : fqids) {
         sa.emplace_back(fqid_range, PRQL_FQID_ATTR.value());
     }
+    remove_string_attr(sa, &PRQL_IDENTIFIER_ATTR);
+    remove_string_attr(sa, &PRQL_DOT_ATTR);
 
     stable_sort(sa.begin(), sa.end());
 }
