@@ -43,17 +43,29 @@ status_field::no_op_action(status_field&)
 {
 }
 
-void
+bool
 status_field::set_value(std::string value)
 {
-    auto& sa = this->sf_value.get_attrs();
+    if (value == this->sf_value.al_string) {
+        return false;
+    }
 
-    sa.clear();
-
-    scrub_ansi_string(value, &sa);
-    this->sf_value.with_string(value);
+    this->sf_value.with_ansi_string(value);
+    return true;
 }
 
+bool
+status_field::set_value(const string_fragment& value)
+{
+    if (value == this->sf_value.al_string) {
+        return false;
+    }
+
+    this->sf_value.with_ansi_string(value);
+    return true;
+}
+
+
 void
 status_field::do_cylon()
 {
@@ -66,7 +78,7 @@ status_field::do_cylon()
         ? cycle_pos
         : (this->sf_width - (cycle_pos - this->sf_width) - 1);
     auto stop = std::min(start + 3, this->sf_width);
-    struct line_range lr(std::max<long>(start, 0L), stop);
+    line_range lr(std::max<long>(start, 0L), stop);
     const auto& vc = view_colors::singleton();
 
     auto attrs = vc.attrs_for_role(role_t::VCR_ACTIVE_STATUS);
@@ -80,7 +92,7 @@ void
 status_field::set_stitch_value(role_t left, role_t right)
 {
     auto& sa = this->sf_value.get_attrs();
-    struct line_range lr(0, 1);
+    line_range lr(0, 1);
 
     this->sf_value.get_string() = "::";
     sa.clear();
@@ -93,16 +105,20 @@ status_field::set_stitch_value(role_t left, role_t right)
 bool
 statusview_curses::do_update()
 {
+    if (!this->vc_needs_update) {
+        return view_curses::do_update();
+    }
+
     int left = 0;
     auto& vc = view_colors::singleton();
     unsigned int width, height;
 
     this->sc_displayed_fields.clear();
-    if (!this->vc_visible || this->sc_window == nullptr) {
+    if (!this->vc_visible || this->vc_window == nullptr) {
         return false;
     }
 
-    ncplane_dim_yx(this->sc_window, &height, &width);
+    ncplane_dim_yx(this->vc_window, &height, &width);
     this->window_change();
 
     int top = this->vc_y < 0 ? height + this->vc_y : this->vc_y;
@@ -113,10 +129,10 @@ statusview_curses::do_update()
     nccell clear_cell;
     nccell_init(&clear_cell);
     nccell_prime(
-        this->sc_window, &clear_cell, " ", 0, view_colors::to_channels(attrs));
-    ncplane_cursor_move_yx(this->sc_window, top, 0);
-    ncplane_hline(this->sc_window, &clear_cell, width);
-    nccell_release(this->sc_window, &clear_cell);
+        this->vc_window, &clear_cell, " ", 0, view_colors::to_channels(attrs));
+    ncplane_cursor_move_yx(this->vc_window, top, 0);
+    ncplane_hline(this->vc_window, &clear_cell, width);
+    nccell_release(this->vc_window, &clear_cell);
 
     if (this->sc_source != nullptr) {
         auto field_count = this->sc_source->statusview_fields();
@@ -134,10 +150,8 @@ statusview_curses::do_update()
                     if (sa.sa_type == &VC_STYLE) {
                         auto sa_attrs = sa.sa_value.get<text_attrs>();
                         sa_attrs.clear_style(text_attrs::style::reverse);
-                        sa_attrs.ta_fg_color
-                            = styling::color_unit::make_empty();
-                        sa_attrs.ta_bg_color
-                            = styling::color_unit::make_empty();
+                        sa_attrs.ta_fg_color = styling::color_unit::EMPTY;
+                        sa_attrs.ta_bg_color = styling::color_unit::EMPTY;
                         sa.sa_value = sa_attrs;
                     } else if (sa.sa_type == &VC_ROLE) {
                         if (sa.sa_value.get<role_t>()
@@ -166,7 +180,7 @@ statusview_curses::do_update()
             }
 
             if (val.length() > sf.get_width()) {
-                static const std::string ELLIPSIS = "\xE2\x8B\xAF";
+                static constexpr auto ELLIPSIS = "\xE2\x8B\xAF"_frag;
 
                 if (sf.get_width() > 11) {
                     size_t half_width = sf.get_width() / 2 - 1;
@@ -189,12 +203,13 @@ statusview_curses::do_update()
             }
 
             auto write_res
-                = mvwattrline(this->sc_window, top, x, val, lr, default_role);
+                = mvwattrline(this->vc_window, top, x, val, lr, default_role);
             this->sc_displayed_fields.emplace_back(
                 line_range{x, static_cast<int>(x + write_res.mr_chars_out)},
                 field);
         }
     }
+    this->vc_needs_update = false;
 
     return true;
 }
@@ -211,7 +226,7 @@ statusview_curses::window_change()
     double remaining = 0;
     std::vector<status_field*> resizable;
 
-    auto width = ncplane_dim_x(this->sc_window);
+    auto width = ncplane_dim_x(this->vc_window);
     remaining = width - 2;
 
     for (int field = 0; field < field_count; field++) {
@@ -248,7 +263,10 @@ statusview_curses::window_change()
         remaining -= (actual_width - sf->get_min_width());
         total_shares -= sf->get_share();
 
-        sf->set_width(actual_width);
+        if (sf->get_width() != actual_width) {
+            sf->set_width(actual_width);
+            this->set_needs_update();
+        }
     }
 }
 
@@ -267,6 +285,10 @@ statusview_curses::handle_mouse(mouse_event& me)
             find_res.value()->df_field_index);
 
         sf.on_click(sf);
+    } else if (me.me_state == mouse_button_state_t::BUTTON_STATE_DRAGGED) {
+        if (this->sc_source->on_drag) {
+            this->sc_source->on_drag(me);
+        }
     }
 
     return true;
