@@ -27,6 +27,8 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <filesystem>
+
 #include "text_anonymizer.hh"
 
 #include <arpa/inet.h>
@@ -37,7 +39,6 @@
 #include "config.h"
 #include "data_scanner.hh"
 #include "diseases-json.h"
-#include <filesystem>
 #include "hasher.hh"
 #include "pcrepp/pcre2pp.hh"
 #include "words-json.h"
@@ -61,18 +62,26 @@ struct random_list {
     }
 };
 
-static const typed_json_path_container<random_list> random_list_handlers = {
-    yajlpp::property_handler("data#").for_field(&random_list::rl_data),
-};
+static const typed_json_path_container<random_list>&
+get_random_list_handlers()
+{
+    static const typed_json_path_container<random_list> retval = {
+        yajlpp::property_handler("data#").for_field(&random_list::rl_data),
+    };
+
+    return retval;
+}
 
 static random_list
 load_word_list()
 {
     static const intern_string_t name
         = intern_string::lookup(words_json.get_name());
-    auto parse_res
-        = random_list_handlers.parser_for(name).with_ignore_unused(false).of(
-            words_json.to_string_fragment());
+    auto sfp = words_json.to_string_fragment_producer();
+    auto parse_res = get_random_list_handlers()
+                         .parser_for(name)
+                         .with_ignore_unused(false)
+                         .of(*sfp);
 
     return parse_res.unwrap();
 }
@@ -90,9 +99,11 @@ load_animal_list()
 {
     static const intern_string_t name
         = intern_string::lookup(animals_json.get_name());
-    auto parse_res
-        = random_list_handlers.parser_for(name).with_ignore_unused(false).of(
-            animals_json.to_string_fragment());
+    auto sfp = animals_json.to_string_fragment_producer();
+    auto parse_res = get_random_list_handlers()
+                         .parser_for(name)
+                         .with_ignore_unused(false)
+                         .of(*sfp);
 
     return parse_res.unwrap();
 }
@@ -110,9 +121,11 @@ load_disease_list()
 {
     static const intern_string_t name
         = intern_string::lookup(diseases_json.get_name());
-    auto parse_res
-        = random_list_handlers.parser_for(name).with_ignore_unused(false).of(
-            diseases_json.to_string_fragment());
+    auto sfp = diseases_json.to_string_fragment_producer();
+    auto parse_res = get_random_list_handlers()
+                         .parser_for(name)
+                         .with_ignore_unused(false)
+                         .of(*sfp);
 
     return parse_res.unwrap();
 }
@@ -342,8 +355,7 @@ text_anonymizer::next(string_fragment line)
                             (unsigned char) ((base_mac >> 0) & 0xff),
                         });
 
-                        return anon_mac.to_string(
-                            std::make_optional(inp[2]));
+                        return anon_mac.to_string(std::make_optional(inp[2]));
                     });
                 break;
             }
@@ -468,7 +480,7 @@ text_anonymizer::next(string_fragment line)
                 } else {
                     static const auto ATTR_RE
                         = lnav::pcre2pp::code::from_const(R"([\w\-]+=)");
-                    static thread_local auto md
+                    thread_local auto md
                         = lnav::pcre2pp::match_data::unitialized();
 
                     auto remaining = string_fragment::from_str_range(
