@@ -35,6 +35,7 @@
 #include "base/ansi_scrubber.hh"
 #include "base/humanize.time.hh"
 #include "base/injector.hh"
+#include "base/lnav_log.hh"
 #include "base/time_util.hh"
 #include "config.h"
 #include "data_scanner.hh"
@@ -77,20 +78,21 @@ text_filter::revert_to_last(logfile_filter_state& lfs, size_t rollback_size)
     }
 }
 
-void
+bool
 text_filter::add_line(logfile_filter_state& lfs,
                       logfile::const_iterator ll,
                       const shared_buffer_ref& line)
 {
-    bool match_state = this->matches(line_source{*lfs.tfs_logfile, ll}, line);
-
     if (ll->is_message()) {
         this->end_of_message(lfs);
     }
+    auto retval = this->matches(line_source{*lfs.tfs_logfile, ll}, line);
 
     lfs.tfs_message_matched[this->lf_index]
-        = lfs.tfs_message_matched[this->lf_index] || match_state;
+        = lfs.tfs_message_matched[this->lf_index] || retval;
     lfs.tfs_lines_for_message[this->lf_index] += 1;
+
+    return retval;
 }
 
 void
@@ -171,7 +173,7 @@ text_accel_source::get_time_offset_for_line(textview_curses& tc, vis_line_t vl)
     } else {
         auto prev_row
             = std::max(prev_umark.value_or(0_vl), prev_emark.value_or(0_vl));
-        auto first_line = this->text_accel_get_line(prev_row);
+        auto* first_line = this->text_accel_get_line(prev_row);
         auto start_tv = first_line->get_timeval();
         diff_tv = curr_tv - start_tv;
     }
@@ -179,13 +181,16 @@ text_accel_source::get_time_offset_for_line(textview_curses& tc, vis_line_t vl)
     return humanize::time::duration::from_tv(diff_tv).to_string();
 }
 
-const bookmark_type_t textview_curses::BM_ERRORS("error");
-const bookmark_type_t textview_curses::BM_WARNINGS("warning");
-const bookmark_type_t textview_curses::BM_USER("user");
-const bookmark_type_t textview_curses::BM_USER_EXPR("user-expr");
-const bookmark_type_t textview_curses::BM_SEARCH("search");
-const bookmark_type_t textview_curses::BM_META("meta");
-const bookmark_type_t textview_curses::BM_PARTITION("partition");
+const DIST_SLICE(bm_types) bookmark_type_t textview_curses::BM_ERRORS("error");
+const DIST_SLICE(bm_types)
+    bookmark_type_t textview_curses::BM_WARNINGS("warning");
+const DIST_SLICE(bm_types) bookmark_type_t textview_curses::BM_USER("user");
+const DIST_SLICE(bm_types)
+    bookmark_type_t textview_curses::BM_USER_EXPR("user-expr");
+const DIST_SLICE(bm_types) bookmark_type_t textview_curses::BM_SEARCH("search");
+const DIST_SLICE(bm_types) bookmark_type_t textview_curses::BM_META("meta");
+const DIST_SLICE(bm_types)
+    bookmark_type_t textview_curses::BM_PARTITION("partition");
 
 textview_curses::textview_curses()
     : lnav_config_listener(__FILE__), tc_search_action(noop_func{})
@@ -248,7 +253,7 @@ textview_curses::reload_config(error_reporter& reporter)
                                          .append_quoted(sc.sc_color))
                                      .with_reason(msg));
                         invalid = true;
-                        return styling::color_unit::make_empty();
+                        return styling::color_unit::EMPTY;
                     }));
             attrs.ta_bg_color = vc.match_color(
                 styling::color_unit::from_str(bg_color).unwrapOrElse(
@@ -259,7 +264,7 @@ textview_curses::reload_config(error_reporter& reporter)
                                          .append_quoted(sc.sc_background_color))
                                      .with_reason(msg));
                         invalid = true;
-                        return styling::color_unit::make_empty();
+                        return styling::color_unit::EMPTY;
                     }));
             if (invalid) {
                 continue;
@@ -334,7 +339,16 @@ textview_curses::grep_begin(grep_proc<vis_line_t>& gp,
             }
         }
         if (pair.first != pair.second) {
-            search_bv.erase(pair.first, pair.second);
+            auto to_del = std::vector<vis_line_t>{};
+            for (auto file_iter = pair.first; file_iter != pair.second;
+                 ++file_iter)
+            {
+                to_del.emplace_back(*file_iter);
+            }
+
+            for (auto cl : to_del) {
+                search_bv.bv_tree.erase(cl);
+            }
         }
     }
 
@@ -347,7 +361,7 @@ textview_curses::grep_end_batch(grep_proc<vis_line_t>& gp)
     if (this->tc_follow_deadline.tv_sec
         && this->tc_follow_selection == this->get_selection())
     {
-        struct timeval now;
+        timeval now;
 
         gettimeofday(&now, nullptr);
         if (this->tc_follow_deadline < now) {
@@ -410,9 +424,6 @@ textview_curses::listview_value_for_rows(const listview_curses& lv,
 bool
 textview_curses::handle_mouse(mouse_event& me)
 {
-    unsigned long width;
-    vis_line_t height;
-
     if (!this->vc_visible || this->lv_height == 0) {
         return false;
     }
@@ -424,7 +435,7 @@ textview_curses::handle_mouse(mouse_event& me)
     auto mouse_line = (me.me_y < 0 || me.me_y >= this->lv_display_lines.size())
         ? empty_space{}
         : this->lv_display_lines[me.me_y];
-    this->get_dimensions(height, width);
+    auto [height, width] = this->get_dimensions();
 
     if (!mouse_line.is<overlay_menu>()
         && (me.me_button != mouse_button_t::BUTTON_LEFT
@@ -463,6 +474,7 @@ textview_curses::handle_mouse(mouse_event& me)
 
     switch (me.me_state) {
         case mouse_button_state_t::BUTTON_STATE_PRESSED: {
+            this->tc_selection_at_press = this->get_selection();
             this->tc_press_line = mouse_line;
             this->tc_press_left = this->lv_left + me.me_press_x;
             if (!this->lv_selectable) {
@@ -471,6 +483,8 @@ textview_curses::handle_mouse(mouse_event& me)
             mouse_line.match(
                 [this, &me, sub_delegate, &mouse_line](const main_content& mc) {
                     this->tc_text_selection_active = true;
+                    this->tc_press_left = this->lv_left
+                        + mc.mc_line_range.lr_start + me.me_press_x;
                     if (this->vc_enabled) {
                         if (this->tc_supports_marks
                             && me.me_button == mouse_button_t::BUTTON_LEFT
@@ -495,6 +509,7 @@ textview_curses::handle_mouse(mouse_event& me)
                 [](const overlay_menu& om) {},
                 [](const static_overlay_content& soc) {},
                 [this](const overlay_content& oc) {
+                    this->set_selection(oc.oc_main_line);
                     this->set_overlay_selection(oc.oc_line);
                 },
                 [](const empty_space& es) {});
@@ -517,8 +532,10 @@ textview_curses::handle_mouse(mouse_event& me)
                             auto line_sf
                                 = string_fragment::from_str(al.get_string());
                             auto cursor_sf = line_sf.sub_cell_range(
-                                this->lv_left + me.me_x,
-                                this->lv_left + me.me_x);
+                                this->lv_left + mc.mc_line_range.lr_start
+                                    + me.me_x,
+                                this->lv_left + mc.mc_line_range.lr_start
+                                    + me.me_x);
                             auto ds = data_scanner(line_sf);
                             auto tf = this->tc_sub_source->get_text_format();
                             while (true) {
@@ -587,10 +604,14 @@ textview_curses::handle_mouse(mouse_event& me)
                 if (mouse_line.is<main_content>()) {
                     auto& mc = mouse_line.get<main_content>();
                     attr_line_t al;
-                    auto low_x = std::min(this->tc_press_left,
-                                          (int) this->lv_left + me.me_x);
-                    auto high_x = std::max(this->tc_press_left,
-                                           (int) this->lv_left + me.me_x);
+                    auto low_x
+                        = std::min(this->tc_press_left,
+                                   (int) this->lv_left
+                                       + mc.mc_line_range.lr_start + me.me_x);
+                    auto high_x
+                        = std::max(this->tc_press_left,
+                                   (int) this->lv_left
+                                       + mc.mc_line_range.lr_start + me.me_x);
 
                     this->set_selection_without_context(mc.mc_line);
                     if (this->tc_supports_marks
@@ -667,9 +688,16 @@ textview_curses::handle_mouse(mouse_event& me)
                 }
             }
             this->tc_text_selection_active = false;
-            if (me.is_click_in(mouse_button_t::BUTTON_RIGHT, 0, INT_MAX)) {
+            if (this->tc_press_line.is<main_content>()
+                && mouse_line.is<main_content>()
+                && me.is_click_in(mouse_button_t::BUTTON_RIGHT, 0, INT_MAX))
+            {
                 auto* lov = this->get_overlay_source();
-                if (lov != nullptr) {
+                if (lov != nullptr
+                    && (!lov->get_show_details_in_overlay()
+                        || this->tc_selection_at_press
+                            == this->get_selection()))
+                {
                     this->set_show_details_in_overlay(
                         !lov->get_show_details_in_overlay());
                 }
@@ -678,16 +706,16 @@ textview_curses::handle_mouse(mouse_event& me)
                 if (this->tc_selection_start) {
                     this->toggle_user_mark(&BM_USER,
                                            this->tc_selection_start.value(),
-                                           this->get_selection());
+                                           this->get_selection().value());
                     this->reload_data();
                 }
                 this->tc_selection_start = std::nullopt;
             }
             if (mouse_line.is<main_content>()) {
-                const auto& [mc_line] = mouse_line.get<main_content>();
+                const auto mc = mouse_line.get<main_content>();
                 attr_line_t al;
 
-                this->textview_value_for_row(mc_line, al);
+                this->textview_value_for_row(mc.mc_line, al);
                 auto line_sf = string_fragment::from_str(al.get_string());
                 auto cursor_sf = line_sf.sub_cell_range(
                     this->lv_left + me.me_x, this->lv_left + me.me_x);
@@ -709,7 +737,7 @@ textview_curses::handle_mouse(mouse_event& me)
                     } else {
                         this->tc_selected_text = selected_text_info{
                             me.me_x,
-                            mc_line,
+                            mc.mc_line,
                             link_iter->sa_range,
                             al.get_attrs(),
                             al.to_string_fragment(link_iter).to_string(),
@@ -717,6 +745,44 @@ textview_curses::handle_mouse(mouse_event& me)
                         };
                     }
                 }
+                if (this->tc_on_click) {
+                    this->tc_on_click(*this, al, cursor_sf.sf_begin);
+                }
+            }
+            if (mouse_line.is<overlay_content>()) {
+                const auto& oc = mouse_line.get<overlay_content>();
+                std::vector<attr_line_t> ov_lines;
+
+                this->lv_overlay_source->list_value_for_overlay(
+                    *this, oc.oc_main_line, ov_lines);
+                const auto& al = ov_lines[oc.oc_line];
+                auto line_sf = string_fragment::from_str(al.get_string());
+                auto cursor_sf = line_sf.sub_cell_range(
+                    this->lv_left + me.me_x, this->lv_left + me.me_x);
+                auto link_iter = find_string_attr_containing(
+                    al.get_attrs(), &VC_HYPERLINK, cursor_sf.sf_begin);
+                if (link_iter != al.get_attrs().end()) {
+                    auto href = link_iter->sa_value.get<std::string>();
+                    auto* ta = dynamic_cast<text_anchors*>(this->tc_sub_source);
+
+                    if (me.me_button == mouse_button_t::BUTTON_LEFT
+                        && ta != nullptr && startswith(href, "#")
+                        && !startswith(href, "#/frontmatter"))
+                    {
+                        auto row_opt = ta->row_for_anchor(href);
+
+                        if (row_opt.has_value()) {
+                            this->tc_sub_source->get_location_history() |
+                                [&oc](auto lh) {
+                                    lh->loc_history_append(oc.oc_main_line);
+                                };
+                            this->set_selection(row_opt.value());
+                        }
+                    }
+                }
+                if (this->tc_on_click) {
+                    this->tc_on_click(*this, al, cursor_sf.sf_begin);
+                }
             }
             if (this->tc_delegate != nullptr) {
                 this->tc_delegate->text_handle_mouse(*this, mouse_line, me);
@@ -748,6 +814,9 @@ textview_curses::apply_highlights(attr_line_t& al,
     }
 
     auto source_format = this->tc_sub_source->get_text_format();
+    if (source_format == text_format_t::TF_BINARY) {
+        return;
+    }
     for (const auto& tc_highlight : this->tc_highlights) {
         bool internal_hl
             = tc_highlight.first.first == highlight_source_t::INTERNAL
@@ -772,8 +841,8 @@ textview_curses::apply_highlights(attr_line_t& al,
         // highlights should apply only to the line itself and not any of
         // the surrounding decorations that are added (for example, the file
         // lines that are inserted at the beginning of the log view).
-        int start_pos = internal_hl ? body.lr_start : orig_line.lr_start;
-        tc_highlight.second.annotate(al, start_pos);
+        auto lr = internal_hl ? body : orig_line;
+        tc_highlight.second.annotate(al, lr);
     }
 }
 
@@ -809,7 +878,7 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
     {
         vis_line_t sel_start, sel_end;
 
-        sel_start = sel_end = this->get_selection();
+        sel_start = sel_end = this->get_selection().value_or(0_vl);
         if (this->tc_selection_start) {
             if (this->tc_selection_start.value() < sel_end) {
                 sel_start = this->tc_selection_start.value();
@@ -819,12 +888,11 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
         }
 
         if (sel_start <= row && row <= sel_end) {
-            auto role = this->get_overlay_selection()
+            auto role = (this->get_overlay_selection() || !this->vc_enabled)
                 ? this->tc_disabled_cursor_role.value()
                 : this->tc_cursor_role.value();
 
-            sa.emplace_back(line_range{orig_line.lr_start, -1},
-                            VC_ROLE.value(role));
+            sa.emplace_back(line_range{0, -1}, VC_ROLE.value(role));
         }
     }
 
@@ -838,10 +906,7 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
 
     const auto& user_marks = this->tc_bookmarks[&BM_USER];
     const auto& user_expr_marks = this->tc_bookmarks[&BM_USER_EXPR];
-    if (std::binary_search(user_marks.begin(), user_marks.end(), row)
-        || std::binary_search(
-            user_expr_marks.begin(), user_expr_marks.end(), row))
-    {
+    if (user_marks.bv_tree.exists(row) || user_expr_marks.bv_tree.exists(row)) {
         sa.emplace_back(line_range{orig_line.lr_start, -1},
                         VC_STYLE.value(text_attrs::with_reverse()));
     }
@@ -986,16 +1051,12 @@ textview_curses::set_user_mark(const bookmark_type_t* bm,
                                vis_line_t vl,
                                bool marked)
 {
-    bookmark_vector<vis_line_t>& bv = this->tc_bookmarks[bm];
-    bookmark_vector<vis_line_t>::iterator iter;
+    auto& bv = this->tc_bookmarks[bm];
 
     if (marked) {
         bv.insert_once(vl);
     } else {
-        iter = std::lower_bound(bv.begin(), bv.end(), vl);
-        if (iter != bv.end() && *iter == vl) {
-            bv.erase(iter);
-        }
+        bv.bv_tree.erase(vl);
     }
     if (this->tc_sub_source) {
         this->tc_sub_source->text_mark(bm, vl, marked);
@@ -1028,15 +1089,7 @@ textview_curses::toggle_user_mark(const bookmark_type_t* bm,
     }
     for (auto curr_line = start_line; curr_line <= end_line; ++curr_line) {
         auto& bv = this->tc_bookmarks[bm];
-        bool added;
-
-        auto iter = bv.insert_once(curr_line);
-        if (iter == bv.end()) {
-            added = true;
-        } else {
-            bv.erase(iter);
-            added = false;
-        }
+        auto [insert_iter, added] = bv.insert_once(curr_line);
         if (this->tc_sub_source) {
             this->tc_sub_source->text_mark(bm, curr_line, added);
         }
@@ -1099,6 +1152,7 @@ textview_curses::set_sub_source(text_sub_source* src)
 std::optional<line_info>
 textview_curses::grep_value_for_line(vis_line_t line, std::string& value_out)
 {
+    // log_debug("grep line %d", line);
     if (this->tc_sub_source
         && line < (int) this->tc_sub_source->text_line_count())
     {
@@ -1112,6 +1166,7 @@ textview_curses::grep_value_for_line(vis_line_t line, std::string& value_out)
             auto new_size = erase_ansi_escapes(value_out);
             value_out.resize(new_size);
         }
+        // log_debug("  line off %lld", retval.li_file_range.fr_offset);
         return retval;
     }
 
@@ -1131,8 +1186,8 @@ text_sub_source::scroll_invoked(textview_curses* tc)
 void
 text_time_translator::ttt_scroll_invoked(textview_curses* tc)
 {
-    if (tc->get_inner_height() > 0 && tc->get_selection() >= 0_vl) {
-        this->time_for_row(tc->get_selection()) |
+    if (tc->get_inner_height() > 0 && tc->get_selection()) {
+        this->time_for_row(tc->get_selection().value()) |
             [this](auto new_top_ri) { this->ttt_top_row_info = new_top_ri; };
     }
 }
@@ -1217,6 +1272,7 @@ filter_stack::delete_filter(const std::string& id)
     }
     if (iter != this->fs_filters.end()) {
         this->fs_filters.erase(iter);
+        this->fs_generation += 1;
         return true;
     }
 
@@ -1282,6 +1338,7 @@ void
 filter_stack::add_filter(const std::shared_ptr<text_filter>& filter)
 {
     this->fs_filters.push_back(filter);
+    this->fs_generation += 1;
 }
 
 void
@@ -1325,6 +1382,32 @@ vis_location_history::loc_history_forward(vis_line_t current_top)
     return this->current_position();
 }
 
+void
+text_sub_source::update_filter_hash_state(hasher& h) const
+{
+    h.update(this->tss_filters.fs_generation);
+
+    const auto* ttt = dynamic_cast<const text_time_translator*>(this);
+    if (ttt != nullptr) {
+        auto min_time = ttt->get_min_row_time();
+        if (min_time) {
+            h.update(min_time->tv_sec);
+            h.update(min_time->tv_usec);
+        } else {
+            h.update(0);
+            h.update(0);
+        }
+        auto max_time = ttt->get_max_row_time();
+        if (max_time) {
+            h.update(max_time->tv_sec);
+            h.update(max_time->tv_usec);
+        } else {
+            h.update(0);
+            h.update(0);
+        }
+    }
+}
+
 void
 text_sub_source::toggle_apply_filters()
 {
