@@ -310,7 +310,7 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
           crender->hcfg = cell_fchannel(targc);
         }
         unsigned fgblends = crender->s.fgblends;
-        cell_blend_fchannel(ncplane_notcurses(p), targc, cell_fchannel(vis), &fgblends);
+        cell_blend_fchannel(p->pile->nc, targc, cell_fchannel(vis), &fgblends);
         crender->s.fgblends = fgblends;
         // crender->highcontrast can only be true if we just set it, since we're
         // about to set targc opaque based on crender->highcontrast (and this
@@ -336,14 +336,14 @@ paint(ncplane* p, struct crender* rvec, int dstleny, int dstlenx,
             vis = &p->basecell;
           }
           unsigned bgblends = crender->s.bgblends;
-          cell_blend_bchannel(ncplane_notcurses(p), targc, cell_bchannel(vis), &bgblends);
+          cell_blend_bchannel(p->pile->nc, targc, cell_bchannel(vis), &bgblends);
           crender->s.bgblends = bgblends;
         }else{ // use the local foreground; we're stacking blittings
           if(nccell_fg_default_p(vis)){
             vis = &p->basecell;
           }
           unsigned bgblends = crender->s.bgblends;
-          cell_blend_bchannel(ncplane_notcurses(p), targc, cell_fchannel(vis), &bgblends);
+          cell_blend_bchannel(p->pile->nc, targc, cell_fchannel(vis), &bgblends);
           crender->s.bgblends = bgblends;
           crender->s.blittedquads = 0;
         }
@@ -497,16 +497,19 @@ postpaint_cell(notcurses* nc, const tinfo* ti, nccell* lastframe, unsigned dimx,
 //       with rasterization--factor out the single cell iteration...
 // FIXME can we not do the blend a single time here, if we track sums in
 //       paint()? tried this before and didn't get a win...
-static void
+static int
 postpaint(notcurses* nc, const tinfo* ti, nccell* lastframe,
           unsigned dimy, unsigned dimx, struct crender* rvec, egcpool* pool){
 //fprintf(stderr, "POSTPAINT BEGINS! %zu %p %d/%d\n", sizeof(*rvec), rvec, dimy, dimx);
+    int totaldamage = 0;
   for(unsigned y = 0 ; y < dimy ; ++y){
     for(unsigned x = 0 ; x < dimx ; ++x){
       struct crender* crender = &rvec[fbcellidx(y, dimx, x)];
       postpaint_cell(nc, ti, lastframe, dimx, crender, pool, y, &x);
+        totaldamage += crender->s.damaged;
     }
   }
+    return totaldamage;
 }
 
 // merging one plane down onto another is basically just performing a render
@@ -731,9 +734,11 @@ term_bg_rgb8(const tinfo* ti, fbuf* f, unsigned r, unsigned g, unsigned b){
       // a single screen, start... combining close ones? For 8-color mode, simple
       // interpolation. I have no idea what to do for 88 colors. FIXME
       if(ti->caps.colors >= 256){
-        return fbuf_emit(f, tiparm(setab, rgb_quantize_256(r, g, b)));
+        TiparmValue argv[] = {tiparm_int(rgb_quantize_256(r, g, b))};
+        return fbuf_emit_parm(f, tiparm_s(setab, 1, argv));
       }else if(ti->caps.colors >= 8){
-        return fbuf_emit(f, tiparm(setab, rgb_quantize_8(r, g, b)));
+        TiparmValue argv[] = {tiparm_int(rgb_quantize_8(r, g, b))};
+        return fbuf_emit_parm(f, tiparm_s(setab, 1, argv));
       }
     }
   }
@@ -756,9 +761,11 @@ int term_fg_rgb8(const tinfo* ti, fbuf* f, unsigned r, unsigned g, unsigned b){
       // a single screen, start... combining close ones? For 8-color mode, simple
       // interpolation. I have no idea what to do for 88 colors. FIXME
       if(ti->caps.colors >= 256){
-        return fbuf_emit(f, tiparm(setaf, rgb_quantize_256(r, g, b)));
+          TiparmValue argv[] = {tiparm_int(rgb_quantize_256(r, g, b))};
+        return fbuf_emit_parm(f, tiparm_s(setaf, 1, argv));
       }else if(ti->caps.colors >= 8){
-        return fbuf_emit(f, tiparm(setaf, rgb_quantize_8(r, g, b)));
+          TiparmValue argv[] = {tiparm_int(rgb_quantize_8(r, g, b))};
+        return fbuf_emit_parm(f, tiparm_s(setaf, 1, argv));
       }
     }
   }
@@ -779,7 +786,8 @@ update_palette(notcurses* nc, fbuf* f){
         r = r * 1000 / 255;
         g = g * 1000 / 255;
         b = b * 1000 / 255;
-        if(fbuf_emit(f, tiparm(initc, damageidx, r, g, b)) < 0){
+        TiparmValue argv[] = {tiparm_int(damageidx), tiparm_int(r), tiparm_int(g), tiparm_int(b),};
+        if(fbuf_emit_parm(f, tiparm_s(initc, 4, argv)) < 0){
           return -1;
         }
         nc->palette_damage[damageidx] = false;
@@ -1519,23 +1527,25 @@ int ncpile_rasterize(ncplane* n){
   clock_gettime(CLOCK_MONOTONIC, &start);
   ncpile* pile = ncplane_pile(n);
   struct notcurses* nc = ncpile_notcurses(pile);
-  const struct tinfo* ti = &ncplane_notcurses_const(n)->tcache;
-  postpaint(nc, ti, nc->lastframe, pile->dimy, pile->dimx, pile->crender, &nc->pool);
-  clock_gettime(CLOCK_MONOTONIC, &rasterdone);
-  int bytes = notcurses_rasterize(nc, pile, &nc->rstate.f);
-  clock_gettime(CLOCK_MONOTONIC, &writedone);
-  pthread_mutex_lock(&nc->stats.lock);
-    // accepts negative |bytes| as an indication of failure
-    update_raster_bytes(&nc->stats.s, bytes);
-    update_raster_stats(&rasterdone, &start, &nc->stats.s);
-    update_write_stats(&writedone, &rasterdone, &nc->stats.s, bytes);
-  pthread_mutex_unlock(&nc->stats.lock);
+  const struct tinfo* ti = &n->pile->nc->tcache;
+    int bytes = 0;
+  if (postpaint(nc, ti, nc->lastframe, pile->dimy, pile->dimx, pile->crender, &nc->pool) > 0) {
+      clock_gettime(CLOCK_MONOTONIC, &rasterdone);
+      bytes = notcurses_rasterize(nc, pile, &nc->rstate.f);
+      clock_gettime(CLOCK_MONOTONIC, &writedone);
+      pthread_mutex_lock(&nc->stats.lock);
+      // accepts negative |bytes| as an indication of failure
+      update_raster_bytes(&nc->stats.s, bytes);
+      update_raster_stats(&rasterdone, &start, &nc->stats.s);
+      update_write_stats(&writedone, &rasterdone, &nc->stats.s, bytes);
+      pthread_mutex_unlock(&nc->stats.lock);
+  }
   // we want to refresh if the screen geometry changed (or if we were just
   // woken up from SIGSTOP), but we mustn't do so until after rasterizing
   // the solved rvec, since this might result in a geometry update.
   if(sigcont_seen_for_render){
     sigcont_seen_for_render = 0;
-    notcurses_refresh(ncplane_notcurses(n), NULL, NULL);
+    notcurses_refresh(n->pile->nc, NULL, NULL);
   }
   if(bytes < 0){
     return -1;
@@ -1567,7 +1577,7 @@ engorge_crender_vector(ncpile* p){
 }
 
 int ncpile_render(ncplane* n){
-  scroll_lastframe(ncplane_notcurses(n), ncplane_pile(n)->scrolls);
+  scroll_lastframe(n->pile->nc, n->pile->scrolls);
   struct timespec start, renderdone;
   clock_gettime(CLOCK_MONOTONIC, &start);
   notcurses* nc = ncplane_notcurses(n);
