@@ -35,10 +35,12 @@
 
 #include "base/humanize.hh"
 #include "base/humanize.time.hh"
+#include "base/itertools.enumerate.hh"
 #include "base/itertools.hh"
 #include "base/keycodes.hh"
 #include "base/math_util.hh"
 #include "command_executor.hh"
+#include "crashd.client.hh"
 #include "intervaltree/IntervalTree.h"
 #include "lnav_util.hh"
 #include "md4cpp.hh"
@@ -204,7 +206,8 @@ timeline_header_overlay::list_static_overlay(const listview_curses& lv,
     struct tm lb_tm;
     auto ub = this->gho_src->gs_upper_bound;
     struct tm ub_tm;
-    auto bounds = this->gho_src->get_time_bounds_for(lv.get_selection());
+    auto bounds
+        = this->gho_src->get_time_bounds_for(lv.get_selection().value_or(0_vl));
 
     if (bounds.first < lb) {
         lb = bounds.first;
@@ -526,7 +529,7 @@ timeline_source::text_value_for_line(textview_curses& tc,
                                      std::string& value_out,
                                      text_sub_source::line_flags_t flags)
 {
-    if (line < this->gs_time_order.size()) {
+    if (line < (ssize_t) this->gs_time_order.size()) {
         const auto& row = this->gs_time_order[line].get();
         auto duration
             = row.or_value.otr_range.tr_end - row.or_value.otr_range.tr_begin;
@@ -567,13 +570,13 @@ timeline_source::text_attrs_for_line(textview_curses& tc,
                                      int line,
                                      string_attrs_t& value_out)
 {
-    if (line < this->gs_time_order.size()) {
+    if (line < (ssize_t) this->gs_time_order.size()) {
         const auto& row = this->gs_time_order[line].get();
 
         value_out = this->gs_rendered_line.get_attrs();
 
         auto lr = line_range{-1, -1, line_range::unit::codepoint};
-        auto sel_bounds = this->get_time_bounds_for(tc.get_selection());
+        auto sel_bounds = this->get_time_bounds_for(tc.get_selection().value_or(0_vl));
 
         if (row.or_value.otr_range.tr_begin <= sel_bounds.second
             && sel_bounds.first <= row.or_value.otr_range.tr_end)
@@ -629,7 +632,7 @@ timeline_source::text_size_for_line(textview_curses& tc,
     return this->gs_total_width;
 }
 
-void
+bool
 timeline_source::rebuild_indexes()
 {
     auto& bm = this->tss_view->get_bookmarks();
@@ -652,11 +655,11 @@ timeline_source::rebuild_indexes()
     this->gs_preview_rows.clear();
     this->gs_preview_status_source.get_description().clear();
 
-    auto min_log_time_opt = this->gs_lss.get_min_log_time();
-    auto max_log_time_opt = this->gs_lss.get_max_log_time();
+    auto min_log_time_opt = this->get_min_row_time();
+    auto max_log_time_opt = this->get_max_row_time();
     auto max_desc_width = size_t{0};
 
-    for (const auto& ld : this->gs_lss) {
+    for (const auto& [index, ld] : lnav::itertools::enumerate(this->gs_lss)) {
         if (ld->get_file_ptr() == nullptr) {
             continue;
         }
@@ -664,6 +667,7 @@ timeline_source::rebuild_indexes()
             continue;
         }
 
+        ld->get_file_ptr()->enable_cache();
         auto format = ld->get_file_ptr()->get_format();
         safe::ReadAccess<logfile::safe_opid_state> r_opid_map(
             ld->get_file_ptr()->get_opids());
@@ -737,6 +741,20 @@ timeline_source::rebuild_indexes()
             }
             active_iter->second.or_value.otr_description.lod_elements.clear();
         }
+
+        if (this->gs_index_progress) {
+            switch (this->gs_index_progress(
+                progress_t{index, this->gs_lss.file_count()}))
+            {
+                case lnav::progress_result_t::ok:
+                    break;
+                case lnav::progress_result_t::interrupt:
+                    return false;
+            }
+        }
+    }
+    if (this->gs_index_progress) {
+        this->gs_index_progress(std::nullopt);
     }
 
     size_t filtered_in_count = 0;
@@ -857,6 +875,8 @@ timeline_source::rebuild_indexes()
                            1 + 16 + 5 + 8 + 5 + 16 + 1 /* header */);
 
     this->tss_view->set_needs_update();
+
+    return true;
 }
 
 std::optional<vis_line_t>
@@ -927,8 +947,11 @@ timeline_source::time_for_row(vis_line_t row)
     }
 
     auto preview_selection = this->gs_preview_view.get_selection();
+    if (!preview_selection) {
+        return std::nullopt;
+    }
     if (preview_selection < this->gs_preview_rows.size()) {
-        return this->gs_preview_rows[preview_selection];
+        return this->gs_preview_rows[preview_selection.value()];
     }
 
     return row_info{
@@ -952,11 +975,11 @@ timeline_source::text_selection_changed(textview_curses& tc)
 
     this->gs_preview_source.clear();
     this->gs_preview_rows.clear();
-    if (sel >= this->gs_time_order.size()) {
+    if (!sel || sel.value() >= this->gs_time_order.size()) {
         return;
     }
 
-    const auto& row = this->gs_time_order[sel].get();
+    const auto& row = this->gs_time_order[sel.value()].get();
     auto low_tv = row.or_value.otr_range.tr_begin;
     auto high_tv = row.or_value.otr_range.tr_end;
     auto id_sf = row.or_name;
@@ -1081,7 +1104,6 @@ void
 timeline_source::text_crumbs_for_line(int line,
                                       std::vector<breadcrumb::crumb>& crumbs)
 {
-    static intern_string_t SRC = intern_string::lookup("crumb");
     text_sub_source::text_crumbs_for_line(line, crumbs);
 
     if (line >= this->gs_time_order.size()) {
@@ -1099,8 +1121,7 @@ timeline_source::text_crumbs_for_line(int line,
                             auto cmd
                                 = fmt::format(FMT_STRING(":goto {}"),
                                               ts.template get<std::string>());
-                            auto src_guard = ec->enter_source(SRC, 1, cmd);
-                            ec->execute(cmd);
+                            ec->execute(INTERNAL_SRC_LOC, cmd);
                         });
     crumbs.back().c_expected_input
         = breadcrumb::crumb::expected_input_t::anything;
