@@ -39,17 +39,20 @@
 #include "document.sections.hh"
 #include "environ_vtab.hh"
 #include "filter_sub_source.hh"
+#include "hasher.hh"
 #include "help-md.h"
 #include "intervaltree/IntervalTree.h"
 #include "lnav.hh"
 #include "lnav.indexing.hh"
+#include "lnav.prompt.hh"
 #include "md2attr_line.hh"
 #include "md4cpp.hh"
 #include "pretty_printer.hh"
-#include "shlex.hh"
 #include "sql_help.hh"
 #include "sql_util.hh"
 #include "static_file_vtab.hh"
+#include "textinput.history.hh"
+#include "textinput_curses.hh"
 #include "timeline_source.hh"
 #include "view_helpers.crumbs.hh"
 #include "view_helpers.examples.hh"
@@ -59,18 +62,16 @@
 using namespace std::chrono_literals;
 using namespace lnav::roles::literals;
 
-const char* const lnav_view_strings[LNV__MAX + 1] = {
-    "log",
-    "text",
-    "help",
-    "histogram",
-    "db",
-    "schema",
-    "pretty",
-    "spectro",
-    "timeline",
-
-    nullptr,
+constexpr std::array<string_fragment, LNV__MAX> lnav_view_strings = {
+    "log"_frag,
+    "text"_frag,
+    "help"_frag,
+    "histogram"_frag,
+    "db"_frag,
+    "schema"_frag,
+    "pretty"_frag,
+    "spectro"_frag,
+    "timeline"_frag,
 };
 
 const char* const lnav_view_titles[LNV__MAX] = {
@@ -113,18 +114,17 @@ view_from_string(const char* name)
         return std::nullopt;
     }
 
-    auto* view_name_iter
+    auto name_sf = string_fragment::from_c_str(name);
+    auto view_name_iter
         = std::find_if(std::begin(lnav_view_strings),
                        std::end(lnav_view_strings),
-                       [&](const char* v) {
-                           return v != nullptr && strcasecmp(v, name) == 0;
-                       });
+                       [&](const auto& v) { return name_sf.iequal(v); });
 
     if (view_name_iter == std::end(lnav_view_strings)) {
         return std::nullopt;
     }
 
-    return lnav_view_t(view_name_iter - lnav_view_strings);
+    return lnav_view_t(view_name_iter - std::begin(lnav_view_strings));
 }
 
 static void
@@ -153,16 +153,19 @@ open_schema_view()
     schema_tc->redo_search();
 }
 
-static void
+static bool
 open_timeline_view()
 {
     auto* timeline_tc = &lnav_data.ld_views[LNV_TIMELINE];
     auto* timeline_src
         = dynamic_cast<timeline_source*>(timeline_tc->get_sub_source());
 
-    timeline_src->rebuild_indexes();
+    if (!timeline_src->rebuild_indexes()) {
+        return false;
+    }
     timeline_tc->reload_data();
     timeline_tc->redo_search();
+    return true;
 }
 
 class pretty_sub_source : public plain_text_source {
@@ -379,26 +382,46 @@ open_pretty_view()
 
         for (auto vl = log_tc->get_top(); vl <= log_tc->get_bottom(); ++vl) {
             auto cl = lss.at(vl);
-            auto lf = lss.find(cl);
+            auto lf = lss.find_file_ptr(cl);
             auto ll = lf->begin() + cl;
-            shared_buffer_ref sbr;
 
             if (line_count > 0_vl && !ll->is_message()) {
                 continue;
             }
-            auto ll_start = lf->message_start(ll);
-            attr_line_t al;
-
             if (vl == log_tc->get_selection()) {
                 pretty_selected_line = line_count;
             }
+            auto flags = text_sub_source::RF_FULL | text_sub_source::RF_REWRITE;
+            if (line_count == 0_vl) {
+                auto ll_start = lf->message_start(ll);
 
-            vl -= vis_line_t(std::distance(ll_start, ll));
-            lss.text_value_for_line(
-                *log_tc,
-                vl,
-                al.get_string(),
-                text_sub_source::RF_FULL | text_sub_source::RF_REWRITE);
+                while (ll_start != ll) {
+                    if (vl <= 0_vl) {
+                        flags = 0;
+                        break;
+                    }
+                    vl -= 1_vl;
+                    auto prev_cl = lss.at(vl);
+                    auto prev_lf = lss.find_file_ptr(prev_cl);
+                    auto prev_ll = lf->begin() + prev_cl;
+                    if (prev_lf != lf) {
+                        flags = 0;
+                        break;
+                    }
+                    if (prev_ll->is_message()) {
+                        flags = 0;
+                        break;
+                    }
+                    if (prev_ll == ll_start) {
+                        flags = 0;
+                        break;
+                    }
+                }
+            } else if (ll->is_continued()) {
+                flags = 0;
+            }
+            attr_line_t al;
+            lss.text_value_for_line(*log_tc, vl, al.get_string(), flags);
             lss.text_attrs_for_line(*log_tc, vl, al.get_attrs());
             {
                 const auto orig_lr
@@ -429,6 +452,7 @@ open_pretty_view()
                 // TODO: dump more details of the line in the output.
                 pp.append_to(pretty_al);
             } else {
+                log_info("skipping pretty-print of log message with no body");
                 pretty_al = orig_al;
             }
 
@@ -552,11 +576,11 @@ open_pretty_view()
     pts->replace_with_mutable(full_text,
                               top_tc->get_sub_source()->get_text_format());
     pretty_tc->set_sub_source(pts);
-    if (lnav_data.ld_last_pretty_print_top != log_tc->get_top()) {
+    if (lnav_data.ld_last_pretty_print_top != top_tc->get_top()) {
         pretty_tc->set_top(0_vl);
     }
     pretty_tc->set_selection(pretty_selected_line.value_or(0_vl));
-    lnav_data.ld_last_pretty_print_top = log_tc->get_top();
+    lnav_data.ld_last_pretty_print_top = top_tc->get_top();
     pretty_tc->redo_search();
 }
 
@@ -568,15 +592,8 @@ build_all_help_text()
     }
 
     auto help_md_str = help_md.to_string_fragment_producer()->to_string();
-    shlex lexer(help_md_str);
-    std::string sub_help_text;
-
-    lexer.with_ignore_quotes(true).eval(
-        sub_help_text,
-        scoped_resolver{&lnav_data.ld_exec_context.ec_global_vars});
-
     md2attr_line mdal;
-    auto parse_res = md4cpp::parse(sub_help_text, mdal);
+    auto parse_res = md4cpp::parse(help_md_str, mdal);
     attr_line_t all_help_text = parse_res.unwrap();
 
     std::map<std::string, const help_text*> sql_funcs;
@@ -640,31 +657,30 @@ build_all_help_text()
 bool
 handle_winch(screen_curses* sc)
 {
-    static auto* filter_source = injector::get<filter_sub_source*>();
+    static auto* breadcrumb_view = injector::get<breadcrumb_curses*>();
+    static auto& prompt = lnav::prompt::get();
 
     if (!lnav_data.ld_winched) {
         return false;
     }
 
     if (sc) {
+        tcsetattr(STDIN_FILENO, TCSANOW, &sc->sc_termios);
         notcurses_refresh(sc->get_notcurses(), nullptr, nullptr);
         notcurses_render(sc->get_notcurses());
         notcurses_refresh(sc->get_notcurses(), nullptr, nullptr);
     }
 
     lnav_data.ld_winched = false;
-    if (lnav_data.ld_rl_view != nullptr) {
-        lnav_data.ld_rl_view->do_update();
-        lnav_data.ld_rl_view->window_change();
-    }
-    filter_source->fss_editor->window_change();
     for (auto& stat : lnav_data.ld_status) {
         stat.window_change();
+        stat.set_needs_update();
     }
+    breadcrumb_view->set_needs_update();
+    prompt.p_editor.set_needs_update();
     lnav_data.ld_view_stack.set_needs_update();
     lnav_data.ld_doc_view.set_needs_update();
     lnav_data.ld_example_view.set_needs_update();
-    lnav_data.ld_match_view.set_needs_update();
     lnav_data.ld_filter_view.set_needs_update();
     lnav_data.ld_files_view.set_needs_update();
     lnav_data.ld_file_details_view.set_needs_update();
@@ -678,10 +694,12 @@ handle_winch(screen_curses* sc)
 void
 layout_views()
 {
-    static constexpr auto FILES_FOCUSED_WIDTH = 40U;
+    static constexpr auto FILES_FOCUSED_WIDTH = 48U;
     static constexpr auto FILES_BLURRED_WIDTH = 20U;
 
     static auto* breadcrumb_view = injector::get<breadcrumb_curses*>();
+    static auto* filter_source = injector::get<filter_sub_source*>();
+    static auto& prompt = lnav::prompt::get();
 
     unsigned int width, height;
     ncplane_dim_yx(lnav_data.ld_window, &height, &width);
@@ -721,7 +739,7 @@ layout_views()
 
     int preview_height0 = lnav_data.ld_preview_hidden
         ? 0
-        : lnav_data.ld_preview_view[0].get_inner_height();
+        : std::min(10_vl, lnav_data.ld_preview_view[0].get_inner_height());
     if (!lnav_data.ld_preview_hidden
         && lnav_data.ld_preview_view[0].get_overlay_source() != nullptr)
     {
@@ -736,10 +754,6 @@ layout_views()
         preview_height1 = 6;  // XXX extra height for db overlay
     }
 
-    int match_rows = lnav_data.ld_match_source.text_line_count();
-    int match_height = std::min(match_rows, (int) (height - 4) / 2);
-    lnav_data.ld_match_view.set_height(vis_line_t(match_height));
-
     int um_rows = lnav_data.ld_user_message_source.text_line_count();
     if (um_rows > 0
         && std::chrono::steady_clock::now()
@@ -769,6 +783,8 @@ layout_views()
     switch (lnav_data.ld_mode) {
         case ln_mode_t::FILES:
         case ln_mode_t::FILTER:
+        case ln_mode_t::SEARCH_FILES:
+        case ln_mode_t::SEARCH_FILTERS:
             filter_height = 5;
             break;
         case ln_mode_t::FILE_DETAILS:
@@ -779,23 +795,25 @@ layout_views()
             break;
     }
 
+    if (files_width > width) {
+        files_width = width / 2;
+    }
+
     bool breadcrumb_open = (lnav_data.ld_mode == ln_mode_t::BREADCRUMBS);
 
-    auto bottom_min = std::min(2U + 3U, height);
-    auto bottom = clamped<int>::from(height, bottom_min, height);
+    auto prompt_height
+        = prompt.p_editor.is_enabled() ? prompt.p_editor.tc_height : 1;
+    auto min_height = std::min(1U + 10 + 2U, height);
+    auto bottom = clamped<int>::from(height, min_height, height);
 
-    lnav_data.ld_rl_view->set_y(height - 1);
-    bottom -= lnav_data.ld_rl_view->get_height();
-    lnav_data.ld_rl_view->set_width(width);
+    bottom -= prompt_height;
+    prompt.p_editor.set_y(bottom);
+    prompt.p_editor.set_width(width);
+    prompt.p_editor.ensure_cursor_visible();
 
     breadcrumb_view->set_width(width);
 
-    bool vis;
-    vis = bottom.try_consume(lnav_data.ld_match_view.get_height());
-    lnav_data.ld_match_view.set_y(bottom);
-    lnav_data.ld_match_view.set_visible(vis);
-
-    vis = bottom.try_consume(um_height);
+    auto vis = bottom.try_consume(um_height);
     lnav_data.ld_user_message_view.set_y(bottom);
     lnav_data.ld_user_message_view.set_visible(vis);
 
@@ -875,17 +893,26 @@ layout_views()
 
     vis = bottom.try_consume(filter_height + (config_panel_open ? 1 : 0)
                              + (filters_supported ? 1 : 0));
-    lnav_data.ld_filter_view.set_height(vis_line_t(filter_height));
+    if (!vis && lnav_data.ld_mode == ln_mode_t::FILE_DETAILS) {
+        filter_height = 5;
+        vis = bottom.try_consume(filter_height + (config_panel_open ? 1 : 0)
+                                 + (filters_supported ? 1 : 0));
+    }
+    lnav_data.ld_filter_view.set_height(vis_line_t(filter_height)
+                                        - (filters_supported ? 0_vl : 1_vl));
     lnav_data.ld_filter_view.set_y(bottom + 2);
     lnav_data.ld_filter_view.set_width(width);
     lnav_data.ld_filter_view.set_visible(filters_open && vis);
+    filter_source->fss_editor->set_width(width - 26);
 
-    lnav_data.ld_files_view.set_height(vis_line_t(filter_height));
+    lnav_data.ld_files_view.set_height(vis_line_t(filter_height)
+                                       - (filters_supported ? 0_vl : 1_vl));
     lnav_data.ld_files_view.set_y(bottom + 2);
     lnav_data.ld_files_view.set_width(files_width);
     lnav_data.ld_files_view.set_visible(files_open && vis);
 
-    lnav_data.ld_file_details_view.set_height(vis_line_t(filter_height));
+    lnav_data.ld_file_details_view.set_height(
+        vis_line_t(filter_height) - (filters_supported ? 0_vl : 1_vl));
     lnav_data.ld_file_details_view.set_y(bottom + 2);
     lnav_data.ld_file_details_view.set_x(files_width);
     lnav_data.ld_file_details_view.set_width(
@@ -922,20 +949,33 @@ layout_views()
 void
 update_hits(textview_curses* tc)
 {
+    static sig_atomic_t counter = 0;
+    static auto& timer = ui_periodic_timer::singleton();
+
 #if 0
     if (isendwin()) {
         return;
     }
 #endif
 
+    if (!timer.time_to_update(counter)
+        && lnav_data.ld_mode != ln_mode_t::SEARCH)
+    {
+        return;
+    }
+
     auto top_tc = lnav_data.ld_view_stack.top();
 
     if (top_tc && tc == *top_tc) {
-        lnav_data.ld_bottom_source.update_hits(tc);
+        if (lnav_data.ld_bottom_source.update_hits(tc)) {
+            lnav_data.ld_status[LNS_BOTTOM].set_needs_update();
+        }
 
         if (lnav_data.ld_mode == ln_mode_t::SEARCH) {
             constexpr auto MAX_MATCH_COUNT = 10_vl;
-            const auto PREVIEW_SIZE = MAX_MATCH_COUNT + 1_vl;
+            constexpr auto PREVIEW_SIZE = MAX_MATCH_COUNT + 1_vl;
+
+            static hasher::array_t last_preview_hash;
 
             int preview_count = 0;
             auto& bm = tc->get_bookmarks();
@@ -944,11 +984,8 @@ update_hits(textview_curses* tc)
             unsigned long width;
             vis_line_t height;
             attr_line_t all_matches;
-            char linebuf[64];
             int last_line = tc->get_inner_height();
-
-            snprintf(linebuf, sizeof(linebuf), "%d", last_line);
-            auto max_line_width = static_cast<int>(strlen(linebuf));
+            auto max_line_width = count_digits(last_line);
 
             tc->get_dimensions(height, width);
             vl += height;
@@ -969,12 +1006,11 @@ update_hits(textview_curses* tc)
                     attr_line_t al;
 
                     tc->textview_value_for_row(prev_vl.value(), al);
-                    snprintf(linebuf,
-                             sizeof(linebuf),
-                             "L%*d: ",
-                             max_line_width,
-                             (int) prev_vl.value());
-                    all_matches.append(linebuf).append(al);
+                    all_matches
+                        .appendf(FMT_STRING("L{:{}}"),
+                                 (int) prev_vl.value(),
+                                 max_line_width)
+                        .append(al);
                     preview_count += 1;
                 }
             }
@@ -997,25 +1033,29 @@ update_hits(textview_curses* tc)
                 if (preview_count > 0) {
                     all_matches.append("\n");
                 }
-                snprintf(linebuf,
-                         sizeof(linebuf),
-                         "L%*d: ",
-                         max_line_width,
-                         (int) vl);
-                all_matches.append(linebuf).append(al);
+                all_matches
+                    .appendf(FMT_STRING("L{:{}}"), (int) vl, max_line_width)
+                    .append(al);
                 preview_count += 1;
             }
 
-            if (preview_count > 0) {
+            auto match_hash = hasher().update(all_matches.al_string).to_array();
+            if (preview_count > 0
+                && (match_hash != last_preview_hash
+                    || lnav_data.ld_preview_view[0].get_sub_source()
+                        == nullptr))
+            {
+                log_debug("updating search preview");
                 lnav_data.ld_preview_status_source[0]
                     .get_description()
                     .set_value("Matching lines for search");
+                lnav_data.ld_status[LNS_PREVIEW0].set_needs_update();
                 lnav_data.ld_preview_view[0].set_sub_source(
                     &lnav_data.ld_preview_source[0]);
                 lnav_data.ld_preview_source[0]
                     .replace_with(all_matches)
                     .set_text_format(text_format_t::TF_UNKNOWN);
-                lnav_data.ld_preview_view[0].set_needs_update();
+                last_preview_hash = match_hash;
             }
         }
     }
@@ -1138,8 +1178,26 @@ eval_example(const help_text& ht, const help_example& ex)
 bool
 toggle_view(textview_curses* toggle_tc)
 {
-    textview_curses* tc = lnav_data.ld_view_stack.top().value_or(nullptr);
-    bool retval = false;
+    auto* tc = lnav_data.ld_view_stack.top().value_or(nullptr);
+    auto retval = false;
+
+    switch (lnav_data.ld_mode) {
+        case ln_mode_t::SQL:
+        case ln_mode_t::COMMAND:
+        case ln_mode_t::EXEC:
+        case ln_mode_t::USER:
+        case ln_mode_t::SEARCH:
+        case ln_mode_t::SEARCH_FILES:
+        case ln_mode_t::SEARCH_FILTERS:
+        case ln_mode_t::SEARCH_SPECTRO_DETAILS: {
+            log_debug("blocking toggle of view while in mode %s",
+                      lnav_mode_strings[lnav::enums::to_underlying(
+                          lnav_data.ld_mode)]);
+            return false;
+        }
+        default:
+            break;
+    }
 
     require(toggle_tc != nullptr);
     require(toggle_tc >= &lnav_data.ld_views[0]);
@@ -1151,6 +1209,8 @@ toggle_view(textview_curses* toggle_tc)
     lnav_data.ld_preview_status_source[0].get_description().clear();
     lnav_data.ld_preview_view[1].set_sub_source(nullptr);
     lnav_data.ld_preview_status_source[1].get_description().clear();
+    lnav_data.ld_status[LNS_PREVIEW0].set_needs_update();
+    lnav_data.ld_status[LNS_PREVIEW1].set_needs_update();
 
     if (tc == toggle_tc) {
         if (lnav_data.ld_view_stack.size() == 1) {
@@ -1177,16 +1237,19 @@ toggle_view(textview_curses* toggle_tc)
         } else if (toggle_tc == &lnav_data.ld_views[LNV_PRETTY]) {
             open_pretty_view();
         } else if (toggle_tc == &lnav_data.ld_views[LNV_TIMELINE]) {
-            open_timeline_view();
+            if (!open_timeline_view()) {
+                auto al = attr_line_t().append(
+                    "interrupted while opening timeline view"_warning);
+                lnav::prompt::get().p_editor.set_inactive_value(al);
+                return false;
+            }
         } else if (toggle_tc == &lnav_data.ld_views[LNV_HISTOGRAM]) {
             // Rebuild to reflect changes in marks.
             rebuild_hist();
         } else if (toggle_tc == &lnav_data.ld_views[LNV_HELP]) {
             build_all_help_text();
-            if (lnav_data.ld_rl_view != nullptr) {
-                lnav_data.ld_rl_view->set_alt_value(
-                    HELP_MSG_1(q, "to return to the previous view"));
-            }
+            lnav::prompt::get().p_editor.set_alt_value(
+                HELP_MSG_1(q, "to return to the previous view"));
         }
         lnav_data.ld_last_view = nullptr;
         lnav_data.ld_view_stack.push_back(toggle_tc);
@@ -1206,7 +1269,7 @@ bool
 ensure_view(textview_curses* expected_tc)
 {
     auto* tc = lnav_data.ld_view_stack.top().value_or(nullptr);
-    bool retval = true;
+    auto retval = true;
 
     if (tc != expected_tc) {
         toggle_view(expected_tc);
@@ -1233,7 +1296,7 @@ next_cluster(std::optional<vis_line_t> (bookmark_vector<vis_line_t>::*f)(
     auto* tc = get_textview_for_mode(lnav_data.ld_mode);
     auto& bm = tc->get_bookmarks();
     auto& bv = bm[bt];
-    bool top_is_marked = binary_search(bv.begin(), bv.end(), top);
+    bool top_is_marked = bv.bv_tree.exists(top);
     vis_line_t last_top(top), tc_height;
     std::optional<vis_line_t> new_top = top;
     unsigned long tc_width;
@@ -1280,11 +1343,15 @@ moveto_cluster(std::optional<vis_line_t> (bookmark_vector<vis_line_t>::*f)(
                const bookmark_type_t* bt,
                vis_line_t top)
 {
-    textview_curses* tc = get_textview_for_mode(lnav_data.ld_mode);
+    auto* tc = get_textview_for_mode(lnav_data.ld_mode);
+    auto sel = tc->get_selection();
+    if (!sel) {
+        return false;
+    }
     auto new_top = next_cluster(f, bt, top);
 
     if (!new_top) {
-        new_top = next_cluster(f, bt, tc->get_selection());
+        new_top = next_cluster(f, bt, sel.value());
     }
     if (new_top != -1) {
         tc->get_sub_source()->get_location_history() |
@@ -1306,7 +1373,7 @@ moveto_cluster(std::optional<vis_line_t> (bookmark_vector<vis_line_t>::*f)(
 vis_line_t
 search_forward_from(textview_curses* tc)
 {
-    vis_line_t height, retval = tc->get_selection();
+    vis_line_t height, retval = tc->get_selection().value_or(0_vl);
 
     if (!tc->is_selectable()) {
         auto& krh = lnav_data.ld_key_repeat_history;
@@ -1372,20 +1439,20 @@ hist_index_delegate::index_line(logfile_sub_source& lss,
         case LEVEL_FATAL:
         case LEVEL_CRITICAL:
         case LEVEL_ERROR:
-            ht = hist_source2::HT_ERROR;
+            ht = hist_source2::hist_type_t::HT_ERROR;
             break;
         case LEVEL_WARNING:
-            ht = hist_source2::HT_WARNING;
+            ht = hist_source2::hist_type_t::HT_WARNING;
             break;
         default:
-            ht = hist_source2::HT_NORMAL;
+            ht = hist_source2::hist_type_t::HT_NORMAL;
             break;
     }
 
     this->hid_source.add_value(ll->get_time<std::chrono::microseconds>(), ht);
     if (ll->is_marked() || ll->is_expr_marked()) {
         this->hid_source.add_value(ll->get_time<std::chrono::microseconds>(),
-                                   hist_source2::HT_MARK);
+                                   hist_source2::hist_type_t::HT_MARK);
     }
 }
 
@@ -1463,16 +1530,21 @@ lnav_crumb_source()
 
     auto* top_view = top_view_opt.value();
     auto view_index = top_view - lnav_data.ld_views;
+    auto view_str
+        = fmt::format(FMT_STRING(" {} \u25bc "), lnav_view_titles[view_index]);
     retval.emplace_back(
         lnav_view_titles[view_index],
-        attr_line_t().append(lnav::roles::status_title(
-            fmt::format(FMT_STRING(" {} "), lnav_view_titles[view_index]))),
+        attr_line_t().append(lnav::roles::status_title(view_str)),
         view_title_poss,
         view_performer);
 
     auto* tss = top_view->get_sub_source();
+    auto sel = top_view->get_selection();
+    if (!sel) {
+        return retval;
+    }
     if (tss != nullptr) {
-        tss->text_crumbs_for_line(top_view->get_selection(), retval);
+        tss->text_crumbs_for_line(sel.value(), retval);
     }
 
     return retval;
@@ -1508,17 +1580,53 @@ set_view_mode(ln_mode_t mode)
             lnav_data.ld_view_stack.set_needs_update();
             break;
         }
+        case ln_mode_t::SQL:
+        case ln_mode_t::EXEC:
+        case ln_mode_t::USER:
+        case ln_mode_t::BUSY:
+        case ln_mode_t::COMMAND:
+        case ln_mode_t::SEARCH:
+        case ln_mode_t::SEARCH_FILES:
+        case ln_mode_t::SEARCH_FILTERS:
+        case ln_mode_t::SEARCH_SPECTRO_DETAILS: {
+            if (mode != ln_mode_t::PAGING && mode != ln_mode_t::FILES
+                && mode != ln_mode_t::FILTER
+                && mode != ln_mode_t::SPECTRO_DETAILS)
+            {
+                log_debug("prompt is active, ignoring change to mode %s",
+                          lnav_mode_strings[lnav::enums::to_underlying(mode)]);
+                return;
+            }
+            break;
+        }
         case ln_mode_t::FILE_DETAILS: {
             lnav_data.ld_file_details_view.tc_cursor_role
                 = role_t::VCR_DISABLED_CURSOR_LINE;
             break;
         }
+        case ln_mode_t::FILTER: {
+            static auto* filter_source = injector::get<filter_sub_source*>();
+
+            if (filter_source->fss_editing) {
+                filter_source->fss_editor->abort();
+            }
+            break;
+        }
         default:
             break;
     }
+    breadcrumb_view->set_enabled(true);
     switch (mode) {
-        case ln_mode_t::SEARCH: {
+        case ln_mode_t::SQL:
+        case ln_mode_t::EXEC:
+        case ln_mode_t::USER:
+        case ln_mode_t::COMMAND:
+        case ln_mode_t::SEARCH:
+        case ln_mode_t::SEARCH_FILES:
+        case ln_mode_t::SEARCH_FILTERS:
+        case ln_mode_t::SEARCH_SPECTRO_DETAILS: {
             lnav_data.ld_status[LNS_DOC].set_needs_update();
+            breadcrumb_view->set_enabled(false);
             break;
         }
         case ln_mode_t::BREADCRUMBS: {
@@ -1529,10 +1637,32 @@ set_view_mode(ln_mode_t mode)
             lnav_data.ld_status[LNS_FILTER].set_needs_update();
             lnav_data.ld_file_details_view.tc_cursor_role
                 = role_t::VCR_CURSOR_LINE;
+            lnav_data.ld_view_stack.top().value()->set_enabled(false);
             break;
         }
-        default:
+        case ln_mode_t::FILES:
+        case ln_mode_t::FILTER:
+        case ln_mode_t::SPECTRO_DETAILS: {
+            if (!lnav_data.ld_filter_view.get_selection()) {
+                lnav_data.ld_filter_view.set_selection(0_vl);
+            }
+            lnav_data.ld_files_source.text_selection_changed(
+                lnav_data.ld_files_view);
+            breadcrumb_view->set_enabled(false);
+            lnav_data.ld_view_stack.top().value()->set_enabled(false);
             break;
+        }
+        case ln_mode_t::PAGING: {
+            lnav_data.ld_view_stack.top().value()->set_enabled(true);
+            break;
+        }
+        case ln_mode_t::BUSY: {
+            lnav_data.ld_view_stack.top().value()->set_enabled(false);
+            break;
+        }
+        case ln_mode_t::CAPTURE: {
+            break;
+        }
     }
     log_info("changing mode from %s to %s",
              lnav_mode_strings[lnav::enums::to_underlying(lnav_data.ld_mode)],
@@ -1540,7 +1670,7 @@ set_view_mode(ln_mode_t mode)
     lnav_data.ld_mode = mode;
 }
 
-static std::vector<view_curses*>
+std::vector<view_curses*>
 all_views()
 {
     static auto* breadcrumb_view = injector::get<breadcrumb_curses*>();
@@ -1548,20 +1678,20 @@ all_views()
     std::vector<view_curses*> retval;
 
     retval.push_back(breadcrumb_view);
-    for (auto& sc : lnav_data.ld_status) {
-        retval.push_back(&sc);
-    }
+    retval.push_back(&lnav::prompt::get().p_editor);
+    retval.push_back(&lnav_data.ld_filter_view);
     retval.push_back(&lnav_data.ld_doc_view);
     retval.push_back(&lnav_data.ld_example_view);
     retval.push_back(&lnav_data.ld_preview_view[0]);
     retval.push_back(&lnav_data.ld_preview_view[1]);
     retval.push_back(&lnav_data.ld_file_details_view);
     retval.push_back(&lnav_data.ld_files_view);
-    retval.push_back(&lnav_data.ld_filter_view);
     retval.push_back(&lnav_data.ld_user_message_view);
     retval.push_back(&lnav_data.ld_spectro_details_view);
     retval.push_back(&lnav_data.ld_timeline_details_view);
-    retval.push_back(lnav_data.ld_rl_view);
+    for (auto& sc : lnav_data.ld_status) {
+        retval.push_back(&sc);
+    }
 
     return retval;
 }
@@ -1614,7 +1744,7 @@ lnav_behavior::mouse_event(
     auto width = ncplane_dim_x(lnav_data.ld_window);
 
     me.me_x = x;
-    if (me.me_x >= width) {
+    if (me.me_x >= (ssize_t) width) {
         me.me_x = width - 1;
     }
     me.me_y = y - 1;
@@ -1626,6 +1756,7 @@ lnav_behavior::mouse_event(
         me.me_press_y = this->lb_last_event.me_press_y;
     }
 
+    this->lb_last_real_event = me;
     switch (me.me_state) {
         case mouse_button_state_t::BUTTON_STATE_PRESSED:
         case mouse_button_state_t::BUTTON_STATE_DOUBLE_CLICK: {
@@ -1633,38 +1764,41 @@ lnav_behavior::mouse_event(
                 if (breadcrumb_view->contains(me.me_x, me.me_y)) {
                     this->lb_last_view = breadcrumb_view;
                     break;
-                } else {
-                    set_view_mode(ln_mode_t::PAGING);
-                    lnav_data.ld_view_stack.set_needs_update();
                 }
+                set_view_mode(ln_mode_t::PAGING);
+                lnav_data.ld_view_stack.set_needs_update();
             }
 
-            auto* tc = *(lnav_data.ld_view_stack.top());
-            if (tc->contains(me.me_x, me.me_y)) {
-                me.me_press_y = me.me_y - tc->get_y();
-                me.me_press_x = me.me_x - tc->get_x();
-                this->lb_last_view = tc;
-
-                switch (lnav_data.ld_mode) {
-                    case ln_mode_t::PAGING:
-                        break;
-                    case ln_mode_t::FILES:
-                    case ln_mode_t::FILE_DETAILS:
-                    case ln_mode_t::FILTER:
-                        // Clicking on the main view when the config panels are
-                        // open should return us to paging.
-                        set_view_mode(ln_mode_t::PAGING);
-                        break;
-                    default:
-                        break;
+            this->lb_last_view = nullptr;
+            for (auto* vc : VIEWS) {
+                auto contained_by = vc->contains(me.me_x, me.me_y);
+                if (contained_by) {
+                    this->lb_last_view = contained_by.value();
+                    me.me_press_y = me.me_y - this->lb_last_view->get_y();
+                    me.me_press_x = me.me_x - this->lb_last_view->get_x();
+                    break;
                 }
-            } else {
-                for (auto* vc : VIEWS) {
-                    if (vc->contains(me.me_x, me.me_y)) {
-                        this->lb_last_view = vc;
-                        me.me_press_y = me.me_y - vc->get_y();
-                        me.me_press_x = me.me_x - vc->get_x();
-                        break;
+            }
+            if (this->lb_last_view == nullptr) {
+                auto* tc = *(lnav_data.ld_view_stack.top());
+                if (tc->contains(me.me_x, me.me_y)) {
+                    me.me_press_y = me.me_y - tc->get_y();
+                    me.me_press_x = me.me_x - tc->get_x();
+                    this->lb_last_view = tc;
+
+                    switch (lnav_data.ld_mode) {
+                        case ln_mode_t::PAGING:
+                            break;
+                        case ln_mode_t::FILES:
+                        case ln_mode_t::FILE_DETAILS:
+                        case ln_mode_t::FILTER:
+                        case ln_mode_t::SPECTRO_DETAILS:
+                            // Clicking on the main view when the config panels
+                            // are open should return us to paging.
+                            set_view_mode(ln_mode_t::PAGING);
+                            break;
+                        default:
+                            break;
                     }
                 }
             }
@@ -1693,3 +1827,20 @@ lnav_behavior::mouse_event(
         this->lb_last_view = nullptr;
     }
 }
+
+void
+lnav_behavior::tick(const timeval& now)
+{
+    if (this->lb_last_view == nullptr
+        || this->lb_last_event.me_state
+            != mouse_button_state_t::BUTTON_STATE_DRAGGED)
+    {
+        return;
+    }
+
+    this->lb_last_event = this->lb_last_real_event;
+    this->lb_last_event.me_y -= this->lb_last_view->get_y();
+    this->lb_last_event.me_x -= this->lb_last_view->get_x();
+    this->lb_last_event.me_time = now;
+    this->lb_last_view->handle_mouse(this->lb_last_event);
+}
