@@ -246,16 +246,16 @@ struct view_options {
 const typed_json_path_container<view_options>&
 get_view_options_handlers()
 {
-    static const json_path_handler_base::enum_value_t ROW_DETAILS_ENUM[] = {
-        {"hide", row_details_t::hide},
-        {"show", row_details_t::show},
+    static constexpr json_path_handler_base::enum_value_t ROW_DETAILS_ENUM[] = {
+        {"hide"_frag, row_details_t::hide},
+        {"show"_frag, row_details_t::show},
 
         json_path_handler_base::ENUM_TERMINATOR,
     };
 
-    static const json_path_handler_base::enum_value_t WORD_WRAP_ENUM[] = {
-        {"none", word_wrap_t::none},
-        {"normal", word_wrap_t::normal},
+    static constexpr json_path_handler_base::enum_value_t WORD_WRAP_ENUM[] = {
+        {"none"_frag, word_wrap_t::none},
+        {"normal"_frag, word_wrap_t::normal},
 
         json_path_handler_base::ENUM_TERMINATOR,
     };
@@ -328,10 +328,12 @@ CREATE TABLE lnav_views (
 
         tc.get_dimensions(height, width);
         switch (col) {
-            case 0:
+            case 0: {
+                const auto& vs = lnav_view_strings[view_index];
                 sqlite3_result_text(
-                    ctx, lnav_view_strings[view_index], -1, SQLITE_STATIC);
+                    ctx, vs.data(), vs.length(), SQLITE_STATIC);
                 break;
+            }
             case 1:
                 sqlite3_result_int(ctx, (int) tc.get_top());
                 break;
@@ -347,10 +349,11 @@ CREATE TABLE lnav_views (
             case 5: {
                 auto* time_source
                     = dynamic_cast<text_time_translator*>(tc.get_sub_source());
+                auto sel = tc.get_selection();
 
-                if (time_source != nullptr && tc.get_inner_height() > 0) {
-                    auto top_ri_opt
-                        = time_source->time_for_row(tc.get_selection());
+                if (time_source != nullptr && tc.get_inner_height() > 0 && sel)
+                {
+                    auto top_ri_opt = time_source->time_for_row(sel.value());
 
                     if (top_ri_opt) {
                         char timestamp[64];
@@ -416,14 +419,15 @@ CREATE TABLE lnav_views (
                     auto* time_source = dynamic_cast<text_time_translator*>(
                         tc.get_sub_source());
                     auto* ta = dynamic_cast<text_anchors*>(tc.get_sub_source());
+                    auto sel = tc.get_selection();
                     std::vector<breadcrumb::crumb> crumbs;
 
                     tss->text_crumbs_for_line(tc.get_top(), crumbs);
 
                     top_line_meta tlm;
-                    if (time_source != nullptr) {
+                    if (sel && time_source != nullptr) {
                         auto top_ri_opt
-                            = time_source->time_for_row(tc.get_selection());
+                            = time_source->time_for_row(sel.value());
 
                         if (top_ri_opt) {
                             char timestamp[64];
@@ -463,9 +467,15 @@ CREATE TABLE lnav_views (
                 }
                 break;
             }
-            case 12:
-                sqlite3_result_int(ctx, (int) tc.get_selection());
+            case 12: {
+                auto sel = tc.get_selection();
+                if (sel) {
+                    sqlite3_result_int(ctx, (int) sel.value());
+                } else {
+                    sqlite3_result_null(ctx);
+                }
                 break;
+            }
             case 13: {
                 if (sqlite3_vtab_nochange(ctx)) {
                     return SQLITE_OK;
@@ -601,15 +611,15 @@ CREATE TABLE lnav_views (
                 selection = top_row;
             }
         } else if (top_time != nullptr && time_source != nullptr) {
+            auto sel = tc.get_selection();
             date_time_scanner dts;
-            struct timeval tv;
+            timeval tv;
 
             log_debug("setting top time for %s to %s",
                       tc.get_title().c_str(),
                       top_time);
-            if (dts.convert_to_timeval(top_time, -1, nullptr, tv)) {
-                auto last_ri_opt
-                    = time_source->time_for_row(tc.get_selection());
+            if (sel && dts.convert_to_timeval(top_time, -1, nullptr, tv)) {
+                auto last_ri_opt = time_source->time_for_row(sel.value());
 
                 if (last_ri_opt) {
                     auto last_time = last_ri_opt->ri_time;
@@ -783,10 +793,11 @@ CREATE TABLE lnav_view_stack (
         auto view = lnav_view_t(tc - lnav_data.ld_views);
 
         switch (col) {
-            case 0:
-                sqlite3_result_text(
-                    ctx, lnav_view_strings[view], -1, SQLITE_STATIC);
+            case 0: {
+                const auto& vs = lnav_view_strings[view];
+                sqlite3_result_text(ctx, vs.data(), vs.length(), SQLITE_STATIC);
                 break;
+            }
         }
 
         return SQLITE_OK;
@@ -923,18 +934,17 @@ CREATE TABLE lnav_view_filters (
 
     int get_column(cursor& vc, sqlite3_context* ctx, int col)
     {
-        textview_curses& tc = lnav_data.ld_views[vc.iter.i_view_index];
-        text_sub_source* tss = tc.get_sub_source();
-        filter_stack& fs = tss->get_filters();
+        auto& tc = lnav_data.ld_views[vc.iter.i_view_index];
+        auto* tss = tc.get_sub_source();
+        auto& fs = tss->get_filters();
         auto tf = *(fs.begin() + vc.iter.i_filter_index);
 
         switch (col) {
-            case 0:
-                sqlite3_result_text(ctx,
-                                    lnav_view_strings[vc.iter.i_view_index],
-                                    -1,
-                                    SQLITE_STATIC);
+            case 0: {
+                const auto& vs = lnav_view_strings[vc.iter.i_view_index];
+                sqlite3_result_text(ctx, vs.data(), vs.length(), SQLITE_STATIC);
                 break;
+            }
             case 1:
                 to_sqlite(ctx, tf->get_index());
                 break;
@@ -1127,7 +1137,7 @@ CREATE TABLE lnav_view_filters (
     {
         auto* mod_vt = (vtab_module<lnav_view_filters>::vtab*) tab;
         auto view_index = lnav_view_t(rowid >> 32);
-        auto filter_index = rowid & 0xffffffffLL;
+        auto filter_index = rowid & 0xffffffffULL;
         auto& tc = lnav_data.ld_views[view_index];
         auto* tss = tc.get_sub_source();
         auto& fs = tss->get_filters();
@@ -1249,12 +1259,11 @@ CREATE TABLE lnav_view_filter_stats (
         auto tf = *(fs.begin() + vc.iter.i_filter_index);
 
         switch (col) {
-            case 0:
-                sqlite3_result_text(ctx,
-                                    lnav_view_strings[vc.iter.i_view_index],
-                                    -1,
-                                    SQLITE_STATIC);
+            case 0: {
+                const auto& vs = lnav_view_strings[vc.iter.i_view_index];
+                sqlite3_result_text(ctx, vs.data(), vs.length(), SQLITE_STATIC);
                 break;
+            }
             case 1:
                 to_sqlite(ctx, tf->get_index());
                 break;
@@ -1308,10 +1317,11 @@ CREATE TABLE lnav_view_files (
         auto& ld = *vc.iter;
 
         switch (col) {
-            case 0:
-                sqlite3_result_text(
-                    ctx, lnav_view_strings[LNV_LOG], -1, SQLITE_STATIC);
+            case 0: {
+                const auto& vs = lnav_view_strings[LNV_LOG];
+                sqlite3_result_text(ctx, vs.data(), vs.length(), SQLITE_STATIC);
                 break;
+            }
             case 1:
                 to_sqlite(ctx,
                           ld->ld_filter_state.lfo_filter_state.tfs_logfile
