@@ -125,6 +125,9 @@ void yajl_string_decode(yajl_buf buf, const unsigned char * str,
         if (str[end] == '\x1b') {
             props->has_ansi = 1;
             end++;
+        } else if (str[end] == '/' || str[end] == '#' || str[end] == '~') {
+            props->ptr_escapes += 1;
+            end++;
         } else if (str[end] == '\\') {
             char utf8Buf[5];
             const char * unescaped = "?";
