@@ -35,14 +35,12 @@
 
 #include "yajlpp.hh"
 
+#include "base/math_util.hh"
 #include "config.h"
 #include "fmt/format.h"
 #include "yajl/api/yajl_parse.h"
 #include "yajlpp_def.hh"
 
-const json_path_handler_base::enum_value_t
-    json_path_handler_base::ENUM_TERMINATOR((const char*) nullptr, 0);
-
 yajl_gen_status
 yajl_gen_tree(yajl_gen hand, yajl_val val)
 {
@@ -211,7 +209,8 @@ json_path_handler_base::gen(yajlpp_gen_context& ygc, yajl_gen handle) const
 
     if (this->jph_children) {
         for (const auto& lpath : local_paths) {
-            std::string full_path = json_ptr::encode_str(lpath);
+            stack_buf allocator;
+            auto full_path = json_ptr::encode(lpath, allocator);
             int start_depth = ygc.ygc_depth;
 
             yajl_gen_string(handle, lpath);
@@ -427,7 +426,7 @@ json_path_handler_base::gen_schema_type(yajlpp_gen_context& ygc) const
                     schema("enum");
 
                     yajlpp_array enum_array(ygc.ygc_handle);
-                    for (int lpc = 0; this->jph_enum_values[lpc].first; lpc++) {
+                    for (int lpc = 0; !this->jph_enum_values[lpc].first.empty(); lpc++) {
                         enum_array.gen(this->jph_enum_values[lpc].first);
                     }
                 }
@@ -460,6 +459,7 @@ json_path_handler_base::walk(
         this->jph_path_provider(root, local_paths);
 
         for (const auto& lpath : local_paths) {
+            stack_buf allocator;
             const void* field = nullptr;
             if (this->jph_field_getter) {
                 field = this->jph_field_getter(root, lpath);
@@ -467,7 +467,7 @@ json_path_handler_base::walk(
             cb(*this,
                fmt::format(FMT_STRING("{}{}{}"),
                            base,
-                           json_ptr::encode_str(lpath),
+                           json_ptr::encode(lpath, allocator),
                            this->jph_children ? "/" : ""),
                field);
         }
@@ -502,7 +502,11 @@ json_path_handler_base::walk(
                 static const intern_string_t POSS_SRC
                     = intern_string::lookup("possibilities");
 
-                std::string full_path = base + json_ptr::encode_str(lpath);
+                stack_buf allocator;
+                auto full_path
+                    = fmt::format(FMT_STRING("{}{}"),
+                                  base,
+                                  json_ptr::encode(lpath, allocator));
                 if (this->jph_children) {
                     full_path += "/";
                 }
@@ -518,7 +522,9 @@ json_path_handler_base::walk(
                     thread_local auto md
                         = lnav::pcre2pp::match_data::unitialized();
 
-                    const auto short_path = json_ptr::encode_str(lpath) + "/";
+                    stack_buf allocator2;
+                    const auto short_path = fmt::format(
+                        FMT_STRING("{}/"), json_ptr::encode(lpath, allocator2));
 
                     if (!this->jph_regex->capture_from(short_path)
                              .into(md)
@@ -554,7 +560,7 @@ json_path_handler_base::walk(
 std::optional<int>
 json_path_handler_base::to_enum_value(const string_fragment& sf) const
 {
-    for (int lpc = 0; this->jph_enum_values[lpc].first; lpc++) {
+    for (int lpc = 0; !this->jph_enum_values[lpc].first.empty(); lpc++) {
         const auto& ev = this->jph_enum_values[lpc];
 
         if (sf == ev.first) {
@@ -565,10 +571,10 @@ json_path_handler_base::to_enum_value(const string_fragment& sf) const
     return std::nullopt;
 }
 
-const char*
+string_fragment
 json_path_handler_base::to_enum_string(int value) const
 {
-    for (int lpc = 0; this->jph_enum_values[lpc].first; lpc++) {
+    for (int lpc = 0; !this->jph_enum_values[lpc].first.empty(); lpc++) {
         const auto& ev = this->jph_enum_values[lpc];
 
         if (ev.second == value) {
@@ -576,7 +582,7 @@ json_path_handler_base::to_enum_string(int value) const
         }
     }
 
-    return "";
+    return {};
 }
 
 std::vector<json_path_handler_base::schema_type_t>
@@ -607,9 +613,9 @@ json_path_handler_base::get_types() const
 
 yajlpp_parse_context::yajlpp_parse_context(
     intern_string_t source, const struct json_path_container* handlers)
-    : ypc_source(source), ypc_handlers(handlers)
+    : ypc_source(source), ypc_handlers(handlers),
+      ypc_path(auto_buffer::alloc(4096))
 {
-    this->ypc_path.reserve(4096);
     this->ypc_path.push_back('/');
     this->ypc_path.push_back('\0');
     this->ypc_callbacks = DEFAULT_CALLBACKS;
@@ -640,9 +646,12 @@ yajlpp_parse_context::map_start(void* ctx)
 }
 
 int
-yajlpp_parse_context::map_key(void* ctx, const unsigned char* key, size_t len)
+yajlpp_parse_context::map_key(void* ctx,
+                              const unsigned char* key,
+                              size_t len,
+                              yajl_string_props_t* props)
 {
-    yajlpp_parse_context* ypc = (yajlpp_parse_context*) ctx;
+    auto* ypc = (yajlpp_parse_context*) ctx;
     int retval = 1;
 
     require(ypc->ypc_path.size() >= 2);
@@ -651,29 +660,37 @@ yajlpp_parse_context::map_key(void* ctx, const unsigned char* key, size_t len)
     if (ypc->ypc_path.back() != '/') {
         ypc->ypc_path.push_back('/');
     }
-    for (size_t lpc = 0; lpc < len; lpc++) {
-        switch (key[lpc]) {
-            case '~':
-                ypc->ypc_path.push_back('~');
-                ypc->ypc_path.push_back('0');
-                break;
-            case '/':
-                ypc->ypc_path.push_back('~');
-                ypc->ypc_path.push_back('1');
-                break;
-            case '#':
-                ypc->ypc_path.push_back('~');
-                ypc->ypc_path.push_back('2');
-                break;
-            default:
-                ypc->ypc_path.push_back(key[lpc]);
-                break;
+    auto path_start = ypc->ypc_path.size();
+    ypc->ypc_path.expand_to(
+        roundup_size(ypc->ypc_path.size() + len + props->ptr_escapes, 4096));
+    ypc->ypc_path.resize_by(len + props->ptr_escapes);
+    if (props->ptr_escapes > 0) {
+        for (size_t lpc = 0; lpc < len; lpc++) {
+            switch (key[lpc]) {
+                case '~':
+                    ypc->ypc_path[path_start++] = '~';
+                    ypc->ypc_path[path_start++] = '0';
+                    break;
+                case '/':
+                    ypc->ypc_path[path_start++] = '~';
+                    ypc->ypc_path[path_start++] = '1';
+                    break;
+                case '#':
+                    ypc->ypc_path[path_start++] = '~';
+                    ypc->ypc_path[path_start++] = '2';
+                    break;
+                default:
+                    ypc->ypc_path[path_start++] = key[lpc];
+                    break;
+            }
         }
+    } else {
+        memcpy(&ypc->ypc_path[path_start], key, len);
     }
     ypc->ypc_path.push_back('\0');
 
     if (ypc->ypc_alt_callbacks.yajl_map_key != nullptr) {
-        retval = ypc->ypc_alt_callbacks.yajl_map_key(ctx, key, len);
+        retval = ypc->ypc_alt_callbacks.yajl_map_key(ctx, key, len, props);
     }
 
     if (ypc->ypc_handlers != nullptr) {
@@ -708,12 +725,15 @@ yajlpp_parse_context::update_callbacks(const json_path_container* orig_handlers,
 
     if (!this->ypc_active_paths.empty()) {
         std::string curr_path(&this->ypc_path[0], this->ypc_path.size() - 1);
+        auto path_iter = this->ypc_active_paths.find(curr_path);
 
-        if (this->ypc_active_paths.find(curr_path)
-            == this->ypc_active_paths.end())
-        {
+        if (path_iter == this->ypc_active_paths.end()) {
             return;
         }
+        path_iter->second += 1;
+        log_trace("%s: found active path: %s",
+                  this->ypc_source.c_str(),
+                  curr_path.c_str());
     }
 
     if (child_start == 0 && !this->ypc_obj_stack.empty()) {
@@ -864,7 +884,7 @@ yajlpp_parse_context::array_end(void* ctx)
 int
 yajlpp_parse_context::handle_unused(void* ctx)
 {
-    yajlpp_parse_context* ypc = (yajlpp_parse_context*) ctx;
+    auto* ypc = (yajlpp_parse_context*) ctx;
 
     if (ypc->ypc_ignore_unused) {
         return 1;
@@ -931,8 +951,9 @@ yajlpp_parse_context::handle_unused(void* ctx)
 
         attr_line_t help_text;
 
-        if (accepted_handlers->jpc_children.size() == 1
-            && accepted_handlers->jpc_children.front().jph_is_array)
+        if (accepted_handlers == nullptr) {
+        } else if (accepted_handlers->jpc_children.size() == 1
+                   && accepted_handlers->jpc_children.front().jph_is_array)
         {
             const auto& jph = accepted_handlers->jpc_children.front();
 
@@ -965,7 +986,7 @@ yajlpp_parse_context::handle_unused(void* ctx)
 int
 yajlpp_parse_context::handle_unused_or_delete(void* ctx)
 {
-    yajlpp_parse_context* ypc = (yajlpp_parse_context*) ctx;
+    auto* ypc = (yajlpp_parse_context*) ctx;
 
     if (!ypc->ypc_handler_stack.empty()
         && ypc->ypc_handler_stack.back()->jph_obj_deleter)
@@ -990,18 +1011,18 @@ yajlpp_parse_context::handle_unused_or_delete(void* ctx)
 }
 
 const yajl_callbacks yajlpp_parse_context::DEFAULT_CALLBACKS = {
-    yajlpp_parse_context::handle_unused_or_delete,
-    (int (*)(void*, int)) yajlpp_parse_context::handle_unused,
-    (int (*)(void*, long long)) yajlpp_parse_context::handle_unused,
-    (int (*)(void*, double)) yajlpp_parse_context::handle_unused,
+    handle_unused_or_delete,
+    (int (*)(void*, int)) handle_unused,
+    (int (*)(void*, long long)) handle_unused,
+    (int (*)(void*, double)) handle_unused,
     nullptr,
     (int (*)(void*, const unsigned char*, size_t, yajl_string_props_t*))
-        yajlpp_parse_context::handle_unused,
-    yajlpp_parse_context::map_start,
-    yajlpp_parse_context::map_key,
-    yajlpp_parse_context::map_end,
-    yajlpp_parse_context::array_start,
-    yajlpp_parse_context::array_end,
+        handle_unused,
+    map_start,
+    map_key,
+    map_end,
+    array_start,
+    array_end,
 };
 
 yajl_status
@@ -1215,12 +1236,10 @@ yajlpp_parse_context::set_path(const std::string& path)
     return *this;
 }
 
-const char*
-yajlpp_parse_context::get_path_fragment(int offset,
-                                        char* frag_in,
-                                        size_t& len_out) const
+string_fragment
+yajlpp_parse_context::get_path_fragment(int offset, stack_buf& allocator) const
 {
-    const char* retval;
+    string_fragment retval;
     size_t start, end;
 
     if (offset < 0) {
@@ -1232,13 +1251,9 @@ yajlpp_parse_context::get_path_fragment(int offset,
     } else {
         end = this->ypc_path.size() - 1;
     }
+    retval = string_fragment::from_bytes(&this->ypc_path[start], end - start);
     if (this->ypc_handlers != nullptr) {
-        len_out
-            = json_ptr::decode(frag_in, &this->ypc_path[start], end - start);
-        retval = frag_in;
-    } else {
-        retval = &this->ypc_path[start];
-        len_out = end - start;
+        retval = json_ptr::decode(retval, allocator);
     }
 
     return retval;
@@ -1436,7 +1451,7 @@ json_path_handler_base::validate_string(yajlpp_parse_context& ypc,
                              .with_reason("empty values are not allowed")
                              .with_snippet(ypc.get_snippet())
                              .with_help(this->get_help_text(&ypc)));
-    } else if (sf.length() < this->jph_min_length) {
+    } else if (sf.length() < (ssize_t) this->jph_min_length) {
         ypc.report_error(
             lnav::console::user_message::error(
                 attr_line_t()
@@ -1516,7 +1531,7 @@ json_path_handler_base::get_help_text(const std::string& full_path) const
     if (this->jph_enum_values != nullptr) {
         retval.append(lnav::roles::h2("Allowed Values")).append("\n  ");
 
-        for (int lpc = 0; this->jph_enum_values[lpc].first; lpc++) {
+        for (int lpc = 0; !this->jph_enum_values[lpc].first.empty(); lpc++) {
             const auto& ev = this->jph_enum_values[lpc];
 
             retval.append(lpc == 0 ? "" : ", ")
