@@ -27,10 +27,6 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifdef __CYGWIN__
-#    include <alloca.h>
-#endif
-
 #include <fstream>
 #include <iostream>
 
@@ -39,6 +35,7 @@
 
 #include "base/injector.bind.hh"
 #include "base/injector.hh"
+#include "base/isc.hh"
 #include "config.h"
 #include "data_parser.hh"
 #include "data_scanner.hh"
@@ -136,8 +133,8 @@ main(int argc, char* argv[])
                         "error: unable to temporary file for writing\n");
                 retval = EXIT_FAILURE;
             } else {
+                isc::supervisor root_superv(injector::get<isc::service_list>());
                 std::shared_ptr<log_format> format;
-                char* log_line;
                 bool found = false;
                 char cmd[2048];
                 std::string line;
@@ -148,8 +145,6 @@ main(int argc, char* argv[])
                     line = "             " + line;
                 }
 
-                log_line = (char*) alloca(line.length());
-                strcpy(log_line, &line[13]);
                 auto sub_line = line.substr(13);
                 struct line_range body(0, sub_line.length());
                 shared_buffer share_manager;
