@@ -28,6 +28,11 @@ run_cap_test ${lnav_test} -nN \
     -c ":cd ${test_dir}" \
     -c ":open logfile_access_log.0"
 
+run_cap_test ${lnav_test} -nN \
+    -c ":cd ${top_srcdir}" \
+    -c ";SELECT * FROM environ WHERE name = 'PWD'" \
+    -c ":write-json-to -"
+
 run_cap_test ${lnav_test} -nN \
     -e "echo Hello, World!"
 
@@ -50,6 +55,13 @@ run_cap_test ${lnav_test} -n \
     -c ":goto 0" \
     ${test_dir}/logfile_access_log.0
 
+run_cap_test ${lnav_test} -n \
+    -c ":goto 1" \
+    -c ":mark" \
+    -c ":hide-unmarked-lines" \
+    -c ":goto 0" \
+    ${test_dir}/logfile_multiline.0
+
 run_cap_test ${lnav_test} -n \
     -c ":unix-time" \
     "${test_dir}/logfile_access_log.*"
@@ -234,7 +246,7 @@ run_cap_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 run_cap_test ${lnav_test} -n \
-    -c ':goto 17:00:01.' \
+    -c ':goto 17:00:01.N' \
     ${test_dir}/logfile_access_log.0
 
 run_cap_test ${lnav_test} -n \
@@ -349,13 +361,13 @@ run_cap_test ${lnav_test} -n \
     -c ":filter-out World" \
     ${test_dir}/logfile_plain.0
 
-TOO_MANY_FILTERS=""
-for i in `seq 1 32`; do
-    TOO_MANY_FILTERS="$TOO_MANY_FILTERS -c ':filter-out $i'"
-done
-run_cap_test eval ${lnav_test} -d /tmp/lnav.err -n \
-    $TOO_MANY_FILTERS \
-    ${test_dir}/logfile_filter.0
+#TOO_MANY_FILTERS=""
+#for i in `seq 1 32`; do
+#    TOO_MANY_FILTERS="$TOO_MANY_FILTERS -c ':filter-out $i'"
+#done
+#run_cap_test ${lnav_test} -d /tmp/lnav.err -n \
+#    $TOO_MANY_FILTERS \
+#    ${test_dir}/logfile_filter.0
 
 run_cap_test ${lnav_test} -n \
     -c ":close" \
@@ -523,6 +535,11 @@ run_cap_test ${lnav_test} -n \
     -c ":switch-to-view histogram" \
     ${test_dir}/logfile_syslog.0
 
+run_cap_test ${lnav_test} -n \
+    -c ":mark" \
+    -c "/vmw" \
+    ${test_dir}/logfile_access_log.0
+
 run_cap_test ${lnav_test} -n \
     -c ":zoom-to bad" \
     ${test_dir}/logfile_access_log.0
@@ -534,6 +551,9 @@ run_cap_test ${lnav_test} -n \
 printf "Hello, World!" | run_cap_test env TEST_COMMENT="text view" ${lnav_test} -n \
   -c ":switch-to-view text"
 
+run_cap_test ${lnav_test} -Nnv \
+    -c ":hide-lines-before badbadbad"
+
 run_cap_test ${lnav_test} -Nnv \
     -c ":hide-lines-before 2009-07-20T22:59:29" \
     -c ":hide-lines-before"
