@@ -8,6 +8,7 @@ rm -rf "${CONFIG_DIR}/.lnav/formats"
 HOME=${CONFIG_DIR}
 unset XDG_CONFIG_HOME
 export HOME
+export YES_COLOR=1
 
 ${lnav_test} -i ${srcdir}/formats/jsontest/format.json
 
@@ -16,6 +17,19 @@ if ! test -f ${CONFIG_DIR}/.lnav/formats/installed/test_log.json; then
     exit 1
 fi
 
+run_cap_test ${lnav_test} -i ${srcdir}/formats/jsontest/format.json
+
+echo corrupt > ${CONFIG_DIR}/.lnav/formats/installed/test_log.json
+
+run_cap_test env TEST_COMMENT='overwrite file' ${lnav_test} -i ${srcdir}/formats/jsontest/format.json
+
+if ! test -f ${CONFIG_DIR}/.lnav/formats/installed/test_log.json.bak; then
+    echo "Format not backed up correctly?"
+    exit 1
+fi
+
+run_cap_test ${lnav_test} -i /non-existent/file
+
 if test x"${TEST_GIT_INSTALL}" = x""; then
     # Hitting the git repos frequently is slow/noisy
     exit 0
