@@ -13,6 +13,12 @@ printf '000\n000\n#Fields: 0\n0\n#Fields: 0\n0' | run_cap_test \
 printf '#Date:\t3/9/3/0\x85 2\n0\n' | run_cap_test \
     env TEST_COMMENT="invalid w3c timestamp" ${lnav_test} -n
 
+printf '[8.0000]0\n' | run_cap_test \
+    env TEST_COMMENT="zero timestamp" ${lnav_test} -n
+
+printf '#Fields: date time 0 date\n2000/2 00:00 0 2000/80' | run_cap_test \
+    env TEST_COMMENT="invalid w3c #1451" ${lnav_test} -n
+
 cat > rollover_in.0 <<EOF
 2600/2 0 00:00:00 0:
 00:2 0 00:00:00 0:
@@ -62,6 +68,9 @@ EOF
 EOF
 
   run_cap_test ${lnav_test} -n ${test_dir}/dhcp-trunc.pcapng
+
+  gunzip -c ${test_dir}/capture.btsnoop.gz > capture.btsnoop
+  run_cap_test ${lnav_test} -n -c ':goto 790' capture.btsnoop
 fi
 
 
@@ -223,6 +232,9 @@ EOF
   error: unable to open file: /test-logs.tgz
  reason: unable to create directory: rotmp/lnav-user-NNN-work/archives -- Permission denied
 EOF
+
+    run_cap_test env TMPDIR=logfile-tmp ${lnav_test} -n \
+        ${srcdir}/abs-archive.tar.gz
 fi
 
 touch unreadable.log
@@ -490,9 +502,9 @@ run_test ./drive_logfile -v -f tcsh_history ${srcdir}/logfile_tcsh_history.0
 
 check_output "TCSH level interpreted incorrectly?" <<EOF
 info 0x0
-info 0x80
 info 0x0
-info 0x80
+info 0x0
+info 0x0
 EOF
 
 run_test ./drive_logfile -v -f access_log ${srcdir}/logfile_access_log.0
@@ -603,6 +615,16 @@ info 0x0
 error 0x0
 EOF
 
+run_cap_test ./drive_logfile -t -f logfmt_log ${srcdir}/logfile_grafana.0
+
+run_cap_test ./drive_logfile -t -f postgres_log ${srcdir}/logfile_postgres.0
+
+run_cap_test ./drive_logfile -t -f mysql_slow_log ${srcdir}/logfile_mysql_slow.0
+
+run_cap_test ./drive_logfile -t -f mysql_error_log ${srcdir}/logfile_mysql_error.0
+
+run_cap_test ./drive_logfile -t -f mysql_gen_log ${srcdir}/logfile_mysql_gen.0
+
 run_test ${lnav_test} -C ${test_dir}/logfile_bad_access_log.0
 
 sed -ibak -e "s|/.*/logfile_bad_access_log.0|logfile_bad_access_log.0|g" `test_err_filename`
@@ -747,3 +769,12 @@ touch -t 202411030000 ${test_dir}/logfile_dst.0
 run_cap_test env TZ=America/Los_Angeles ${lnav_test} -n \
     -c ':set-file-timezone America/Los_Angeles' \
     ${test_dir}/logfile_dst.0
+
+cat ${test_dir}/logfile_generic.0 | run_cap_test ${lnav_test} -n \
+    -c ':test-comment generic before piper'
+
+run_cap_test ${lnav_test} -n ${test_dir}/logfile_logfmt.0
+
+run_cap_test ${lnav_test} -n ${test_dir}/logfile_laravel.0
+
+run_cap_test ${lnav_test} -n ${test_dir}/logfile_laravel.1
