@@ -6,6 +6,23 @@ unset XDG_CONFIG_HOME
 rm -rf "./sessions"
 mkdir -p $HOME
 
+cat ${test_dir}/logfile_access_log.0 | run_cap_test ${lnav_test} -n \
+    -c ":filter-out vmk" \
+    -c ":export-session-to exported-stdin-session.0.lnav"
+
+# run_cap_test cat exported-stdin-session.0.lnav
+
+run_cap_test ${lnav_test} -nN \
+    -c "|exported-stdin-session.0.lnav"
+
+run_cap_test ${lnav_test} -n \
+    -e "cat ${test_dir}/logfile_access_log.0" \
+    -c ":filter-out vmk" \
+    -c ":export-session-to exported-sh-session.0.lnav"
+
+run_cap_test ${lnav_test} -nN \
+    -c "|exported-sh-session.0.lnav"
+
 run_cap_test ${lnav_test} -n \
     -c ":reset-session" \
     -c ":goto 0" \
@@ -42,6 +59,7 @@ run_cap_test ${lnav_test} -nq \
 run_cap_test ${lnav_test} -nq \
     -c ";update access_log set log_mark = 1 where sc_bytes > 60000" \
     -c ":hide-fields cs_user_agent" \
+    -c ":highlight vmw" \
     -c ":set-min-log-level debug" \
     -c ":hide-lines-before 2005" \
     -c ":hide-lines-after 2030" \
@@ -99,6 +117,11 @@ run_cap_test ${lnav_test} -n \
     -c ":test-comment adjust time in session" \
     ${test_dir}/logfile_access_log.0
 
+run_cap_test ${lnav_test} -n \
+    -c ":load-session" \
+    -c ":clear-adjusted-log-time" \
+    ${test_dir}/logfile_access_log.0
+
 run_cap_test ${lnav_test} -n \
     -c ":load-session" \
     -c ":reset-session" \
