@@ -27,3 +27,7 @@ run_cap_test ${lnav_test} -n \
     -c ':create-search-table asl_mod ASL Module "(?<name>[^"]+)"' \
     -c ';SELECT * FROM asl_mod' \
     ${test_dir}/logfile_syslog.3
+
+run_cap_test ${lnav_test} -n \
+    -c ';SELECT * FROM mysql_slow_stats' \
+    ${test_dir}/logfile_mysql_slow.0
